@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,11 +26,10 @@
 #if defined(HAVE_LIB_PTHREAD)
 static void *stress_resources_pthread_func(void *ctxt)
 {
-        static void *nowt;
-
         (void)ctxt;
         (void)sleep(1);
-        return &nowt;
+
+        return &g_nowt;
 }
 #endif
 
@@ -44,9 +43,18 @@ static void stress_resources_init(stress_resources_t *resources, const size_t nu
 
 	for (i = 0; i < num_resources; i++) {
 		resources[i].m_malloc = NULL;
+		resources[i].m_malloc_size = 0;
+		resources[i].m_sbrk = NULL;
+		resources[i].m_sbrk_size = 0;
 		resources[i].m_mmap = MAP_FAILED;
+		resources[i].m_mmap_size = 0;
+		resources[i].fd_pipe[0] = -1;
+		resources[i].fd_pipe[1] = -1;
 		resources[i].pipe_ret = -1;
 		resources[i].fd_open = -1;
+		resources[i].fd_sock = -1;
+		resources[i].fd_socketpair[0] = -1;
+		resources[i].fd_socketpair[1] = -1;
 #if defined(HAVE_EVENTFD)
 		resources[i].fd_ev = -1;
 #endif
@@ -58,9 +66,6 @@ static void stress_resources_init(stress_resources_t *resources, const size_t nu
 		resources[i].fd_memfd_secret = -1;
 		resources[i].ptr_memfd_secret = NULL;
 #endif
-		resources[i].fd_sock = -1;
-		resources[i].fd_socketpair[0] = -1;
-		resources[i].fd_socketpair[1] = -1;
 #if defined(HAVE_USERFAULTFD)
 		resources[i].fd_uf = -1;
 #endif
@@ -70,14 +75,22 @@ static void stress_resources_init(stress_resources_t *resources, const size_t nu
 #if defined(HAVE_LIB_PTHREAD)
 		resources[i].pthread_ret = -1;
 		resources[i].pthread = (pthread_t)0;
-#if defined(HAVE_PTHREAD_MUTEX_T) &&	\
+#endif
+#if defined(HAVE_LIB_PTHREAD) &&	\
+    defined(HAVE_PTHREAD_MUTEX_T) &&	\
     defined(HAVE_PTHREAD_MUTEX_INIT) &&	\
     defined(HAVE_PTHREAD_MUTEX_DESTROY)
 		(void)shim_memset(&resources[i].mutex, 0, sizeof(resources[i].mutex));
 		resources[i].mutex_ret = -1;
 #endif
+#if defined(HAVE_LIB_PTHREAD) &&	\
+    defined(HAVE_THREADS_H) &&		\
+    defined(HAVE_MTX_T) &&		\
+    defined(HAVE_MTX_DESTROY) &&	\
+    defined(HAVE_MTX_INIT)
+		(void)shim_memset(&resources[i].mtx, 0, sizeof(resources[i].mtx));
+		resources[i].mtx_ret = -1;
 #endif
-
 #if defined(HAVE_LIB_RT) &&		\
     defined(HAVE_TIMER_CREATE) &&	\
     defined(HAVE_TIMER_DELETE) &&	\
@@ -123,6 +136,9 @@ static void stress_resources_init(stress_resources_t *resources, const size_t nu
 #endif
 #if defined(HAVE_PIDFD_OPEN)
 		resources[i].pid_fd = -1;
+#if defined(HAVE_PIDFD_GETFD)
+		resources[i].pid_fd_getfd = -1;
+#endif
 #endif
 		resources[i].pid = 0;
 	}
@@ -148,6 +164,9 @@ size_t stress_resources_allocate(
 	size_t i, n = 0;
 	size_t shmall, freemem, totalmem, freeswap, totalswap;
 	const pid_t pid = getpid();
+#if defined(HAVE_PIDFD_OPEN)
+	const pid_t ppid = getppid();
+#endif
 	const size_t page_size = args->page_size;
 	static const int domains[] = { AF_INET, AF_INET6 };
 	static const int types[] = { SOCK_STREAM, SOCK_DGRAM };
@@ -190,23 +209,28 @@ size_t stress_resources_allocate(
 		char name[64];
 #endif
 		resources[i].m_malloc = NULL;
+		resources[i].m_malloc_size = 0;
 		resources[i].m_mmap = MAP_FAILED;
+		resources[i].m_mmap_size = 0;
 		resources[i].pipe_ret = -1;
 		resources[i].fd_open = -1;
+		resources[i].fd_sock = -1;
+		resources[i].fd_socketpair[0] = -1;
+		resources[i].fd_socketpair[1] = -1;
+		resources[i].pid = 0;
 #if defined(HAVE_EVENTFD)
 		resources[i].fd_ev = -1;
 #endif
 #if defined(HAVE_MEMFD_CREATE)
 		resources[i].fd_memfd = -1;
 		resources[i].ptr_memfd = NULL;
+		resources[i].ptr_memfd_size = 0;
 #endif
 #if defined(__NR_memfd_secret)
 		resources[i].fd_memfd_secret = -1;
 		resources[i].ptr_memfd_secret = NULL;
+		resources[i].ptr_memfd_secret_size = 0;
 #endif
-		resources[i].fd_sock = -1;
-		resources[i].fd_socketpair[0] = -1;
-		resources[i].fd_socketpair[1] = -1;
 #if defined(HAVE_USERFAULTFD)
 		resources[i].fd_uf = -1;
 #endif
@@ -216,14 +240,30 @@ size_t stress_resources_allocate(
 #if defined(HAVE_LIB_PTHREAD)
 		resources[i].pthread_ret = -1;
 		resources[i].pthread = (pthread_t)0;
-#if defined(HAVE_PTHREAD_MUTEX_T) &&	\
+#endif
+#if defined(HAVE_LIB_PTHREAD) &&	\
+    defined(HAVE_PTHREAD_MUTEX_T) &&	\
     defined(HAVE_PTHREAD_MUTEX_INIT) &&	\
     defined(HAVE_PTHREAD_MUTEX_DESTROY)
 		(void)shim_memset(&resources[i].mutex, 0, sizeof(resources[i].mutex));
 		resources[i].mutex_ret = -1;
 #endif
+#if defined(HAVE_LIB_PTHREAD) &&	\
+    defined(HAVE_THREADS_H) &&		\
+    defined(HAVE_MTX_T) &&		\
+    defined(HAVE_MTX_DESTROY) &&	\
+    defined(HAVE_MTX_INIT)
+		(void)shim_memset(&resources[i].mtx, 0, sizeof(resources[i].mtx));
+		resources[i].mtx_ret = -1;
+#endif
+#if defined(HAVE_SYS_INOTIFY)
+		resources[i].wd_inotify = -1;
+		resources[i].fd_inotify = -1;
+#endif
+#if defined(HAVE_PTSNAME)
+		resources[i].pty = -1;
+		resources[i].pty_mtx = -1;
 #endif
-
 #if defined(HAVE_LIB_RT) &&		\
     defined(HAVE_TIMER_CREATE) &&	\
     defined(HAVE_TIMER_DELETE) &&	\
@@ -237,17 +277,10 @@ size_t stress_resources_allocate(
     defined(CLOCK_REALTIME)
 		resources[i].timer_fd = -1;
 #endif
-#if defined(HAVE_SYS_INOTIFY)
-		resources[i].wd_inotify = -1;
-		resources[i].fd_inotify = -1;
-#endif
-#if defined(HAVE_PTSNAME)
-		resources[i].pty = -1;
-		resources[i].pty_mtx = -1;
-#endif
 #if defined(HAVE_LIB_PTHREAD) &&	\
     defined(HAVE_SEM_POSIX)
 		resources[i].semok = false;
+		(void)shim_memset(&resources[i].sem, 0, sizeof(resources[i].sem));
 #endif
 #if defined(HAVE_SEM_SYSV)
 		resources[i].sem_id = -1;
@@ -269,41 +302,45 @@ size_t stress_resources_allocate(
 #endif
 #if defined(HAVE_PIDFD_OPEN)
 		resources[i].pid_fd = -1;
+#if defined(HAVE_PIDFD_GETFD)
+		resources[i].pid_fd_getfd = -1;
+#endif
 #endif
-		resources[i].pid = 0;
-
 		/*
 		 *  Ensure we tidy half complete resources since n is off by one
 		 *  if we break out of the loop to early
 		 */
 		n = i + 1;
 
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 
 		stress_get_memlimits(&shmall, &freemem, &totalmem, &freeswap, &totalswap);
-		if ((freemem > 0) && (freemem < min_mem_free))
+		if (UNLIKELY((freemem > 0) && (freemem < min_mem_free)))
 			break;
 
-		if ((stress_mwc8() & 0xf) == 0) {
-			resources[i].m_malloc = calloc(1, page_size);
+		if (UNLIKELY((stress_mwc8() & 0xf) == 0)) {
+			resources[i].m_malloc = (void *)calloc(1, page_size);
+			resources[i].m_malloc_size = page_size;
 			if (!stress_continue_flag())
 				break;
 		}
-		if ((stress_mwc8() & 0xf) == 0) {
+		if (UNLIKELY((stress_mwc8() & 0xf) == 0)) {
 			resources[i].m_sbrk = shim_sbrk((intptr_t)page_size);
+			resources[i].m_sbrk_size = page_size;
 			if (!stress_continue_flag())
 				break;
 		}
-		if ((stress_mwc8() & 0xf) == 0) {
+		if (UNLIKELY((stress_mwc8() & 0xf) == 0)) {
 			resources[i].m_mmap_size = page_size * 2;
 			resources[i].m_mmap = mmap(NULL, resources[i].m_mmap_size,
 				PROT_READ | PROT_WRITE, MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
-			if (!stress_continue_flag())
+			if (UNLIKELY(!stress_continue_flag()))
 				break;
 			if (resources[i].m_mmap != MAP_FAILED) {
 				const size_t locked = STRESS_MINIMUM(mlock_size, resources[i].m_mmap_size);
 
+				stress_set_vma_anon_name(resources[i].m_mmap, resources[i].m_mmap_size, "resources-mmap");
 				(void)stress_madvise_random(resources[i].m_mmap, resources[i].m_mmap_size);
 				(void)stress_mincore_touch_pages_interruptible(resources[i].m_mmap, resources[i].m_mmap_size);
 				if (locked > 0) {
@@ -325,14 +362,14 @@ size_t stress_resources_allocate(
 		UNEXPECTED
 		(void)pipe_size;
 #endif
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 		resources[i].fd_open = open("/dev/null", O_RDONLY);
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 #if defined(HAVE_EVENTFD)
 		resources[i].fd_ev = eventfd(0, 0);
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 #else
 		UNEXPECTED
@@ -356,12 +393,15 @@ size_t stress_resources_allocate(
 					resources[i].fd_memfd, 0);
 				if (resources[i].ptr_memfd == MAP_FAILED)
 					resources[i].ptr_memfd = NULL;
-				else
+				else {
+					resources[i].ptr_memfd_size = page_size;
+					stress_set_vma_anon_name(resources[i].ptr_memfd, page_size, "resources-memfd");
 					(void)stress_madvise_mergeable(resources[i].ptr_memfd, page_size);
+				}
 			}
 			shim_fallocate(resources[i].fd_memfd, 0, 0, (off_t)stress_mwc16());
 		}
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 #else
 		UNEXPECTED
@@ -376,17 +416,20 @@ size_t stress_resources_allocate(
 					resources[i].fd_memfd_secret, 0);
 				if (resources[i].ptr_memfd_secret == MAP_FAILED)
 					resources[i].ptr_memfd_secret = NULL;
-				else
+				else {
+					resources[i].ptr_memfd_secret_size = page_size;
+					stress_set_vma_anon_name(resources[i].ptr_memfd_secret, page_size, "resources-memfd-secret");
 					(void)stress_madvise_mergeable(resources[i].ptr_memfd_secret, page_size);
+				}
 			}
 		}
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 #endif
 		resources[i].fd_sock = socket(
 			domains[stress_mwc32modn((uint32_t)SIZEOF_ARRAY(domains))],
 			types[stress_mwc32modn((uint32_t)SIZEOF_ARRAY(types))], 0);
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 
 		if (socketpair(AF_UNIX, SOCK_STREAM, 0,
@@ -397,7 +440,7 @@ size_t stress_resources_allocate(
 
 #if defined(HAVE_USERFAULTFD)
 		resources[i].fd_uf = shim_userfaultfd(0);
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 #else
 		UNEXPECTED
@@ -405,7 +448,7 @@ size_t stress_resources_allocate(
 #if defined(O_TMPFILE)
 		resources[i].fd_tmp = open("/tmp", O_TMPFILE | O_RDWR,
 				      S_IRUSR | S_IWUSR);
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 		if (resources[i].fd_tmp != -1) {
 			(void)shim_fallocate(resources[i].fd_tmp, 0, 0, (off_t)page_size);
@@ -476,7 +519,7 @@ size_t stress_resources_allocate(
 			resources[i].fd_inotify = -1;
 			resources[i].wd_inotify = -1;
 		}
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 #endif
 #if defined(HAVE_PTSNAME)
@@ -488,7 +531,7 @@ size_t stress_resources_allocate(
 			if (ptyname)
 				resources[i].pty = open(ptyname, O_RDWR);
 		}
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 #endif
 
@@ -497,16 +540,27 @@ size_t stress_resources_allocate(
 			resources[i].pthread_ret =
 				pthread_create(&resources[i].pthread, NULL,
 					stress_resources_pthread_func, NULL);
-#if defined(HAVE_PTHREAD_MUTEX_T) &&	\
-    defined(HAVE_PTHREAD_MUTEX_INIT) &&	\
-    defined(HAVE_PTHREAD_MUTEX_DESTROY)
-			resources[i].mutex_ret = pthread_mutex_init(&resources[i].mutex, NULL);
-#endif
 			if (!stress_continue_flag())
 				break;
 		}
 #endif
-
+#if defined(HAVE_LIB_PTHREAD) &&	\
+    defined(HAVE_PTHREAD_MUTEX_T) &&	\
+    defined(HAVE_PTHREAD_MUTEX_INIT) &&	\
+    defined(HAVE_PTHREAD_MUTEX_DESTROY)
+		resources[i].mutex_ret = pthread_mutex_init(&resources[i].mutex, NULL);
+		if (UNLIKELY(!stress_continue_flag()))
+			break;
+#endif
+#if defined(HAVE_LIB_PTHREAD) &&	\
+    defined(HAVE_THREADS_H) &&		\
+    defined(HAVE_MTX_T) &&		\
+    defined(HAVE_MTX_DESTROY) &&	\
+    defined(HAVE_MTX_INIT)
+		resources[i].mtx_ret = mtx_init(&resources[i].mtx, mtx_plain);
+		if (UNLIKELY(!stress_continue_flag()))
+			break;
+#endif
 #if defined(HAVE_LIB_RT) &&		\
     defined(HAVE_TIMER_CREATE) &&	\
     defined(HAVE_TIMER_DELETE) &&	\
@@ -531,7 +585,7 @@ size_t stress_resources_allocate(
     defined(CLOCK_REALTIME)
 		if (!i) {
 			resources[i].timer_fd = timerfd_create(CLOCK_REALTIME, 0);
-			if (!stress_continue_flag())
+			if (UNLIKELY(!stress_continue_flag()))
 				break;
 		}
 #endif
@@ -539,7 +593,7 @@ size_t stress_resources_allocate(
 #if defined(HAVE_LIB_PTHREAD) &&	\
     defined(HAVE_SEM_POSIX)
 		resources[i].semok = (sem_init(&resources[i].sem, 1, 1) >= 0);
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 #endif
 
@@ -551,7 +605,7 @@ size_t stress_resources_allocate(
 
 			resources[i].sem_id = semget(sem_key, 1,
 				IPC_CREAT | S_IRUSR | S_IWUSR);
-			if (!stress_continue_flag())
+			if (UNLIKELY(!stress_continue_flag()))
 				break;
 		}
 #endif
@@ -561,7 +615,7 @@ size_t stress_resources_allocate(
     defined(HAVE_SYS_MSG_H)
 		resources[i].msgq_id = msgget(IPC_PRIVATE,
 				S_IRUSR | S_IWUSR | IPC_CREAT | IPC_EXCL);
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 #endif
 
@@ -581,7 +635,7 @@ size_t stress_resources_allocate(
 
 			resources[i].mq = mq_open(resources[i].mq_name,
 				O_CREAT | O_RDWR, S_IRUSR | S_IWUSR, &attr);
-			if (!stress_continue_flag())
+			if (UNLIKELY(!stress_continue_flag()))
 				break;
 		}
 #endif
@@ -589,13 +643,16 @@ size_t stress_resources_allocate(
     defined(HAVE_PKEY_FREE)
 		resources[i].pkey = shim_pkey_alloc(0, 0);
 #endif
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 
 #if defined(HAVE_PIDFD_OPEN)
-		resources[i].pid_fd = shim_pidfd_open(pid, 0);
+		resources[i].pid_fd = shim_pidfd_open(ppid, 0);
+#if defined(HAVE_PIDFD_GETFD)
+		/* get parent pid stdout */
+		resources[i].pid_fd_getfd = shim_pidfd_getfd(resources[i].pid_fd, 1, 0);
+#endif
 #endif
-
 		if (do_fork) {
 			resources[i].pid = fork();
 			if (resources[i].pid == 0) {
@@ -631,7 +688,7 @@ size_t stress_resources_allocate(
  *	resource frees
  */
 void stress_resources_free(
-	stress_args_t *args,
+	const stress_args_t *args,
 	stress_resources_t *resources,
         const size_t num_resources)
 {
@@ -705,27 +762,35 @@ void stress_resources_free(
 			resources[i].fd_uf = -1;
 		}
 #endif
-
 #if defined(O_TMPFILE)
 		if (resources[i].fd_tmp != -1) {
 			(void)close(resources[i].fd_tmp);
 			resources[i].fd_tmp = -1;
 		}
 #endif
-
 #if defined(HAVE_LIB_PTHREAD)
 		if ((!i) && (!resources[i].pthread_ret) && (resources[i].pthread)) {
 			(void)pthread_join(resources[i].pthread, NULL);
 			resources[i].pthread = (pthread_t)0;
-#if defined(HAVE_PTHREAD_MUTEX_T) &&	\
-    defined(HAVE_PTHREAD_MUTEX_INIT) &&	\
-    defined(HAVE_PTHREAD_MUTEX_DESTROY)
-			(void)pthread_mutex_destroy(&resources[i].mutex);
-			(void)shim_memset(&resources[i].mutex, 0, sizeof(resources[i].mutex));
-#endif
 		}
 #endif
-
+#if defined(HAVE_LIB_PTHREAD) && 	\
+    defined(HAVE_PTHREAD_MUTEX_T) &&	\
+    defined(HAVE_PTHREAD_MUTEX_INIT) &&	\
+    defined(HAVE_PTHREAD_MUTEX_DESTROY)
+		(void)pthread_mutex_destroy(&resources[i].mutex);
+		(void)shim_memset(&resources[i].mutex, 0, sizeof(resources[i].mutex));
+#endif
+#if defined(HAVE_LIB_PTHREAD) &&	\
+    defined(HAVE_THREADS_H) &&		\
+    defined(HAVE_MTX_T) &&		\
+    defined(HAVE_MTX_DESTROY) &&	\
+    defined(HAVE_MTX_INIT)
+		if (resources[i].mtx_ret == 0) {
+			mtx_destroy(&resources[i].mtx);
+			(void)shim_memset(&resources[i].mtx, 0, sizeof(resources[i].mtx));
+		}
+#endif
 #if defined(HAVE_LIB_RT) &&		\
     defined(HAVE_TIMER_CREATE) &&	\
     defined(HAVE_TIMER_DELETE) &&	\
@@ -817,6 +882,12 @@ void stress_resources_free(
 		if (resources[i].pid_fd > -1) {
 			(void)close(resources[i].pid_fd);
 			resources[i].pid_fd = -1;
+#if defined(HAVE_PIDFD_GETFD)
+		if (resources[i].pid_fd_getfd > -1) {
+			(void)close(resources[i].pid_fd_getfd);
+			resources[i].pid_fd_getfd = -1;
+		}
+#endif
 		}
 #endif
 		if (resources[i].pid > 0) {
@@ -826,3 +897,110 @@ void stress_resources_free(
 	}
 }
 
+/*
+ *  stress_resources_access()
+ *	access a wide range of resources
+ */
+void stress_resources_access(
+	const stress_args_t *args,
+	stress_resources_t *resources,
+        const size_t num_resources)
+{
+	const size_t page_size = args->page_size;
+	size_t i;
+
+	(void)page_size;
+
+	for (i = 0; i < num_resources; i++) {
+		if (resources[i].m_malloc)
+			(void)shim_memset(resources[i].m_malloc, (int)i, resources[i].m_malloc_size);
+		if (resources[i].m_mmap && (resources[i].m_mmap != MAP_FAILED))
+			(void)shim_memset(resources[i].m_mmap, (int)i, resources[i].m_mmap_size);
+#if defined(F_GETFL)
+		if (resources[i].pipe_ret != -1) {
+			VOID_RET(int, fcntl(resources[i].fd_pipe[0], F_GETFL, 0));
+			VOID_RET(int, fcntl(resources[i].fd_pipe[1], F_GETFL, 0));
+		}
+#endif
+#if defined(F_GETFL)
+		if (resources[i].fd_open != -1)
+			VOID_RET(int, fcntl(resources[i].fd_open, F_GETFL, 0));
+#endif
+
+#if defined(HAVE_EVENTFD) && 	\
+    defined(F_GETFL)
+		if (resources[i].fd_ev != -1)
+			VOID_RET(int, fcntl(resources[i].fd_ev, F_GETFL, 0));
+#endif
+#if defined(HAVE_MEMFD_CREATE)
+#if defined(F_GETFL)
+		if (resources[i].fd_memfd != -1)
+			VOID_RET(int, fcntl(resources[i].fd_memfd, F_GETFL, 0));
+#endif
+		if (resources[i].ptr_memfd && (resources[i].ptr_memfd != MAP_FAILED))
+			(void)shim_memset(resources[i].ptr_memfd, (int)i, resources[i].ptr_memfd_size);
+#endif
+#if defined(__NR_memfd_secret)
+#if defined(F_GETFL)
+		if (resources[i].fd_memfd_secret != -1)
+			VOID_RET(int, fcntl(resources[i].fd_memfd_secret, F_GETFL, 0));
+#endif
+		if (resources[i].ptr_memfd_secret && (resources[i].ptr_memfd_secret != MAP_FAILED))
+			(void)shim_memset(resources[i].ptr_memfd_secret, (int)i, resources[i].ptr_memfd_secret_size);
+#endif
+#if defined(F_GETFL)
+		if (resources[i].fd_sock != -1)
+			 VOID_RET(int, fcntl(resources[i].fd_sock, F_GETFL, 0));
+#endif
+#if defined(F_GETFL)
+		if (resources[i].fd_socketpair[0] != -1)
+			 VOID_RET(int, fcntl(resources[i].fd_socketpair[0], F_GETFL, 0));
+		if (resources[i].fd_socketpair[1] != -1)
+			 VOID_RET(int, fcntl(resources[i].fd_socketpair[1], F_GETFL, 0));
+#endif
+#if defined(HAVE_USERFAULTFD) &&	\
+    defined(F_GETFL)
+		if (resources[i].fd_uf != -1)
+			 VOID_RET(int, fcntl(resources[i].fd_uf, F_GETFL, 0));
+#endif
+#if defined(O_TMPFILE) &&		\
+    defined(F_GETFL)
+		if (resources[i].fd_tmp != -1)
+			 VOID_RET(int, fcntl(resources[i].fd_tmp, F_GETFL, 0));
+#endif
+#if defined(HAVE_SYS_TIMERFD_H) &&	\
+    defined(HAVE_TIMERFD_CREATE) &&	\
+    defined(HAVE_TIMERFD_GETTIME) &&	\
+    defined(HAVE_TIMERFD_SETTIME) &&	\
+    defined(CLOCK_REALTIME) &&		\
+    defined(F_GETFL)
+		if ((!i) && (resources[i].timer_fd != -1))
+			 VOID_RET(int, fcntl(resources[i].timer_fd, F_GETFL, 0));
+#endif
+#if defined(HAVE_SYS_INOTIFY) &&	\
+    defined(F_GETFL)
+		if (resources[i].wd_inotify != -1)
+			VOID_RET(int, fcntl(resources[i].wd_inotify, F_GETFL, 0));
+		if (resources[i].fd_inotify != -1)
+			VOID_RET(int, fcntl(resources[i].fd_inotify, F_GETFL, 0));
+#endif
+#if defined(HAVE_PTSNAME) &&		\
+    defined(F_GETFL)
+		if (resources[i].pty != -1)
+			VOID_RET(int, fcntl(resources[i].pty, F_GETFL, 0));
+		if (resources[i].pty_mtx != -1)
+			VOID_RET(int, fcntl(resources[i].pty_mtx, F_GETFL, 0));
+#endif
+#if defined(HAVE_PIDFD_OPEN) &&		\
+    defined(F_GETFL)
+		if (resources[i].pid_fd > -1)
+			VOID_RET(int, fcntl(resources[i].pid_fd, F_GETFL, 0));
+#if defined(HAVE_PIDFD_GETFD)
+		if (resources[i].pid_fd_getfd > -1)
+			VOID_RET(int, fcntl(resources[i].pid_fd_getfd, F_GETFL, 0));
+#endif
+#endif
+		if (resources[i].pid > 0)
+			(void)kill(resources[i].pid, 0);
+	}
+}
