@@ -17,8 +17,14 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
 #include "core-builtin.h"
-#include "core-cache.h"
+#include "core-cpu-cache.h"
+#include "core-killpid.h"
+#include "core-lock.h"
+#include "core-pragma.h"
+
+#include <sched.h>
 
 #define DEFAULT_L1_SIZE		(64)
 
@@ -31,11 +37,11 @@
 #define EXERCISE(data)		\
 do {				\
 	(data)++;		\
-	shim_mb();		\
+	stress_asm_mb();	\
 	data = shim_rol8(data);	\
-	shim_mb();		\
+	stress_asm_mb();	\
 	data = shim_ror8(data);	\
-	shim_mb();		\
+	stress_asm_mb();	\
 } while (0)
 
 static const stress_help_t help[] = {
@@ -47,8 +53,8 @@ static const stress_help_t help[] = {
 };
 
 typedef int (*stress_cacheline_func)(
-        const stress_args_t *args,
-        const int index,
+        stress_args_t *args,
+        const int idx,
         const bool parent,
         const size_t l1_cacheline_size);
 
@@ -57,15 +63,15 @@ typedef struct {
 	const stress_cacheline_func	func;
 } stress_cacheline_method_t;
 
-static uint64_t get_L1_line_size(const stress_args_t *args)
+static uint64_t get_L1_line_size(stress_args_t *args)
 {
 	uint64_t cache_size = DEFAULT_L1_SIZE;
 #if defined(__linux__) ||	\
     defined(__APPLE__)
-	stress_cpus_t *cpu_caches;
+	stress_cpu_cache_cpus_t *cpu_caches;
 	stress_cpu_cache_t *cache = NULL;
 
-	cpu_caches = stress_get_all_cpu_cache_details();
+	cpu_caches = stress_cpu_cache_get_all_details();
 	if (!cpu_caches) {
 		if (!args->instance)
 			pr_inf("%s: using built-in defaults as unable to "
@@ -73,7 +79,7 @@ static uint64_t get_L1_line_size(const stress_args_t *args)
 		return cache_size;
 	}
 
-	cache = stress_get_cpu_cache(cpu_caches, 1);
+	cache = stress_cpu_cache_get(cpu_caches, 1);
 	if (!cache) {
 		if (!args->instance)
 			pr_inf("%s: using built-in defaults as no suitable "
@@ -100,47 +106,48 @@ static uint64_t get_L1_line_size(const stress_args_t *args)
 }
 
 static int stress_cacheline_adjacent(
-	const stress_args_t *args,
-	const int index,
+	stress_args_t *args,
+	const int idx,
 	const bool parent,
 	const size_t l1_cacheline_size)
 {
 	register int i;
-	volatile uint8_t *cacheline = (volatile uint8_t *)g_shared->cacheline;
-	volatile uint8_t *data8 = cacheline + index;
+	volatile uint8_t *buffer = (volatile uint8_t *)g_shared->cacheline.buffer;
+	volatile uint8_t *data8 = buffer + idx;
 	register uint8_t val8 = *(data8);
 	volatile uint8_t *data8adjacent = (volatile uint8_t *)(((uintptr_t)data8) ^ 1);
 
 	(void)parent;
 	(void)l1_cacheline_size;
 
+PRAGMA_UNROLL
 	for (i = 0; i < 1024; i++) {
 		(*data8)++;
 		(void)(*data8adjacent);
-		shim_mb();
+		stress_asm_mb();
 		(*data8)++;
 		(void)(*data8adjacent);
-		shim_mb();
+		stress_asm_mb();
 		(*data8)++;
 		(void)(*data8adjacent);
-		shim_mb();
+		stress_asm_mb();
 		(*data8)++;
 		(void)(*data8adjacent);
-		shim_mb();
+		stress_asm_mb();
 		(*data8)++;
 		(void)(*data8adjacent);
-		shim_mb();
+		stress_asm_mb();
 		(*data8)++;
 		(void)(*data8adjacent);
-		shim_mb();
+		stress_asm_mb();
 		(*data8)++;
 		(void)(*data8adjacent);
-		shim_mb();
+		stress_asm_mb();
 		val8 += 7;
 
 		if (UNLIKELY(*data8 != val8)) {
 			pr_fail("%s: adjacent method: cache line error in offset 0x%x, expected %2" PRIx8 ", got %2" PRIx8 "\n",
-				args->name, index, val8, *data8);
+				args->name, idx, val8, *data8);
 			return EXIT_FAILURE;
 		}
 	}
@@ -148,19 +155,20 @@ static int stress_cacheline_adjacent(
 }
 
 static int stress_cacheline_copy(
-	const stress_args_t *args,
-	const int index,
+	stress_args_t *args,
+	const int idx,
 	const bool parent,
 	const size_t l1_cacheline_size)
 {
 	register int i;
-	volatile uint8_t *cacheline = (volatile uint8_t *)g_shared->cacheline;
-	volatile uint8_t *data8 = cacheline + index;
+	volatile uint8_t *buffer = (volatile uint8_t *)g_shared->cacheline.buffer;
+	volatile uint8_t *data8 = buffer + idx;
 	const volatile uint8_t *data8adjacent = (volatile uint8_t *)(((uintptr_t)data8) ^ 1);
 
 	(void)parent;
 	(void)l1_cacheline_size;
 
+PRAGMA_UNROLL
 	for (i = 0; i < 1024; i++) {
 		register uint8_t val8;
 
@@ -176,7 +184,7 @@ static int stress_cacheline_copy(
 
 		if (UNLIKELY(*data8 != val8)) {
 			pr_fail("%s: copy method: cache line error in offset 0x%x, expected %2" PRIx8 ", got %2" PRIx8 "\n",
-				args->name, index, val8, *data8);
+				args->name, idx, val8, *data8);
 			return EXIT_FAILURE;
 		}
 	}
@@ -184,39 +192,40 @@ static int stress_cacheline_copy(
 }
 
 static int stress_cacheline_inc(
-	const stress_args_t *args,
-	const int index,
+	stress_args_t *args,
+	const int idx,
 	const bool parent,
 	const size_t l1_cacheline_size)
 {
 	register int i;
-	volatile uint8_t *cacheline = (volatile uint8_t *)g_shared->cacheline;
-	volatile uint8_t *data8 = cacheline + index;
+	volatile uint8_t *buffer = (volatile uint8_t *)g_shared->cacheline.buffer;
+	volatile uint8_t *data8 = buffer + idx;
 	register uint8_t val8 = *(data8);
 
 	(void)parent;
 	(void)l1_cacheline_size;
 
+PRAGMA_UNROLL
 	for (i = 0; i < 1024; i++) {
 		(*data8)++;
-		shim_mb();
+		stress_asm_mb();
 		(*data8)++;
-		shim_mb();
+		stress_asm_mb();
 		(*data8)++;
-		shim_mb();
+		stress_asm_mb();
 		(*data8)++;
-		shim_mb();
+		stress_asm_mb();
 		(*data8)++;
-		shim_mb();
+		stress_asm_mb();
 		(*data8)++;
-		shim_mb();
+		stress_asm_mb();
 		(*data8)++;
-		shim_mb();
+		stress_asm_mb();
 		val8 += 7;
 
 		if (UNLIKELY(*data8 != val8)) {
 			pr_fail("%s: inc method: cache line error in offset 0x%x, expected %2" PRIx8 ", got %2" PRIx8 "\n",
-				args->name, index, val8, *data8);
+				args->name, idx, val8, *data8);
 			return EXIT_FAILURE;
 		}
 	}
@@ -224,74 +233,75 @@ static int stress_cacheline_inc(
 }
 
 static int stress_cacheline_rdwr(
-	const stress_args_t *args,
-	const int index,
+	stress_args_t *args,
+	const int idx,
 	const bool parent,
 	const size_t l1_cacheline_size)
 {
 	register int i;
-	volatile uint8_t *cacheline = (volatile uint8_t *)g_shared->cacheline;
-	volatile uint8_t *data8 = cacheline + index;
+	volatile uint8_t *buffer = (volatile uint8_t *)g_shared->cacheline.buffer;
+	volatile uint8_t *data8 = buffer + idx;
 	register uint8_t val8 = *(data8);
 
 	(void)parent;
 	(void)l1_cacheline_size;
 
+PRAGMA_UNROLL
 	for (i = 0; i < 1024; i++) {
 		uint8_t tmp;
 
 		tmp = *data8;
 		*data8 = tmp;
-		shim_mb();
+		stress_asm_mb();
 		tmp = *data8;
 		*data8 = tmp;
-		shim_mb();
+		stress_asm_mb();
 		tmp = *data8;
 		*data8 = tmp;
-		shim_mb();
+		stress_asm_mb();
 		tmp = *data8;
 		*data8 = tmp;
-		shim_mb();
+		stress_asm_mb();
 		tmp = *data8;
 		*data8 = tmp;
-		shim_mb();
+		stress_asm_mb();
 		tmp = *data8;
 		*data8 = tmp;
-		shim_mb();
+		stress_asm_mb();
 		tmp = *data8;
 		*data8 = tmp;
-		shim_mb();
+		stress_asm_mb();
 		tmp = *data8;
 		*data8 = tmp;
-		shim_mb();
+		stress_asm_mb();
 		tmp = *data8;
 		*data8 = tmp;
-		shim_mb();
+		stress_asm_mb();
 		tmp = *data8;
 		*data8 = tmp;
-		shim_mb();
+		stress_asm_mb();
 		tmp = *data8;
 		*data8 = tmp;
-		shim_mb();
+		stress_asm_mb();
 		tmp = *data8;
 		*data8 = tmp;
-		shim_mb();
+		stress_asm_mb();
 		tmp = *data8;
 		*data8 = tmp;
-		shim_mb();
+		stress_asm_mb();
 		tmp = *data8;
 		*data8 = tmp;
-		shim_mb();
+		stress_asm_mb();
 		tmp = *data8;
 		*data8 = tmp;
-		shim_mb();
+		stress_asm_mb();
 		tmp = *data8;
 		*data8 = tmp;
-		shim_mb();
+		stress_asm_mb();
 
 		if (UNLIKELY(*data8 != val8)) {
 			pr_fail("%s: rdwr method: cache line error in offset 0x%x, expected %2" PRIx8 ", got %2" PRIx8 "\n",
-				args->name, index, val8, *data8);
+				args->name, idx, val8, *data8);
 			return EXIT_FAILURE;
 		}
 	}
@@ -299,19 +309,20 @@ static int stress_cacheline_rdwr(
 }
 
 static int stress_cacheline_mix(
-	const stress_args_t *args,
-	const int index,
+	stress_args_t *args,
+	const int idx,
 	const bool parent,
 	const size_t l1_cacheline_size)
 {
 	register int i;
-	volatile uint8_t *cacheline = (volatile uint8_t *)(uintptr_t)g_shared->cacheline;
-	volatile uint8_t *data8 = cacheline + index;
+	volatile uint8_t *buffer = (volatile uint8_t *)(uintptr_t)g_shared->cacheline.buffer;
+	volatile uint8_t *data8 = buffer + idx;
 	static uint8_t tmp = 0xa5;
 
 	(void)parent;
 	(void)l1_cacheline_size;
 
+PRAGMA_UNROLL
 	for (i = 0; i < 1024; i++) {
 		register uint8_t val8;
 
@@ -321,7 +332,7 @@ static int stress_cacheline_mix(
 		EXERCISE(val8);
 		if (UNLIKELY(val8 != *data8)) {
 			pr_fail("%s: mix method: cache line error in offset 0x%x, expected %2" PRIx8 ", got %2" PRIx8 "\n",
-				args->name, index, val8, *data8);
+				args->name, idx, val8, *data8);
 			return EXIT_FAILURE;
 		}
 		tmp = val8;
@@ -330,16 +341,16 @@ static int stress_cacheline_mix(
 }
 
 static int stress_cacheline_rdrev64(
-	const stress_args_t *args,
-	const int index,
+	stress_args_t *args,
+	const int idx,
 	const bool parent,
 	const size_t l1_cacheline_size)
 {
 	register int i;
-	volatile uint8_t *cacheline = (volatile uint8_t *)g_shared->cacheline;
-	volatile uint8_t *data8 = cacheline + index;
-	const ssize_t cacheline_size = (ssize_t)g_shared->cacheline_size;
-	uintptr_t aligned_cacheline = (uintptr_t)cacheline & ~(l1_cacheline_size - 1);
+	volatile uint8_t *buffer = (volatile uint8_t *)g_shared->cacheline.buffer;
+	volatile uint8_t *data8 = buffer + idx;
+	const ssize_t cacheline_size = (ssize_t)g_shared->cacheline.size;
+	uintptr_t aligned_cacheline = (uintptr_t)buffer & ~(l1_cacheline_size - 1);
 
 	(void)parent;
 	(void)l1_cacheline_size;
@@ -352,15 +363,16 @@ static int stress_cacheline_rdrev64(
 		val8 = *data8;
 
 		/* read cache line backwards */
+PRAGMA_UNROLL
 		for (j = cacheline_size - 8; j >= 0; j -= 8) {
 			volatile uint64_t *data64 = (volatile uint64_t *)(aligned_cacheline + (size_t)j);
 
 			(void)*data64;
-			shim_mb();
+			stress_asm_mb();
 		}
 		if (UNLIKELY(val8 != *data8)) {
 			pr_fail("%s: rdrev64 method: cache line error in offset 0x%x, expected %2" PRIx8 ", got %2" PRIx8 "\n",
-				args->name, index, val8, *data8);
+				args->name, idx, val8, *data8);
 			return EXIT_FAILURE;
 		}
 	}
@@ -368,16 +380,16 @@ static int stress_cacheline_rdrev64(
 }
 
 static int stress_cacheline_rdfwd64(
-	const stress_args_t *args,
-	const int index,
+	stress_args_t *args,
+	const int idx,
 	const bool parent,
 	const size_t l1_cacheline_size)
 {
 	register int i;
-	volatile uint8_t *cacheline = (volatile uint8_t *)g_shared->cacheline;
-	volatile uint8_t *data8 = cacheline + index;
-	const size_t cacheline_size = g_shared->cacheline_size;
-	uintptr_t aligned_cacheline = (uintptr_t)cacheline & ~(l1_cacheline_size - 1);
+	volatile uint8_t *buffer = (volatile uint8_t *)g_shared->cacheline.buffer;
+	volatile uint8_t *data8 = buffer + idx;
+	const size_t cacheline_size = g_shared->cacheline.size;
+	uintptr_t aligned_cacheline = (uintptr_t)buffer & ~(l1_cacheline_size - 1);
 
 	(void)parent;
 	(void)l1_cacheline_size;
@@ -390,15 +402,18 @@ static int stress_cacheline_rdfwd64(
 		val8 = *data8;
 
 		/* read cache line forwards */
+#if !defined(HAVE_COMPILER_CLANG)
+PRAGMA_UNROLL
+#endif
 		for (j = 0; j < cacheline_size; j += 8) {
 			volatile uint64_t *data64 = (volatile uint64_t *)(aligned_cacheline + j);
 
 			(void)*data64;
-			shim_mb();
+			stress_asm_mb();
 		}
 		if (UNLIKELY(val8 != *data8)) {
 			pr_fail("%s: rdfwd64: cache line error in offset 0x%x, expected %2" PRIx8 ", got %2" PRIx8 "\n",
-				args->name, index, val8, *data8);
+				args->name, idx, val8, *data8);
 			return EXIT_FAILURE;
 		}
 	}
@@ -406,14 +421,14 @@ static int stress_cacheline_rdfwd64(
 }
 
 static int stress_cacheline_rdints(
-	const stress_args_t *args,
-	const int index,
+	stress_args_t *args,
+	const int idx,
 	const bool parent,
 	const size_t l1_cacheline_size)
 {
 	register int i;
-	volatile uint8_t *cacheline = (volatile uint8_t *)g_shared->cacheline;
-	volatile uint8_t *data8 = cacheline + index;
+	volatile uint8_t *buffer = (volatile uint8_t *)g_shared->cacheline.buffer;
+	volatile uint8_t *data8 = buffer + idx;
 	volatile uint16_t *data16 = (uint16_t *)(((uintptr_t)data8) & ~(uintptr_t)1);
 	volatile uint32_t *data32 = (uint32_t *)(((uintptr_t)data8) & ~(uintptr_t)3);
 	volatile uint64_t *data64 = (uint64_t *)(((uintptr_t)data8) & ~(uintptr_t)7);
@@ -424,34 +439,35 @@ static int stress_cacheline_rdints(
 	(void)parent;
 	(void)l1_cacheline_size;
 
+PRAGMA_UNROLL
 	for (i = 0; i < 1024; i++) {
 		uint8_t val8;
 
 		/* 1 byte increment and read */
 		(*data8)++;
 		val8 = *data8;
-		shim_mb();
+		stress_asm_mb();
 
 		/* 2 byte reads from same location */
 		(void)*(data16);
-		shim_mb();
+		stress_asm_mb();
 
 		/* 4 byte reads from same location */
 		(void)*(data32);
-		shim_mb();
+		stress_asm_mb();
 
 		/* 8 byte reads from same location */
 		(void)*(data64);
-		shim_mb();
+		stress_asm_mb();
 
 #if defined(HAVE_INT128_T)
 		/* 116 byte reads from same location */
 		(void)*(data128);
-		shim_mb();
+		stress_asm_mb();
 #endif
 		if (UNLIKELY(val8 != *data8)) {
 			pr_fail("%s: rdints method: cache line error in offset 0x%x, expected %2" PRIx8 ", got %2" PRIx8 "\n",
-				args->name, index, val8, *data8);
+				args->name, idx, val8, *data8);
 			return EXIT_FAILURE;
 		}
 	}
@@ -459,18 +475,19 @@ static int stress_cacheline_rdints(
 }
 
 static int stress_cacheline_bits(
-	const stress_args_t *args,
-	const int index,
+	stress_args_t *args,
+	const int idx,
 	const bool parent,
 	const size_t l1_cacheline_size)
 {
 	register int i;
-	volatile uint8_t *cacheline = (volatile uint8_t *)g_shared->cacheline;
-	volatile uint8_t *data8 = cacheline + index;
+	volatile uint8_t *buffer = (volatile uint8_t *)g_shared->cacheline.buffer;
+	volatile uint8_t *data8 = buffer + idx;
 
 	(void)parent;
 	(void)l1_cacheline_size;
 
+PRAGMA_UNROLL
 	for (i = 0; i < 1024; i++) {
 		register uint8_t val8;
 
@@ -479,18 +496,18 @@ static int stress_cacheline_bits(
 
 		val8 = (uint8_t)(1U << (i & 7));
 		*data8 = val8;
-		shim_mb();
-		if (*data8 != val8) {
+		stress_asm_mb();
+		if (UNLIKELY(*data8 != val8)) {
 			pr_fail("%s: bits method: cache line error in offset 0x%x, expected %2" PRIx8 ", got %2" PRIx8 "\n",
-				args->name, index, val8, *data8);
+				args->name, idx, val8, *data8);
 			return EXIT_FAILURE;
 		}
 		val8 ^= 0xff;
 		*data8 = val8;
-		shim_mb();
-		if (*data8 != val8) {
+		stress_asm_mb();
+		if (UNLIKELY(*data8 != val8)) {
 			pr_fail("%s: bits method: cache line error in offset 0x%x, expected %2" PRIx8 ", got %2" PRIx8 "\n",
-				args->name, index, val8, *data8);
+				args->name, idx, val8, *data8);
 			return EXIT_FAILURE;
 		}
 	}
@@ -499,19 +516,20 @@ static int stress_cacheline_bits(
 
 #if defined(SHIM_ATOMIC_INC)
 static int stress_cacheline_atomicinc(
-	const stress_args_t *args,
-	const int index,
+	stress_args_t *args,
+	const int idx,
 	const bool parent,
 	const size_t l1_cacheline_size)
 {
 	register int i;
-	volatile uint8_t *cacheline = (volatile uint8_t *)g_shared->cacheline;
-	volatile uint8_t *data8 = cacheline + index;
+	volatile uint8_t *buffer = (volatile uint8_t *)g_shared->cacheline.buffer;
+	volatile uint8_t *data8 = buffer + idx;
 	register uint8_t val8 = *(data8);
 
 	(void)parent;
 	(void)l1_cacheline_size;
 
+PRAGMA_UNROLL
 	for (i = 0; i < 1024; i++) {
 		SHIM_ATOMIC_INC(data8);
 		SHIM_ATOMIC_INC(data8);
@@ -524,7 +542,7 @@ static int stress_cacheline_atomicinc(
 
 		if (UNLIKELY(*data8 != val8)) {
 			pr_fail("%s: atomicinc method: cache line error in offset 0x%x, expected %2" PRIx8 ", got %2" PRIx8 "\n",
-				args->name, index, val8, *data8);
+				args->name, idx, val8, *data8);
 			return EXIT_FAILURE;
 		}
 	}
@@ -533,8 +551,8 @@ static int stress_cacheline_atomicinc(
 #endif
 
 static int stress_cacheline_all(
-	const stress_args_t *args,
-	const int index,
+	stress_args_t *args,
+	const int idx,
 	const bool parent,
 	const size_t l1_cacheline_size);
 
@@ -554,54 +572,30 @@ static const stress_cacheline_method_t cacheline_methods[] = {
 	{ "rdwr",	stress_cacheline_rdwr },
 };
 
+static const char *stress_cacheline_method(const size_t i)
+{
+	return (i < SIZEOF_ARRAY(cacheline_methods)) ? cacheline_methods[i].name : NULL;
+}
+
 static int stress_cacheline_all(
-	const stress_args_t *args,
-	const int index,
+	stress_args_t *args,
+	const int idx,
 	const bool parent,
 	const size_t l1_cacheline_size)
 {
 	size_t i;
 	const size_t n = SIZEOF_ARRAY(cacheline_methods);
 
-	for (i = 1; keep_stressing(args) && (i < n); i++) {
+	for (i = 1; stress_continue(args) && (i < n); i++) {
 		int rc;
 
-		rc = cacheline_methods[i].func(args, index, parent, l1_cacheline_size);
+		rc = cacheline_methods[i].func(args, idx, parent, l1_cacheline_size);
 		if (rc != EXIT_SUCCESS)
 			return rc;
 	}
 	return EXIT_SUCCESS;
 }
 
-static int stress_set_cacheline_affinity(const char *opt)
-{
-	return stress_set_setting_true("cacheline-affinity", opt);
-}
-
-/*
- *  stress_set_cacheline_method()
- *	set the default cachline stress method
- */
-static int stress_set_cacheline_method(const char *name)
-{
-	size_t i;
-
-	for (i = 0; i < SIZEOF_ARRAY(cacheline_methods); i++) {
-		if (!strcmp(cacheline_methods[i].name, name)) {
-			stress_set_setting("cacheline-method", TYPE_ID_SIZE_T, &i);
-			return 0;
-		}
-	}
-
-	(void)fprintf(stderr, "cacheline-method must be one of:");
-	for (i = 0; i < SIZEOF_ARRAY(cacheline_methods); i++) {
-		(void)fprintf(stderr, " %s", cacheline_methods[i].name);
-	}
-	(void)fprintf(stderr, "\n");
-
-	return -1;
-}
-
 #if defined(HAVE_SCHED_GETAFFINITY) &&	\
     defined(HAVE_SCHED_SETAFFINITY)
 /*
@@ -610,23 +604,27 @@ static int stress_set_cacheline_method(const char *name)
  *	and parent/child offset modulo number of CPUs
  */
 static inline void stress_cacheline_change_affinity(
-	const stress_args_t *args,
-	const uint32_t cpus,
+	stress_args_t *args,
+	const uint32_t n_cpus,
+	const uint32_t *cpus,
 	bool parent)
 {
-	cpu_set_t mask;
-	double now = stress_time_now() * 100;
-	uint32_t cpu = ((uint32_t)args->instance + (uint32_t)parent + (uint32_t)now) % cpus;
+	if (n_cpus > 0) {
+		cpu_set_t mask;
+		const double now = stress_time_now() * 100;
+		const uint32_t cpu_idx = ((uint32_t)args->instance + (uint32_t)parent + (uint32_t)now) % n_cpus;
+		const uint32_t cpu = cpus[cpu_idx];
 
-	CPU_ZERO(&mask);
-	CPU_SET((int)cpu, &mask);
-	VOID_RET(int, sched_setaffinity(0, sizeof(mask), &mask));
+		CPU_ZERO(&mask);
+		CPU_SET((int)cpu, &mask);
+		VOID_RET(int, sched_setaffinity(0, sizeof(mask), &mask));
+	}
 }
 #endif
 
 static int stress_cacheline_child(
-	const stress_args_t *args,
-	const int index,
+	stress_args_t *args,
+	const int idx,
 	const bool parent,
 	const size_t l1_cacheline_size,
 	stress_cacheline_func func,
@@ -635,44 +633,100 @@ static int stress_cacheline_child(
 	int rc;
 #if defined(HAVE_SCHED_GETAFFINITY) &&	\
     defined(HAVE_SCHED_SETAFFINITY)
-	const uint32_t cpus = (uint32_t)stress_get_processors_configured();
+	uint32_t *cpus;
+	const uint32_t n_cpus = stress_get_usable_cpus(&cpus, true);
 #endif
 
 	(void)cacheline_affinity;
 
 	do {
-		rc = func(args, index, parent, l1_cacheline_size);
+		rc = func(args, idx, parent, l1_cacheline_size);
 		if (parent)
-			inc_counter(args);
+			stress_bogo_inc(args);
 
 #if defined(HAVE_SCHED_GETAFFINITY) &&	\
     defined(HAVE_SCHED_SETAFFINITY)
 		if (cacheline_affinity)
-			stress_cacheline_change_affinity(args, cpus, parent);
+			stress_cacheline_change_affinity(args, n_cpus, cpus, parent);
 #endif
-	} while ((rc == EXIT_SUCCESS) && keep_stressing(args));
-
-	/* Child tell parent it has finished */
-	if (!parent)
-		(void)kill(getppid(), SIGALRM);
+	} while ((rc == EXIT_SUCCESS) && stress_continue(args));
 
+#if defined(HAVE_SCHED_GETAFFINITY) &&	\
+    defined(HAVE_SCHED_SETAFFINITY)
+	stress_free_usable_cpus(&cpus);
+#endif
 	return rc;
 }
 
 /*
- *  stress_cacheline()
- *	execise a cacheline by multiple processes
+ *  stress_cacheline_init()
+ *	called once by stress-ng, so we can set idx to 0
  */
-static int stress_cacheline(const stress_args_t *args)
+static void stress_cacheline_init(const uint32_t num_instances)
+{
+	(void)num_instances;
+
+	g_shared->cacheline.index = 0;
+	g_shared->cacheline.lock = stress_lock_create("cacheline");
+}
+
+/*
+ *  stress_cacheline_deinit()
+ *	called once by stress-ng, so we can set idx to 0
+ */
+static void stress_cacheline_deinit(void)
+{
+	if (g_shared->cacheline.lock) {
+		stress_lock_destroy(g_shared->cacheline.lock);
+		g_shared->cacheline.lock = NULL;
+		g_shared->cacheline.index = 0;
+	}
+}
+
+static int stress_cacheline_next_idx(void)
+{
+	int ret;
+
+	if (stress_lock_acquire(g_shared->cacheline.lock) < 0)
+		return -1;
+
+	ret = g_shared->cacheline.index;
+	g_shared->cacheline.index++;
+
+	if (stress_lock_release(g_shared->cacheline.lock) < 0)
+		return -1;
+
+	return ret * 2;
+}
+
+/*
+ *  stress_cacheline()
+ *	exercise a cacheline by multiple processes
+ */
+static int stress_cacheline(stress_args_t *args)
 {
 	size_t l1_cacheline_size = (size_t)get_L1_line_size(args);
-	const int index = (int)(args->instance * 2);
+	int idx;
 	pid_t pid;
 	int rc = EXIT_SUCCESS;
 	size_t cacheline_method = 0;
 	stress_cacheline_func func;
 	bool cacheline_affinity = false;
 
+	if (stress_sigchld_set_handler(args) < 0)
+		return EXIT_NO_RESOURCE;
+
+	if (!g_shared->cacheline.lock) {
+		pr_inf("%s: failed to initialized cacheline lock, skipping stressor\n", args->name);
+		return EXIT_NO_RESOURCE;
+	}
+
+	idx = stress_cacheline_next_idx();
+	if (idx < 0) {
+		pr_inf("%s: failed to get cacheline idx, skipping stressor\n", args->name);
+		return EXIT_NO_RESOURCE;
+	}
+
 	(void)stress_get_setting("cacheline-affinity", &cacheline_affinity);
 	(void)stress_get_setting("cacheline-method", &cacheline_method);
 
@@ -688,30 +742,26 @@ static int stress_cacheline(const stress_args_t *args)
 
 	func = cacheline_methods[cacheline_method].func;
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			goto finish;
 		pr_err("%s: fork failed: errno=%d: (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	} else if (pid == 0) {
-		rc = stress_cacheline_child(args, index + 1, false, l1_cacheline_size, func, cacheline_affinity);
+		rc = stress_cacheline_child(args, idx + 1, false, l1_cacheline_size, func, cacheline_affinity);
 		_exit(rc);
 	} else {
-		int status;
-
-		stress_cacheline_child(args, index, true, l1_cacheline_size, func, cacheline_affinity);
-
-		(void)kill(pid, SIGALRM);
-		(void)shim_waitpid(pid, &status, 0);
-
-		if (WIFEXITED(status) && (WEXITSTATUS(status) != EXIT_SUCCESS))
-			rc = WEXITSTATUS(status);
+		stress_cacheline_child(args, idx, true, l1_cacheline_size, func, cacheline_affinity);
+		if (stress_kill_and_wait(args, pid, SIGALRM, false) != EXIT_SUCCESS)
+			rc = EXIT_FAILURE;
 	}
 
 finish:
@@ -720,16 +770,18 @@ finish:
 	return rc;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_cacheline_affinity,	stress_set_cacheline_affinity },
-	{ OPT_cacheline_method,		stress_set_cacheline_method },
-	{ 0,				NULL },
+static const stress_opt_t opts[] = {
+	{ OPT_cacheline_affinity, "cacheline-affinity", TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_cacheline_method,   "cacheline-method",   TYPE_ID_SIZE_T_METHOD, 0, 0, stress_cacheline_method },
+	END_OPT,
 };
 
-stressor_info_t stress_cacheline_info = {
+const stressor_info_t stress_cacheline_info = {
 	.stressor = stress_cacheline,
 	.class = CLASS_CPU_CACHE,
 	.verify = VERIFY_ALWAYS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
+	.init = stress_cacheline_init,
+	.deinit = stress_cacheline_deinit,
 	.help = help
 };
