@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,6 +20,7 @@
 #include "stress-ng.h"
 #include "core-capabilities.h"
 #include "core-pthread.h"
+#include <sys/socket.h>
 
 #if defined(__NR_userfaultfd)
 #define HAVE_USERFAULTFD
@@ -127,15 +128,24 @@ static const int types[] = {
 	0,
 };
 
+static const int close_range_flags[] = {
+#if defined(CLOSE_RANGE_UNSHARE)
+	CLOSE_RANGE_UNSHARE,
+#endif
+#if defined(CLOSE_RANGE_CLOEXEC)
+	CLOSE_RANGE_CLOEXEC,
+#endif
+	0
+};
+
 /*
  *  stress_close_func()
- *	pthread that exits immediately
+ *	pthread that close file descriptors
  */
 static void *stress_close_func(void *arg)
 {
-	static void *nowt = NULL;
 	stress_pthread_args_t *pargs = (stress_pthread_args_t *)arg;
-	const stress_args_t *args = pargs->args;
+	stress_args_t *args = pargs->args;
 
 	/*
 	 *  Block all signals, let controlling thread
@@ -146,16 +156,17 @@ static void *stress_close_func(void *arg)
 	(void)sigprocmask(SIG_BLOCK, &set, NULL);
 #endif
 
-	while (keep_stressing(args)) {
+	while (stress_continue(args)) {
 		const uint64_t delay =
-			max_delay_us ? stress_mwc32() % max_delay_us : 0;
+			max_delay_us ? stress_mwc64modn(max_delay_us) : 0;
 		int fds[FDS_TO_DUP], i, ret;
+		int flag;
 
 		for (i = 0; i < FDS_TO_DUP; i++) {
 			fds[i] = dup2(fileno(stderr), i + FDS_START);
 		}
 
-		shim_usleep_interruptible(delay);
+		(void)shim_usleep_interruptible(delay);
 		if (fd != -1)
 			(void)close(fd);
 		if (dupfd != -1)
@@ -163,7 +174,7 @@ static void *stress_close_func(void *arg)
 
 #if defined(F_GETFL)
 		{
-			int fd_rnd = (int)stress_mwc32() + 64;
+			const int fd_rnd = (int)stress_mwc32() + 64;
 
 			/*
 			 *  close random unused fd to force EBADF
@@ -177,7 +188,8 @@ static void *stress_close_func(void *arg)
 		/*
 		 *  close a range of fds
 		 */
-		ret = shim_close_range(FDS_START, FDS_START + FDS_TO_DUP, 0);
+		flag = close_range_flags[stress_mwc8modn(SIZEOF_ARRAY(close_range_flags))];
+		ret = shim_close_range(FDS_START, FDS_START + FDS_TO_DUP, flag);
 		if ((ret < 0) || (errno == ENOSYS)) {
 			for (i = 0; i < FDS_TO_DUP; i++)
 				(void)close(fds[i]);
@@ -193,14 +205,14 @@ static void *stress_close_func(void *arg)
 		VOID_RET(int, shim_close_range(FDS_START + FDS_TO_DUP, FDS_START, ~0U));
 	}
 
-	return &nowt;
+	return &g_nowt;
 }
 
 /*
  *  stress_close()
- *	stress by creating pthreads
+ *	stress by closing file descriptors
  */
-static int stress_close(const stress_args_t *args)
+static int stress_close(stress_args_t *args)
 {
 	stress_pthread_args_t pargs;
 	pthread_t pthread[MAX_PTHREADS];
@@ -211,6 +223,7 @@ static int stress_close(const stress_args_t *args)
 	const uid_t uid = getuid();
 	const gid_t gid = getgid();
 	const bool not_root = !stress_check_capability(SHIM_CAP_IS_ROOT);
+	bool close_failure = false;
 	double max_duration = 0.0, duration = 0.0, count = 0.0, rate;
 #if defined(HAVE_FACCESSAT)
 	int file_fd = -1;
@@ -261,6 +274,8 @@ static int stress_close(const stress_args_t *args)
 #else
 	UNEXPECTED
 #endif
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -272,10 +287,10 @@ static int stress_close(const stress_args_t *args)
 		fd = -1;
 		t1 = stress_time_now();
 
-		switch (stress_mwc8() % 15) {
+		switch (stress_mwc8() & 0x0f) {
 		case 0:
-			domain = stress_mwc8() % SIZEOF_ARRAY(domains);	/* cppcheck-suppress moduloofone */
-			type = stress_mwc8() % SIZEOF_ARRAY(types);	/* cppcheck-suppress moduloofone */
+			domain = stress_mwc8modn((uint8_t)SIZEOF_ARRAY(domains));
+			type = stress_mwc8modn((uint8_t)SIZEOF_ARRAY(types));
 			fd = socket(domains[domain], types[type], 0);
 			break;
 		case 1:
@@ -312,8 +327,9 @@ static int stress_close(const stress_args_t *args)
 			break;
 #endif
 		case 7:
-			if (pipe(pipefds) == 0) {
+			if (LIKELY(pipe(pipefds) == 0)) {
 				fd = pipefds[0];
+				pipefds[0] = -1;
 				(void)close(pipefds[1]);
 			}
 			break;
@@ -365,13 +381,17 @@ static int stress_close(const stress_args_t *args)
 			fd = dup(0);
 #endif
 			break;
+		case 15:
+			fd = open("/dev/stdin", O_RDONLY);
+			break;
 		default:
+			fd = open(".", O_RDONLY | O_DIRECTORY);
 			break;
 		}
-		if (fd == -1)
+		if (UNLIKELY(fd == -1))
 			fd = open("/dev/null", O_RDWR);
 
-		if (fd != -1) {
+		if (LIKELY(fd != -1)) {
 			double t;
 
 			dupfd = dup(fd);
@@ -401,25 +421,45 @@ static int stress_close(const stress_args_t *args)
 			 * is a file descriptor referring to a file other than a directory
 			 */
 			ret = faccessat(file_fd, "./", F_OK, 0);
-			if (ret >= 0) {
+			if (UNLIKELY(ret >= 0)) {
 				pr_fail("%s: faccessat opened file descriptor succeeded unexpectedly, "
 					"errno=%d (%s)\n", args->name, errno, strerror(errno));
-				(void)close(ret);
+				(void)close(fd);
+				if (dupfd != -1)
+					(void)close(dupfd);
+				rc = EXIT_FAILURE;
 				goto tidy;
 			}
 #else
 			UNEXPECTED
 #endif
-			VOID_RET(int, fstat(fd, &statbuf));
+			VOID_RET(int, shim_fstat(fd, &statbuf));
 
-			VOID_RET(int, close(dup(STDOUT_FILENO)));
+			ret = dup(STDOUT_FILENO);
+			if (LIKELY(ret != -1))
+				(void)close(ret);
 
 			t = stress_time_now();
-			if (close(fd) == 0) {
+			if (LIKELY(close(fd) == 0)) {
 				duration += stress_time_now() - t;
 				count += 1.0;
-			}
 
+				/*
+				 *  A close on a successfully closed fd should fail
+				 */
+				if (UNLIKELY(close(fd) == 0)) {
+					pr_fail("%s: unexpectedly able to close the same file %d twice\n",
+						args->name, fd);
+					close_failure = true;
+				} else {
+					if (UNLIKELY((errno != EBADF) &&
+						     (errno != EINTR))) {
+						pr_fail("%s: expected error on close failure, error=%d (%s)\n",
+							args->name,  errno, strerror(errno));
+						close_failure = true;
+					}
+				}
+			}
 			if (dupfd != -1)
 				(void)close(dupfd);
 		}
@@ -434,15 +474,16 @@ static int stress_close(const stress_args_t *args)
 		max_duration *= 0.995;
 		if (max_duration < 1.0)
 			max_duration = 1.0;
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
-	rc = EXIT_SUCCESS;
+	rc = close_failure ? EXIT_FAILURE : EXIT_SUCCESS;
 tidy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	rate = (duration > 0.0) ? count / duration : 0.0;
-	stress_misc_stats_set(args->misc_stats, 0, "close calls per sec", rate);
+	stress_metrics_set(args, 0, "close calls per sec",
+		rate, STRESS_METRIC_HARMONIC_MEAN);
 
 	for (i = 0; i < MAX_PTHREADS; i++) {
 		if (rets[i] == -1)
@@ -465,15 +506,17 @@ tidy:
 	return rc;
 }
 
-stressor_info_t stress_close_info = {
+const stressor_info_t stress_close_info = {
 	.stressor = stress_close,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_close_info = {
+const stressor_info_t stress_close_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without pthread support"
 };
