@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 
 #define MIN_COPY_FILE_BYTES	(128 * MB)
 #define MAX_COPY_FILE_BYTES	(MAX_FILE_LIMIT)
@@ -32,41 +33,51 @@ static const stress_help_t help[] = {
 
 };
 
-static int stress_set_copy_file_bytes(const char *opt)
-{
-	uint64_t copy_file_bytes;
-
-	copy_file_bytes = stress_get_uint64_byte_filesystem(opt, 1);
-	stress_check_range_bytes("copy-file-bytes", copy_file_bytes,
-		MIN_COPY_FILE_BYTES, MAX_COPY_FILE_BYTES);
-	return stress_set_setting("copy-file-bytes", TYPE_ID_UINT64, &copy_file_bytes);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_copy_file_bytes,	stress_set_copy_file_bytes },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_copy_file_bytes, "copy-file-bytes", TYPE_ID_UINT64_BYTES_FS, MIN_COPY_FILE_BYTES, MAX_COPY_FILE_BYTES, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_COPY_FILE_RANGE)
 
 #define COPY_FILE_MAX_BUF_SIZE	(4096)
 
+/*
+ *  stress_copy_file_seek64()
+ *	seek with off64_t
+ */
+static inline shim_off64_t stress_copy_file_seek64(int fd, shim_off64_t off64, int whence)
+{
+#if defined(HAVE_LSEEK64)
+	return lseek64(fd, off64, whence);
+#else
+	/* max signed off_t without causing signed overflow */
+	const off_t max_off64 = ((((off_t)1U << ((sizeof(off_t) * 8) - 2)) - 1) * 2 + 1);
+
+	if (off64 > max_off64) {
+		errno = EINVAL;
+		return -1;
+	}
+	return (shim_off64_t)lseek(fd, (off_t)off64, whence);
+#endif
+}
+
 /*
  *  stress_copy_file_fill()
  *	fill chunk of file with random value
  */
 static int stress_copy_file_fill(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
-	const off_t off,
+	const shim_off64_t off64,
 	const ssize_t size)
 {
 	char buf[COPY_FILE_MAX_BUF_SIZE];
 	ssize_t sz = size;
 
-	(void)memset(buf, stress_mwc8(), sizeof(buf));
+	(void)shim_memset(buf, stress_mwc8(), sizeof(buf));
 
-	if (lseek(fd, off, SEEK_SET) < 0)
+	if (stress_copy_file_seek64(fd, off64, SEEK_SET) < 0)
 		return -1;
 
 	while (sz > 0) {
@@ -76,7 +87,7 @@ static int stress_copy_file_fill(
 		if (n < 0)
 			return -1;
 		sz -= n;
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			return 0;
 	}
 	return 0;
@@ -89,9 +100,9 @@ static int stress_copy_file_fill(
  */
 static int stress_copy_file_range_verify(
 	const int fd_in,
-	off_t *off_in,
+	shim_off64_t *off64_in,
 	const int fd_out,
-	off_t *off_out,
+	shim_off64_t *off64_out,
 	const ssize_t bytes)
 {
 	size_t bytes_left = (size_t)bytes;
@@ -99,29 +110,15 @@ static int stress_copy_file_range_verify(
 	while (bytes_left > 0) {
 		ssize_t n, bytes_in, bytes_out;
 		size_t sz = STRESS_MINIMUM(bytes_left, COPY_FILE_MAX_BUF_SIZE);
-
 		char buf_in[COPY_FILE_MAX_BUF_SIZE];
 		char buf_out[COPY_FILE_MAX_BUF_SIZE];
+		shim_off64_t off_ret;
 
-#if defined(HAVE_PREAD)
-		bytes_in = pread(fd_in, buf_in, sz, *off_in);
-		if (bytes_in == 0)
-			return 0;
-		if (bytes_in < 0)
-			break;
-		bytes_out = pread(fd_out, buf_out, sz, *off_out);
-		if (bytes_out == 0)
-			return 0;
-		if (bytes_out <= 0)
-			break;
-#else
-		off_t off_ret;
-
-		off_ret = lseek(fd_in, *off_in, SEEK_SET);
-		if (off_ret != *off_in)
+		off_ret = stress_copy_file_seek64(fd_in, *off64_in, SEEK_SET);
+		if (off_ret != *off64_in)
 			return -1;
-		off_ret = lseek(fd_out, *off_out, SEEK_SET);
-		if (off_ret != *off_out)
+		off_ret = stress_copy_file_seek64(fd_out, *off64_out, SEEK_SET);
+		if (off_ret != *off64_out)
 			return -1;
 		bytes_in = read(fd_in, buf_in, sz);
 		if (bytes_in == 0)
@@ -133,14 +130,13 @@ static int stress_copy_file_range_verify(
 			return 0;
 		if (bytes_out <= 0)
 			break;
-#endif
 
 		n = STRESS_MINIMUM(bytes_in, bytes_out);
-		if (memcmp(buf_in, buf_out, (size_t)n) != 0)
+		if (shim_memcmp(buf_in, buf_out, (size_t)n) != 0)
 			return -1;
 		bytes_left -= n;
-		*off_in += n;
-		*off_out += n;
+		*off64_in += n;
+		*off64_out += n;
 	}
 	if (bytes_left > 0)
 		return -1;
@@ -151,13 +147,14 @@ static int stress_copy_file_range_verify(
  *  stress_copy_file
  *	stress reading chunks of file using copy_file_range()
  */
-static int stress_copy_file(const stress_args_t *args)
+static int stress_copy_file(stress_args_t *args)
 {
 	int fd_in, fd_out, rc = EXIT_FAILURE, ret;
 	const int fd_bad = stress_get_bad_fd();
 	char filename[PATH_MAX - 5], tmp[PATH_MAX];
 	uint64_t copy_file_bytes = DEFAULT_COPY_FILE_BYTES;
 	double duration = 0.0, bytes = 0.0, rate;
+	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 
 	if (!stress_get_setting("copy-file-bytes", &copy_file_bytes)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
@@ -190,17 +187,9 @@ static int stress_copy_file(const stress_args_t *args)
 		rc = stress_exit_status(errno);
 		pr_fail("%s: ftruncated failed, errno=%d (%s)%s\n",
 			args->name, errno, strerror(errno),
-			stress_fs_type(tmp));
+			stress_get_fs_type(tmp));
 		goto tidy_in;
 	}
-	/*
-	if (shim_fsync(fd_in) < 0) {
-		pr_fail("%s: fsync failed, errno=%d (%s)%s\n",
-			args->name, errno, strerror(errno),
-			stress_fs_type(tmp));
-		goto tidy_in;
-	}
-	*/
 
 	(void)snprintf(tmp, sizeof(tmp), "%s-copy", filename);
 	if ((fd_out = open(tmp, O_CREAT | O_RDWR,  S_IRUSR | S_IWUSR)) < 0) {
@@ -209,28 +198,46 @@ static int stress_copy_file(const stress_args_t *args)
 			args->name, tmp, errno, strerror(errno));
 		goto tidy_in;
 	}
+
+#if defined(HAVE_PATHCONF)
+	/* Exercise some pathconf options */
+#if defined(_PC_REC_INCR_XFER_SIZE)
+	VOID_RET(long int, pathconf(tmp, _PC_REC_INCR_XFER_SIZE));
+#endif
+#if defined(_PC_REC_MAX_XFER_SIZE)
+	VOID_RET(long int, pathconf(tmp, _PC_REC_MAX_XFER_SIZE));
+#endif
+#if defined(_PC_REC_MIN_XFER_SIZE)
+	VOID_RET(long int, pathconf(tmp, _PC_REC_MIN_XFER_SIZE));
+#endif
+#if defined(_PC_REC_XFER_ALIGN)
+	VOID_RET(long int, pathconf(tmp, _PC_REC_XFER_ALIGN));
+#endif
+#endif
 	(void)shim_unlink(tmp);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		ssize_t copy_ret;
-		shim_loff_t off_in, off_out, off_in_orig, off_out_orig;
+		shim_off64_t off64_in, off64_out, off64_in_orig, off64_out_orig;
 		double t;
 
-		off_in_orig = (shim_loff_t)stress_mwc64modn(copy_file_bytes - DEFAULT_COPY_FILE_SIZE);
-		off_in = off_in_orig;
-		off_out_orig = (shim_loff_t)stress_mwc64modn(copy_file_bytes - DEFAULT_COPY_FILE_SIZE);
-		off_out = off_out_orig;
+		off64_in_orig = (shim_loff_t)stress_mwc64modn(copy_file_bytes - DEFAULT_COPY_FILE_SIZE);
+		off64_in = off64_in_orig;
+		off64_out_orig = (shim_loff_t)stress_mwc64modn(copy_file_bytes - DEFAULT_COPY_FILE_SIZE);
+		off64_out = off64_out_orig;
 
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
-		stress_copy_file_fill(args, fd_in, (off_t)off_in, DEFAULT_COPY_FILE_SIZE);
-		if (!keep_stressing(args))
+		stress_copy_file_fill(args, fd_in, off64_in, DEFAULT_COPY_FILE_SIZE);
+		if (!stress_continue(args))
 			break;
 		t = stress_time_now();
-		copy_ret = shim_copy_file_range(fd_in, &off_in, fd_out,
-						&off_out, DEFAULT_COPY_FILE_SIZE, 0);
+		copy_ret = shim_copy_file_range(fd_in, &off64_in, fd_out,
+						&off64_out, DEFAULT_COPY_FILE_SIZE, 0);
 		if (LIKELY(copy_ret >= 0)) {
 			duration += stress_time_now() - t;
 			bytes += (double)copy_ret;
@@ -239,10 +246,17 @@ static int stress_copy_file(const stress_args_t *args)
 			    (errno == EINTR) ||
 			    (errno == ENOSPC))
 				continue;
+			if (errno == ENOSYS) {
+				pr_inf_skip("%s: copy_file_range failed, system "
+					"call not implemented, skipping stressor\n",
+					args->name);
+				rc = EXIT_NO_RESOURCE;
+				goto tidy_out;
+			}
 			if (errno == EINVAL) {
 				pr_inf_skip("%s: copy_file_range failed, the "
 					"kernel splice may be not implemented "
-					"for the file system, skipping stressor.\n",
+					"for the file system, skipping stressor\n",
 					args->name);
 				rc = EXIT_NO_RESOURCE;
 				goto tidy_out;
@@ -251,45 +265,46 @@ static int stress_copy_file(const stress_args_t *args)
 				args->name, errno, strerror(errno));
 			goto tidy_out;
 		}
-		if (g_opt_flags & OPT_FLAGS_VERIFY) {
+		if (verify) {
 			int verify_ret;
-			off_in = off_in_orig;
-			off_out = off_out_orig;
+			off64_in = off64_in_orig;
+			off64_out = off64_out_orig;
 
-			verify_ret = stress_copy_file_range_verify(fd_in, &off_in,
-					fd_out, &off_out, copy_ret);
+			verify_ret = stress_copy_file_range_verify(fd_in, &off64_in,
+					fd_out, &off64_out, copy_ret);
 			if (verify_ret < 0) {
-				pr_fail("%s: copy_file_range verify failed, input offset=%jd, output offset=%jd\n",
-					args->name, (intmax_t)off_in_orig, (intmax_t)off_out_orig);
+				pr_fail("%s: copy_file_range verify failed, input offset=%" PRIdMAX " output offset=%" PRIdMAX "\n",
+					args->name, (intmax_t)off64_in_orig, (intmax_t)off64_out_orig);
 			}
 		}
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 
 		/*
 		 *  Exercise with bad fds
 		 */
-		VOID_RET(ssize_t, shim_copy_file_range(fd_bad, &off_in, fd_out,
-						&off_out, DEFAULT_COPY_FILE_SIZE, 0));
-		VOID_RET(ssize_t, shim_copy_file_range(fd_in, &off_in, fd_bad,
-						&off_out, DEFAULT_COPY_FILE_SIZE, 0));
-		VOID_RET(ssize_t, shim_copy_file_range(fd_out, &off_in, fd_in,
-						&off_out, DEFAULT_COPY_FILE_SIZE, 0));
+		VOID_RET(ssize_t, shim_copy_file_range(fd_bad, &off64_in, fd_out,
+						&off64_out, DEFAULT_COPY_FILE_SIZE, 0));
+		VOID_RET(ssize_t, shim_copy_file_range(fd_in, &off64_in, fd_bad,
+						&off64_out, DEFAULT_COPY_FILE_SIZE, 0));
+		VOID_RET(ssize_t, shim_copy_file_range(fd_out, &off64_in, fd_in,
+						&off64_out, DEFAULT_COPY_FILE_SIZE, 0));
 		(void)copy_ret;
 		/*
 		 *  Exercise with bad flags
 		 */
-		VOID_RET(ssize_t, shim_copy_file_range(fd_in, &off_in, fd_out,
-						&off_out, DEFAULT_COPY_FILE_SIZE, ~0U));
-		if (!keep_stressing(args))
+		VOID_RET(ssize_t, shim_copy_file_range(fd_in, &off64_in, fd_out,
+						&off64_out, DEFAULT_COPY_FILE_SIZE, ~0U));
+		if (!stress_continue(args))
 			break;
 		(void)shim_fsync(fd_out);
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 	rc = EXIT_SUCCESS;
 
 	rate = (duration > 0.0) ? bytes / duration : 0.0;
-	stress_metrics_set(args, 0, "MB per sec copy rate", rate / (double)MB);
+	stress_metrics_set(args, 0, "MB per sec copy rate",
+		rate / (double)MB, STRESS_METRIC_HARMONIC_MEAN);
 
 tidy_out:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -304,18 +319,19 @@ tidy_dir:
 	return rc;
 }
 
-stressor_info_t stress_copy_file_info = {
+const stressor_info_t stress_copy_file_info = {
 	.stressor = stress_copy_file,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
-stressor_info_t stress_copy_file_info = {
+const stressor_info_t stress_copy_file_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
+	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "built without clone() system call"
 };
