@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,7 +29,6 @@
 #define	STRESS_FILENAME_PROBE	(0)	/* Default */
 #define STRESS_FILENAME_POSIX	(1)	/* POSIX 2008.1 */
 #define STRESS_FILENAME_EXT	(2)	/* EXT* filesystems */
-#define STRESS_FILENAME_UNDEF	(0xff)	/* Undefined */
 
 typedef struct {
 	const uint8_t opt;
@@ -40,7 +39,6 @@ static const stress_filename_opts_t filename_opts[] = {
 	{ STRESS_FILENAME_PROBE,	"probe" },
 	{ STRESS_FILENAME_POSIX,	"posix" },
 	{ STRESS_FILENAME_EXT,		"ext" },
-	{ STRESS_FILENAME_UNDEF,	NULL }
 };
 
 static const stress_help_t help[] = {
@@ -57,30 +55,11 @@ static char allowed[256];
  * The Open Group Base Specifications Issue 7
  * POSIX.1-2008, 3.278 Portable Filename Character Set
  */
-static char posix_allowed[] =
+static const char posix_allowed[] =
 	"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 	"abcdefghijklmnopqrstuvwxyz"
 	"0123456789._-";
 
-static int stress_set_filename_opts(const char *opt)
-{
-	size_t i;
-
-	for (i = 0; filename_opts[i].opt_text; i++) {
-		if (!strcmp(opt, filename_opts[i].opt_text)) {
-			uint8_t filename_opt = filename_opts[i].opt;
-			stress_set_setting("filename-opts", TYPE_ID_UINT8, &filename_opt);
-			return 0;
-		}
-	}
-	(void)fprintf(stderr, "filename-opts option '%s' not known, options are:", opt);
-	for (i = 0; filename_opts[i].opt_text; i++)
-		(void)fprintf(stderr, "%s %s",
-			i == 0 ? "" : ",", filename_opts[i].opt_text);
-	(void)fprintf(stderr, "\n");
-	return -1;
-}
-
 /*
  *  stress_filename_probe_length()
  *	see if advertised path size is ok, shorten if
@@ -102,6 +81,8 @@ static int stress_filename_probe_length(
 		*(ptr + i + 1) = '\0';
 
 		if ((fd = creat(filename, S_IRUSR | S_IWUSR)) < 0) {
+			if (errno == ENOTSUP)
+				break;
 			if (errno == ENAMETOOLONG)
 				break;
 			pr_err("%s: creat() failed when probing "
@@ -163,7 +144,7 @@ static int stress_filename_probe(
 			 */
 			if ((errno != EINVAL) &&
 			    (errno != ENOENT) &&
-			    (errno == ENAMETOOLONG) &&
+			    ((errno == ENAMETOOLONG) || (errno == ENOTSUP)) &&
 			    (errno != EILSEQ)) {
 				pr_err("%s: creat() failed when probing "
 					"for allowed filename characters, "
@@ -263,6 +244,7 @@ static void stress_filename_generate_random(
 
 	for (i = 0; i < sz_max; i++) {
 		const size_t j = (size_t)stress_mwc32modn(chars_allowed);
+
 		filename[i] = allowed[j];
 	}
 	if (*filename == '.')
@@ -281,7 +263,8 @@ static void stress_filename_test(
 	const char *filename,
 	const size_t sz_max,
 	const bool should_pass,
-	const pid_t pid)
+	const pid_t pid,
+	int *rc)
 {
 	int fd;
 	int ret;
@@ -289,17 +272,22 @@ static void stress_filename_test(
 
 	/* exercise dcache lookup of non-existent filename */
 	ret = shim_stat(filename, &buf);
-	if (ret == 0)
+	if (ret == 0) {
 		pr_fail("%s: stat succeeded on non-existent file\n",
 			args->name);
+		*rc = EXIT_FAILURE;
+	}
 
 	if ((fd = creat(filename, S_IRUSR | S_IWUSR)) < 0) {
+		if (errno == ENOTSUP)
+			return;
 		if ((!should_pass) && (errno == ENAMETOOLONG))
 			return;
 
-		pr_fail("%s: open failed on file of length "
+		pr_fail("%s: creat() failed on file of length "
 			"%zu bytes, errno=%d (%s)\n",
 			args->name, sz_max, errno, strerror(errno));
+		*rc = EXIT_FAILURE;
 	} else {
 		stress_read_fdinfo(pid, fd);
 		(void)close(fd);
@@ -312,9 +300,11 @@ static void stress_filename_test(
 
 	/* exercise dcache lookup of non-existent filename */
 	ret = shim_stat(filename, &buf);
-	if (ret == 0)
+	if (ret == 0) {
 		pr_fail("%s: stat succeeded on non-existent unlinked file\n",
 			args->name);
+		*rc = EXIT_FAILURE;
+	}
 }
 
 /*
@@ -323,7 +313,7 @@ static void stress_filename_test(
  */
 static int stress_filename(stress_args_t *args)
 {
-	int ret, rc = EXIT_FAILURE;
+	int ret, rc = EXIT_SUCCESS;
 	size_t sz_left, sz_max;
 	char pathname[PATH_MAX - 256];
 	char filename[PATH_MAX];
@@ -421,6 +411,8 @@ static int stress_filename(stress_args_t *args)
 		goto tidy_dir;
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
 	if (!stress_continue_flag()) {
@@ -471,6 +463,8 @@ again:
 					goto again;
 				}
 			}
+		} else if (WIFEXITED(status)) {
+			rc = WEXITSTATUS(status);
 		}
 	} else {
 		/* Child, wrapped to catch OOMs */
@@ -494,72 +488,83 @@ again:
 
 			/* Should succeed */
 			stress_filename_generate(ptr, 1, ch);
-			stress_filename_test(args, filename, 1, true, mypid);
+			stress_filename_test(args, filename, 1, true, mypid, &rc);
 			if (!stress_continue(args))
 				break;
 			stress_filename_generate_random(ptr, 1, chars_allowed);
-			stress_filename_test(args, filename, 1, true, mypid);
+			stress_filename_test(args, filename, 1, true, mypid, &rc);
 			if (!stress_continue(args))
 				break;
 
 			/* Should succeed */
 			stress_filename_generate(ptr, sz_max, ch);
-			stress_filename_test(args, filename, sz_max, true, mypid);
+			stress_filename_test(args, filename, sz_max, true, mypid, &rc);
 			if (!stress_continue(args))
 				break;
 			stress_filename_generate_random(ptr, sz_max, chars_allowed);
-			stress_filename_test(args, filename, sz_max, true, mypid);
+			stress_filename_test(args, filename, sz_max, true, mypid, &rc);
 			if (!stress_continue(args))
 				break;
 
 			/* Should succeed */
 			stress_filename_generate(ptr, sz_max - 1, ch);
-			stress_filename_test(args, filename, sz_max - 1, true, mypid);
+			stress_filename_test(args, filename, sz_max - 1, true, mypid, &rc);
 			if (!stress_continue(args))
 				break;
 			stress_filename_generate_random(ptr, sz_max - 1, chars_allowed);
-			stress_filename_test(args, filename, sz_max - 1, true, mypid);
+			stress_filename_test(args, filename, sz_max - 1, true, mypid, &rc);
 			if (!stress_continue(args))
 				break;
 
 			/* Should fail */
 			stress_filename_generate(ptr, sz_max + 1, ch);
-			stress_filename_test(args, filename, sz_max + 1, false, mypid);
+			stress_filename_test(args, filename, sz_max + 1, false, mypid, &rc);
 			if (!stress_continue(args))
 				break;
 			stress_filename_generate_random(ptr, sz_max + 1, chars_allowed);
-			stress_filename_test(args, filename, sz_max + 1, false, mypid);
+			stress_filename_test(args, filename, sz_max + 1, false, mypid, &rc);
 			if (!stress_continue(args))
 				break;
 
 			/* Should succeed */
 			stress_filename_generate(ptr, sz, ch);
-			stress_filename_test(args, filename, sz, true, mypid);
+			stress_filename_test(args, filename, sz, true, mypid, &rc);
 			if (!stress_continue(args))
 				break;
 			stress_filename_generate_random(ptr, sz, chars_allowed);
-			stress_filename_test(args, filename, sz, true, mypid);
+			stress_filename_test(args, filename, sz, true, mypid, &rc);
 			if (!stress_continue(args))
 				break;
 
 			/* Should succeed */
 			stress_filename_generate(ptr, rnd_sz, ch);
-			stress_filename_test(args, filename, rnd_sz, true, mypid);
+			stress_filename_test(args, filename, rnd_sz, true, mypid, &rc);
 			if (!stress_continue(args))
 				break;
 			stress_filename_generate_random(ptr, rnd_sz, chars_allowed);
-			stress_filename_test(args, filename, rnd_sz, true, mypid);
+			stress_filename_test(args, filename, rnd_sz, true, mypid, &rc);
 			if (!stress_continue(args))
 				break;
 
+#if defined(HAVE_PATHCONF)
+#if defined(_PC_NAME_MAX)
+			VOID_RET(long int, pathconf(pathname, _PC_NAME_MAX));
+#endif
+#if defined(_PC_PATH_MAX)
+			VOID_RET(long int, pathconf(pathname, _PC_PATH_MAX));
+#endif
+#if defined(_PC_NO_TRUNC)
+			VOID_RET(long int, pathconf(pathname, _PC_NO_TRUNC));
+#endif
+#endif
+
 			sz++;
 			if (sz > sz_max)
 				sz = 1;
 			stress_bogo_inc(args);
 		} while (stress_continue(args));
-		_exit(EXIT_SUCCESS);
+		_exit(rc);
 	}
-	rc = EXIT_SUCCESS;
 
 tidy_dir:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -568,15 +573,20 @@ tidy_dir:
 	return rc;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_filename_opts,	stress_set_filename_opts },
-	{ 0,			NULL }
+static const char *stress_filename_opts(const size_t i)
+{
+	return (i < SIZEOF_ARRAY(filename_opts)) ? filename_opts[i].opt_text : NULL;
+}
+
+static const stress_opt_t opts[] = {
+	{ OPT_filename_opts, "filename-opts", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_filename_opts },
+	END_OPT,
 };
 
-stressor_info_t stress_filename_info = {
+const stressor_info_t stress_filename_info = {
 	.stressor = stress_filename,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
