@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,7 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
+#include "core-killpid.h"
 #include "core-net.h"
 
 #if defined(HAVE_SYS_EPOLL_H)
@@ -64,47 +66,53 @@ struct shim_kcmp_epoll_slot {
 
 #define SHIM_KCMP(pid1, pid2, type, idx1, idx2)				\
 	(int)shim_kcmp((pid_t)pid1, (pid_t)pid2, (int)type,		\
-		  (unsigned long)idx1, (unsigned long)idx2)		\
+		  (unsigned long int)idx1, (unsigned long int)idx2)	\
 
 #define KCMP(pid1, pid2, type, idx1, idx2)				\
 do {									\
-	int rc = SHIM_KCMP(pid1, pid2, type, idx1, idx2);		\
+	const int rc = SHIM_KCMP(pid1, pid2, type, idx1, idx2);		\
 									\
-	if (rc < 0) {	 						\
+	if (UNLIKELY(rc < 0)) {	 					\
 		if (errno == EPERM) {					\
 			pr_inf("%s: %s", capfail, args->name);		\
 			goto reap;					\
 		}							\
-		if ((errno != EINVAL) && (errno != ENOSYS))		\
-			pr_fail("%s kcmp " # type " failed, "		\
+		if ((errno != EINVAL) &&				\
+		    (errno != ENOSYS) &&				\
+		    (errno != EBADF)) {					\
+			pr_fail("%s: kcmp " # type " failed, "		\
 				"errno=%d (%s)\n", args->name,		\
 				errno, strerror(errno));		\
+			ret = EXIT_FAILURE;				\
+		}							\
 	}								\
-	if (!keep_stressing_flag())					\
-		goto reap;						\
 } while (0)
 
 #define KCMP_VERIFY(pid1, pid2, type, idx1, idx2, res)			\
 do {									\
-	int rc = SHIM_KCMP(pid1, pid2, type, idx1, idx2);		\
+	const int rc = SHIM_KCMP(pid1, pid2, type, idx1, idx2);		\
 									\
-	if (rc != res) {						\
+	if (UNLIKELY(rc != res)) {					\
 		if (rc < 0) {						\
 			if (errno == EPERM) {				\
 				pr_inf("%s: %s", capfail, args->name);	\
 				goto reap;				\
 			}						\
-			if ((errno != EINVAL) && (errno != ENOSYS))	\
-				pr_fail("%s kcmp " # type " failed, "	\
+			if ((errno != EINVAL) &&			\
+			    (errno != ENOSYS) &&			\
+			    (errno != EBADF)) {				\
+				pr_fail("%s: kcmp " # type " failed, "	\
 					"errno=%d (%s)\n", args->name,	\
 					errno, strerror(errno));	\
+				ret = EXIT_FAILURE;			\
+			}						\
 		} else {						\
 			pr_fail( "%s: kcmp " # type			\
 			" returned %d, expected: %d\n",			\
 			args->name, rc, ret);				\
 		}							\
 	}								\
-	if (!keep_stressing_flag())					\
+	if (UNLIKELY(!stress_continue_flag()))				\
 		goto reap;						\
 } while (0)
 
@@ -112,14 +120,14 @@ do {									\
  *  stress_kcmp
  *	stress sys_kcmp
  */
-static int stress_kcmp(const stress_args_t *args)
+static int stress_kcmp(stress_args_t *args)
 {
 	pid_t pid1;
 	int fd1;
 
 #if defined(HAVE_SYS_EPOLL_H) &&	\
     NEED_GLIBC(2,3,2)
-	int efd, sfd;
+	int efd = -1, sfd = -1;
 	int so_reuseaddr = 1;
 	struct epoll_event ev;
 	struct sockaddr *addr = NULL;
@@ -129,6 +137,10 @@ static int stress_kcmp(const stress_args_t *args)
 	int ret = EXIT_SUCCESS;
 	const int bad_fd = stress_get_bad_fd();
 	const bool is_root = stress_check_capability(SHIM_CAP_IS_ROOT);
+#if defined(HAVE_SYS_EPOLL_H) &&	\
+    NEED_GLIBC(2,3,2)
+	int port = 23000, reserved_port;
+#endif
 
 	static const char *capfail =
 		"need CAP_SYS_PTRACE capability to run kcmp stressor, "
@@ -142,62 +154,65 @@ static int stress_kcmp(const stress_args_t *args)
 
 #if defined(HAVE_SYS_EPOLL_H) &&	\
     NEED_GLIBC(2,3,2)
-	efd = -1;
-	if ((sfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
-		sfd = -1;
-		goto again;
-	}
-	if (setsockopt(sfd, SOL_SOCKET, SO_REUSEADDR,
-			&so_reuseaddr, sizeof(so_reuseaddr)) < 0) {
-		(void)close(sfd);
-		sfd = -1;
-		goto again;
-	}
-	if (stress_set_sockaddr(args->name, args->instance, mypid,
-		AF_INET, 23000, &addr, &addr_len, NET_ADDR_ANY) < 0) {
-		(void)close(sfd);
-		sfd = -1;
-		goto again;
-	}
+	reserved_port = stress_net_reserve_ports(port, port);
 
-	if (bind(sfd, addr, addr_len) < 0) {
-		(void)close(sfd);
-		sfd = -1;
-		goto again;
-	}
-	if (listen(sfd, SOMAXCONN) < 0) {
-		(void)close(sfd);
-		sfd = -1;
-		goto again;
-	}
-
-	efd = epoll_create1(0);
-	if (efd < 0) {
-		(void)close(sfd);
-		sfd = -1;
-		efd = -1;
-		goto again;
-	}
-
-	(void)memset(&ev, 0, sizeof(ev));
-	ev.data.fd = efd;
-	ev.events = EPOLLIN | EPOLLET;
-	if (epoll_ctl(efd, EPOLL_CTL_ADD, sfd, &ev) < 0) {
-		(void)close(sfd);
-		(void)close(efd);
-		sfd = -1;
+	if (reserved_port >= 0) {
 		efd = -1;
+		if ((sfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+			sfd = -1;
+			goto again;
+		}
+		if (setsockopt(sfd, SOL_SOCKET, SO_REUSEADDR,
+				&so_reuseaddr, sizeof(so_reuseaddr)) < 0) {
+			(void)close(sfd);
+			sfd = -1;
+			goto again;
+		}
+		if (stress_set_sockaddr(args->name, args->instance, mypid,
+					AF_INET, reserved_port, &addr, &addr_len, NET_ADDR_ANY) < 0) {
+			(void)close(sfd);
+			sfd = -1;
+			goto again;
+		}
+		if (bind(sfd, addr, addr_len) < 0) {
+			(void)close(sfd);
+			sfd = -1;
+			goto again;
+		}
+		if (listen(sfd, SOMAXCONN) < 0) {
+			(void)close(sfd);
+			sfd = -1;
+			goto again;
+		}
+		efd = epoll_create1(0);
+		if (efd < 0) {
+			(void)close(sfd);
+			sfd = -1;
+			efd = -1;
+			goto again;
+		}
+		(void)shim_memset(&ev, 0, sizeof(ev));
+		ev.data.fd = efd;
+		ev.events = EPOLLIN | EPOLLET;
+		if (epoll_ctl(efd, EPOLL_CTL_ADD, sfd, &ev) < 0) {
+			(void)close(sfd);
+			(void)close(efd);
+			sfd = -1;
+			efd = -1;
+		}
 	}
 #endif
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
 	pid1 = fork();
 	if (pid1 < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 		(void)close(fd1);
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			goto finish;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -212,7 +227,7 @@ again:
 		(void)sched_settings_apply(true);
 
 		/* Child */
-		while (keep_stressing_flag())
+		while (stress_continue_flag())
 			(void)pause();
 
 		/* will never get here */
@@ -227,7 +242,8 @@ again:
 		_exit(EXIT_SUCCESS);
 	} else {
 		/* Parent */
-		int fd2, status, pid2;
+		int fd2, pid2;
+		const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 
 		pid2 = getpid();
 		if ((fd2 = open("/dev/null", O_WRONLY)) < 0) {
@@ -269,20 +285,20 @@ again:
 
 #if defined(HAVE_SYS_EPOLL_H) &&	\
     NEED_GLIBC(2,3,2)
-			if (efd != -1) {
+			if (LIKELY((sfd != -1) && (efd != -1))) {
 				struct shim_kcmp_epoll_slot slot;
 
 				slot.efd = (uint32_t)efd;
 				slot.tfd = (uint32_t)sfd;
 				slot.toff = (uint32_t)0;
-				KCMP(pid1, pid2, SHIM_KCMP_EPOLL_TFD, efd, (unsigned long)&slot);
-				KCMP(pid2, pid1, SHIM_KCMP_EPOLL_TFD, efd, (unsigned long)&slot);
-				KCMP(pid2, pid2, SHIM_KCMP_EPOLL_TFD, efd, (unsigned long)&slot);
+				KCMP(pid1, pid2, SHIM_KCMP_EPOLL_TFD, efd, (unsigned long int)&slot);
+				KCMP(pid2, pid1, SHIM_KCMP_EPOLL_TFD, efd, (unsigned long int)&slot);
+				KCMP(pid2, pid2, SHIM_KCMP_EPOLL_TFD, efd, (unsigned long int)&slot);
 			}
 #endif
 
 			/* Same simple checks */
-			if (g_opt_flags & OPT_FLAGS_VERIFY) {
+			if (verify) {
 				KCMP_VERIFY(pid1, pid1, SHIM_KCMP_FILE, fd1, fd1, 0);
 				KCMP_VERIFY(pid1, pid1, SHIM_KCMP_FILES, 0, 0, 0);
 				KCMP_VERIFY(pid1, pid1, SHIM_KCMP_FS, 0, 0, 0);
@@ -293,13 +309,13 @@ again:
 				KCMP_VERIFY(pid1, pid2, SHIM_KCMP_SYSVSEM, 0, 0, 0);
 #if defined(HAVE_SYS_EPOLL_H) &&	\
     NEED_GLIBC(2,3,2)
-				if (efd != -1) {
+				if ((sfd != -1) && (efd != -1)) {
 					struct shim_kcmp_epoll_slot slot;
 
 					slot.efd = (uint32_t)efd;
 					slot.tfd = (uint32_t)sfd;
 					slot.toff = (uint32_t)0;
-					KCMP(pid1, pid2, SHIM_KCMP_EPOLL_TFD, efd, (unsigned long)&slot);
+					KCMP(pid1, pid2, SHIM_KCMP_EPOLL_TFD, efd, (unsigned long int)&slot);
 				}
 #endif
 			}
@@ -314,13 +330,12 @@ again:
 			if (!is_root)
 				(void)SHIM_KCMP(1, pid2, SHIM_KCMP_SIGHAND, 0, 0);
 
-			inc_counter(args);
-		} while (keep_stressing(args));
+			stress_bogo_inc(args);
+		} while (stress_continue(args));
 reap:
 		if (fd2 >= 0)
 			(void)close(fd2);
-		(void)kill(pid1, SIGKILL);
-		(void)shim_waitpid(pid1, &status, 0);
+		(void)stress_kill_pid_wait(pid1, NULL);
 		(void)close(fd1);
 	}
 finish:
@@ -336,16 +351,18 @@ finish:
 	return ret;
 }
 
-stressor_info_t stress_kcmp_info = {
+const stressor_info_t stress_kcmp_info = {
 	.stressor = stress_kcmp,
 	.class = CLASS_OS,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
-stressor_info_t stress_kcmp_info = {
-	.stressor = stress_not_implemented,
+const stressor_info_t stress_kcmp_info = {
+	.stressor = stress_unimplemented,
 	.class = CLASS_OS,
-	.help = help
+	.verify = VERIFY_OPTIONAL,
+	.help = help,
+	.unimplemented_reason = "built without kcmp() system call support"
 };
 #endif
