@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"locka N",	"start N workers locking a file via advisory locks" },
@@ -64,7 +67,7 @@ static stress_locka_info_t *stress_locka_info_new(void)
 		locka_infos.free = new->next;
 		new->next = NULL;
 	} else {
-		new = calloc(1, sizeof(*new));
+		new = (stress_locka_info_t *)calloc(1, sizeof(*new));
 		if (!new)
 			return NULL;
 	}
@@ -130,12 +133,12 @@ static void stress_locka_info_free(void)
  *  stress_locka_unlock()
  *	pop oldest lock record off list and unlock it
  */
-static int stress_locka_unlock(const stress_args_t *args, const int fd)
+static int stress_locka_unlock(stress_args_t *args, const int fd)
 {
 	struct flock f;
 
 	/* Pop one off list */
-	if (!locka_infos.head)
+	if (UNLIKELY(!locka_infos.head))
 		return 0;
 
 	f.l_type = F_UNLCK;
@@ -146,7 +149,7 @@ static int stress_locka_unlock(const stress_args_t *args, const int fd)
 
 	stress_locka_info_head_remove();
 
-	if (fcntl(fd, F_SETLK, &f) < 0) {
+	if (UNLIKELY(fcntl(fd, F_SETLK, &f) < 0)) {
 		pr_fail("%s: fcntl F_SETLK failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return -1;
@@ -159,7 +162,7 @@ static int stress_locka_unlock(const stress_args_t *args, const int fd)
  *	hammer advisory lock/unlock to create some file lock contention
  */
 static int stress_locka_contention(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd)
 {
 	stress_mwc_reseed();
@@ -172,7 +175,7 @@ static int stress_locka_contention(
 		struct flock f;
 
 		if (locka_infos.length >= LOCK_MAX)
-			if (stress_locka_unlock(args, fd) < 0)
+			if (UNLIKELY(stress_locka_unlock(args, fd) < 0))
 				return -1;
 
 		len = (stress_mwc16() + 1) & 0xfff;
@@ -184,14 +187,16 @@ static int stress_locka_contention(
 		f.l_len = len;
 		f.l_pid = args->pid;
 
+		if (UNLIKELY(!stress_continue_flag()))
+			break;
 		rc = fcntl(fd, F_GETLK, &f);
-		if (rc < 0)
+		if (UNLIKELY(rc < 0))
 			continue;
 
 		/* Locked OK, add to lock list */
 
 		locka_info = stress_locka_info_new();
-		if (!locka_info) {
+		if (UNLIKELY(!locka_info)) {
 			pr_err("%s: calloc failed, out of memory\n", args->name);
 			return -1;
 		}
@@ -199,8 +204,8 @@ static int stress_locka_contention(
 		locka_info->len = len;
 		locka_info->pid = args->pid;
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	return 0;
 }
@@ -209,9 +214,9 @@ static int stress_locka_contention(
  *  stress_locka
  *	stress file locking via advisory locking
  */
-static int stress_locka(const stress_args_t *args)
+static int stress_locka(stress_args_t *args)
 {
-	int fd, ret = EXIT_FAILURE;
+	int fd, ret = EXIT_FAILURE, parent_cpu;
 	pid_t cpid = -1;
 	char filename[PATH_MAX];
 	char pathname[PATH_MAX];
@@ -219,7 +224,7 @@ static int stress_locka(const stress_args_t *args)
 	off_t offset;
 	ssize_t rc;
 
-	(void)memset(buffer, 0, sizeof(buffer));
+	(void)shim_memset(buffer, 0, sizeof(buffer));
 
 	/*
 	 *  There will be a race to create the directory
@@ -258,7 +263,7 @@ static int stress_locka(const stress_args_t *args)
 	}
 	for (offset = 0; offset < LOCK_FILE_SIZE; offset += sizeof(buffer)) {
 redo:
-		if (!keep_stressing_flag()) {
+		if (!stress_continue_flag()) {
 			ret = EXIT_SUCCESS;
 			goto tidy;
 		}
@@ -273,19 +278,23 @@ redo:
 		}
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
+	parent_cpu = stress_get_cpu();
 	cpid = fork();
 	if (cpid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			goto tidy;
 		pr_err("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto tidy;
 	}
 	if (cpid == 0) {
+		(void)stress_change_cpu(args, parent_cpu);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
@@ -300,12 +309,8 @@ again:
 tidy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	if (cpid > 0) {
-		int status;
-
-		(void)kill(cpid, SIGKILL);
-		(void)shim_waitpid(cpid, &status, 0);
-	}
+	if (cpid > 1)
+		stress_kill_and_wait(args, cpid, SIGALRM, true);
 	stress_locka_info_free();
 
 	(void)close(fd);
@@ -314,16 +319,17 @@ tidy:
 
 	return ret;
 }
-stressor_info_t stress_locka_info = {
+const stressor_info_t stress_locka_info = {
 	.stressor = stress_locka,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_locka_info = {
+const stressor_info_t stress_locka_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without fcntl() F_GETLK, F_SETLK, F_SETLKW, F_WRLCK or F_UNLCK commands"
 };
