@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,9 +19,16 @@
  */
 #include "stress-ng.h"
 #include "core-arch.h"
-#include "core-cache.h"
+#include "core-asm-x86.h"
+#include "core-builtin.h"
+#include "core-cpu-cache.h"
+#include "core-madvise.h"
+#include "core-nt-load.h"
 #include "core-nt-store.h"
+#include "core-numa.h"
+#include "core-out-of-memory.h"
 #include "core-pthread.h"
+#include "core-pragma.h"
 #include "core-target-clones.h"
 
 #if defined(HAVE_LINUX_MEMPOLICY_H) &&	\
@@ -30,6 +37,9 @@
 #define HAVE_MEMTHRASH_NUMA	(1)
 #endif
 
+#define BITS_PER_BYTE		(8)
+#define NUMA_LONG_BITS		(sizeof(unsigned long int) * BITS_PER_BYTE)
+
 static const stress_help_t help[] = {
 	{ NULL,	"memthrash N",		"start N workers thrashing a 16MB memory buffer" },
 	{ NULL,	"memthrash-method M",	"specify memthrash method M, default is all" },
@@ -49,23 +59,20 @@ static const stress_help_t help[] = {
 
 
 typedef struct {
-	const stress_args_t *args;
+	stress_args_t *args;
 	const struct stress_memthrash_method_info *memthrash_method;
 	uint32_t total_cpus;
 	uint32_t max_threads;
 #if defined(HAVE_MEMTHRASH_NUMA)
-	unsigned long numa_nodes;
-	unsigned long max_numa_nodes;
-	unsigned long *numa_node_mask;
-	size_t numa_node_mask_size;
+	stress_numa_mask_t *numa_mask;
 #endif
 } stress_memthrash_context_t;
 
 typedef void (*stress_memthrash_func_t)(const stress_memthrash_context_t *context, size_t mem_size);
 
 typedef struct stress_memthrash_method_info {
-	const char		*name;	/* human readable form of stressor */
-	stress_memthrash_func_t	func;	/* the method function */
+	const char		*name;		/* human readable form of stressor */
+	const stress_memthrash_func_t	func;	/* the method function */
 } stress_memthrash_method_info_t;
 
 /* Per-pthread information */
@@ -87,28 +94,21 @@ static sigset_t set;
 
 static stress_memthrash_primes_t stress_memthrash_primes[MEM_SIZE_PRIMES];
 
-#if (((defined(__GNUC__) || defined(__clang__)) && 	\
-       defined(STRESS_ARCH_X86)) ||			\
-     (defined(__GNUC__) && 				\
-      defined(HAVE_ATOMIC_ADD_FETCH) &&			\
-      defined(__ATOMIC_SEQ_CST) &&			\
-      NEED_GNUC(4,7,0) && 				\
+#if (((defined(HAVE_COMPILER_GCC_OR_MUSL) || defined(HAVE_COMPILER_CLANG)) &&	\
+       defined(STRESS_ARCH_X86)) ||						\
+     (defined(HAVE_COMPILER_GCC_OR_MUSL) && 					\
+      defined(HAVE_ATOMIC_ADD_FETCH) &&						\
+      defined(__ATOMIC_SEQ_CST) &&						\
+      NEED_GNUC(4,7,0) && 							\
       defined(STRESS_ARCH_ARM)))
-
 #if defined(HAVE_ATOMIC_ADD_FETCH)
-#define MEM_LOCK(ptr, inc) 				\
-do {							\
-	__atomic_add_fetch(ptr, inc, __ATOMIC_SEQ_CST);	\
-} while (0)
+#define MEM_LOCK(ptr, inc)	__atomic_add_fetch(ptr, inc, __ATOMIC_SEQ_CST)
 #else
-#define MEM_LOCK(ptr, inc)				\
-do {							\
-	asm volatile("lock addl %1,%0" : "+m" (*ptr) : "ir" (inc));	\
-} while (0);
+#define MEM_LOCK(ptr, inc)	stress_asm_x86_lock_add(ptr, inc)
 #endif
 #endif
 
-static inline HOT OPTIMIZE3 void stress_memthrash_random_chunk(
+static inline OPTIMIZE3 void stress_memthrash_random_chunk(
 	const size_t chunk_size,
 	const size_t mem_size)
 {
@@ -120,26 +120,22 @@ static inline HOT OPTIMIZE3 void stress_memthrash_random_chunk(
 		chunks = 1;
 
 	for (i = 0; !thread_terminate && (i < max); i++) {
-		const size_t chunk = stress_mwc32() % chunks;
+		const size_t chunk = stress_mwc32modn(chunks);
 		const size_t offset = chunk * chunk_size;
 		void *ptr = (void *)(((uint8_t *)mem) + offset);
 
-#if defined(__GNUC__)
-		(void)__builtin_memset(ptr, stress_mwc8(), chunk_size);
-#else
-		(void)memset(ptr, stress_mwc8(), chunk_size);
-#endif
+		(void)shim_memset(ptr, stress_mwc8(), chunk_size);
 	}
 }
 
-static void HOT OPTIMIZE3 stress_memthrash_random_chunkpage(
+static void OPTIMIZE3 stress_memthrash_random_chunkpage(
 	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	stress_memthrash_random_chunk(context->args->page_size, mem_size);
 }
 
-static void HOT OPTIMIZE3 stress_memthrash_random_chunk256(
+static void OPTIMIZE3 stress_memthrash_random_chunk256(
 	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
@@ -148,7 +144,7 @@ static void HOT OPTIMIZE3 stress_memthrash_random_chunk256(
 	stress_memthrash_random_chunk(256, mem_size);
 }
 
-static void HOT OPTIMIZE3 stress_memthrash_random_chunk64(
+static void OPTIMIZE3 stress_memthrash_random_chunk64(
 	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
@@ -157,7 +153,7 @@ static void HOT OPTIMIZE3 stress_memthrash_random_chunk64(
 	stress_memthrash_random_chunk(64, mem_size);
 }
 
-static void HOT OPTIMIZE3 stress_memthrash_random_chunk8(
+static void OPTIMIZE3 stress_memthrash_random_chunk8(
 	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
@@ -166,7 +162,7 @@ static void HOT OPTIMIZE3 stress_memthrash_random_chunk8(
 	stress_memthrash_random_chunk(8, mem_size);
 }
 
-static void HOT OPTIMIZE3 stress_memthrash_random_chunk1(
+static void OPTIMIZE3 stress_memthrash_random_chunk1(
 	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
@@ -181,13 +177,32 @@ static void stress_memthrash_memset(
 {
 	(void)context;
 
-#if defined(__GNUC__)
-	(void)__builtin_memset((void *)mem, stress_mwc8(), mem_size);
-#else
-	(void)memset((void *)mem, stress_mwc8(), mem_size);
-#endif
+	(void)shim_memset((void *)mem, stress_mwc8(), mem_size);
 }
 
+#if defined(HAVE_ASM_X86_REP_STOSD) &&	\
+    !defined(__ILP32__)
+static inline void OPTIMIZE3 stress_memtrash_memsetstosd(
+	const stress_memthrash_context_t *context,
+	const size_t mem_size)
+{
+	register void *p = (void *)mem;
+	register const uint32_t l = (uint32_t)(mem_size >> 2);
+
+	(void)context;
+
+	__asm__ __volatile__(
+		"mov $0x00000000,%%eax\n;"
+		"mov %0,%%rdi\n;"
+		"mov %1,%%ecx\n;"
+		"rep stosl %%eax,%%es:(%%rdi);\n"	/* gcc calls it stosl and not stosw */
+		:
+		: "r" (p),
+		  "r" (l)
+		: "ecx","rdi","eax");
+}
+#endif
+
 static void stress_memthrash_memmove(
 	const stress_memthrash_context_t *context,
 	const size_t mem_size)
@@ -195,14 +210,10 @@ static void stress_memthrash_memmove(
 	char *dst = ((char *)mem) + 1;
 
 	(void)context;
-#if defined(__GNUC__)
-	(void)shim_builtin_memmove((void *)dst, mem, mem_size - 1);
-#else
-	(void)memmove((void *)dst, mem, mem_size - 1);
-#endif
+	(void)shim_memmove((void *)dst, mem, mem_size - 1);
 }
 
-static void HOT OPTIMIZE3 stress_memthrash_memset64(
+static void OPTIMIZE3 stress_memthrash_memset64(
 	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
@@ -230,6 +241,7 @@ static void HOT OPTIMIZE3 stress_memthrash_memset64(
 #endif
 	/* normal temporal stores, non-SSE fallback */
 
+PRAGMA_UNROLL_N(4)
 	while (LIKELY(ptr < end)) {
 		*ptr++ = val;
 		*ptr++ = val;
@@ -262,7 +274,7 @@ static void OPTIMIZE3 TARGET_CLONES stress_memthrash_swap64(
 		r5 = ptr[5];
 		r6 = ptr[6];
 		r7 = ptr[7];
-		shim_mb();
+		stress_asm_mb();
 
 		ptr[0] = r4;
 		ptr[1] = r5;
@@ -272,7 +284,7 @@ static void OPTIMIZE3 TARGET_CLONES stress_memthrash_swap64(
 		ptr[5] = r1;
 		ptr[6] = r2;
 		ptr[7] = r3;
-		shim_mb();
+		stress_asm_mb();
 		ptr += 8;
 
 		r0 = ptr[0];
@@ -283,7 +295,7 @@ static void OPTIMIZE3 TARGET_CLONES stress_memthrash_swap64(
 		r5 = ptr[5];
 		r6 = ptr[6];
 		r7 = ptr[7];
-		shim_mb();
+		stress_asm_mb();
 
 		ptr[0] = r4;
 		ptr[1] = r5;
@@ -293,7 +305,7 @@ static void OPTIMIZE3 TARGET_CLONES stress_memthrash_swap64(
 		ptr[5] = r1;
 		ptr[6] = r2;
 		ptr[7] = r3;
-		shim_mb();
+		stress_asm_mb();
 		ptr += 8;
 	}
 }
@@ -304,7 +316,8 @@ static void OPTIMIZE3 TARGET_CLONES stress_memthrash_copy128(
 	const size_t mem_size)
 {
 	__uint128_t *ptr = (__uint128_t *)mem;
-	register const __uint128_t *end = (__uint128_t *)(((uintptr_t)mem) + mem_size - 128);
+	size_t end_offset = sizeof(*ptr) * 16;
+	register const __uint128_t *end = (__uint128_t *)(((uintptr_t)mem) + mem_size - end_offset);
 
 	(void)context;
 
@@ -327,32 +340,14 @@ static void OPTIMIZE3 TARGET_CLONES stress_memthrash_copy128(
 		ptr[5] = r5;
 		ptr[6] = r6;
 		ptr[7] = r7;
-		shim_mb();
-		ptr += 8;
+		stress_asm_mb();
 
-		r0 = ptr[8];
-		r1 = ptr[9];
-		r2 = ptr[10];
-		r3 = ptr[11];
-		r4 = ptr[12];
-		r5 = ptr[13];
-		r6 = ptr[14];
-		r7 = ptr[15];
-		ptr[0] = r0;
-		ptr[1] = r1;
-		ptr[2] = r2;
-		ptr[3] = r3;
-		ptr[4] = r4;
-		ptr[5] = r5;
-		ptr[6] = r6;
-		ptr[7] = r7;
-		shim_mb();
 		ptr += 8;
 	}
 }
 #endif
 
-static void HOT OPTIMIZE3 stress_memthrash_flip_mem(
+static void OPTIMIZE3 stress_memthrash_flip_mem(
 	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
@@ -367,13 +362,13 @@ static void HOT OPTIMIZE3 stress_memthrash_flip_mem(
 	}
 }
 
-static void HOT OPTIMIZE3 stress_memthrash_swap(
+static void OPTIMIZE3 stress_memthrash_swap(
 	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	size_t i;
-	register size_t offset1 = stress_mwc32() % mem_size;
-	register size_t offset2 = stress_mwc32() % mem_size;
+	register size_t offset1 = stress_mwc32modn(mem_size);
+	register size_t offset2 = stress_mwc32modn(mem_size);
 	uint8_t *mem_u8 = (uint8_t *)mem;
 
 	(void)context;
@@ -394,7 +389,7 @@ static void HOT OPTIMIZE3 stress_memthrash_swap(
 	}
 }
 
-static void HOT OPTIMIZE3 stress_memthrash_matrix(
+static void OPTIMIZE3 stress_memthrash_matrix(
 	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
@@ -417,7 +412,7 @@ static void HOT OPTIMIZE3 stress_memthrash_matrix(
 	}
 }
 
-static void HOT OPTIMIZE3 stress_memthrash_prefetch(
+static void OPTIMIZE3 stress_memthrash_prefetch(
 	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
@@ -427,7 +422,7 @@ static void HOT OPTIMIZE3 stress_memthrash_prefetch(
 	(void)context;
 
 	for (i = 0; !thread_terminate && (i < max); i++) {
-		size_t offset = stress_mwc32() % mem_size;
+		size_t offset = stress_mwc32modn(mem_size);
 		uint8_t *const ptr = ((uint8_t *)mem) + offset;
 		volatile uint8_t *const vptr = ptr;
 
@@ -438,7 +433,7 @@ static void HOT OPTIMIZE3 stress_memthrash_prefetch(
 }
 
 #if defined(HAVE_ASM_X86_CLFLUSH)
-static void HOT OPTIMIZE3 stress_memthrash_flush(
+static void OPTIMIZE3 stress_memthrash_flush(
 	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
@@ -448,7 +443,7 @@ static void HOT OPTIMIZE3 stress_memthrash_flush(
 	(void)context;
 
 	for (i = 0; !thread_terminate && (i < max); i++) {
-		size_t offset = stress_mwc32() % mem_size;
+		size_t offset = stress_mwc32modn(mem_size);
 		uint8_t *const ptr = ((uint8_t *)mem) + offset;
 		volatile uint8_t *const vptr = ptr;
 
@@ -458,7 +453,7 @@ static void HOT OPTIMIZE3 stress_memthrash_flush(
 }
 #endif
 
-static void HOT OPTIMIZE3 stress_memthrash_mfence(
+static void OPTIMIZE3 stress_memthrash_mfence(
 	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
@@ -468,7 +463,7 @@ static void HOT OPTIMIZE3 stress_memthrash_mfence(
 	(void)context;
 
 	for (i = 0; !thread_terminate && (i < max); i++) {
-		size_t offset = stress_mwc32() % mem_size;
+		size_t offset = stress_mwc32modn(mem_size);
 		volatile uint8_t *ptr = ((uint8_t *)mem) + offset;
 
 		*ptr = i & 0xff;
@@ -477,7 +472,7 @@ static void HOT OPTIMIZE3 stress_memthrash_mfence(
 }
 
 #if defined(MEM_LOCK)
-static void HOT OPTIMIZE3 stress_memthrash_lock(
+static void OPTIMIZE3 stress_memthrash_lock(
 	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
@@ -486,7 +481,7 @@ static void HOT OPTIMIZE3 stress_memthrash_lock(
 	(void)context;
 
 	for (i = 0; !thread_terminate && (i < 64); i++) {
-		size_t offset = stress_mwc32() % mem_size;
+		size_t offset = stress_mwc32modn(mem_size);
 		volatile uint8_t *ptr = ((uint8_t *)mem) + offset;
 
 		MEM_LOCK(ptr, 1);
@@ -494,16 +489,43 @@ static void HOT OPTIMIZE3 stress_memthrash_lock(
 }
 #endif
 
-static void HOT OPTIMIZE3 stress_memthrash_spinread(
+static void OPTIMIZE3 stress_memthrash_spinread(
 	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	uint32_t i;
 	volatile uint32_t *ptr;
 	const size_t size = mem_size - (8 * sizeof(*ptr));
-	const size_t offset = (stress_mwc32() % size) & ~(size_t)3;
+	const size_t offset = stress_mwc32modn(size) & ~(size_t)3;
 
 	(void)context;
+
+#if defined(HAVE_NT_LOAD32)
+	if (stress_cpu_x86_has_sse2()) {
+		uint32_t *nt_ptr = (uint32_t *)(((uintptr_t)mem) + offset);
+
+		for (i = 0; !thread_terminate && (i < 65536); i++) {
+			(void)stress_nt_load32(nt_ptr);
+			stress_asm_mb();
+			(void)stress_nt_load32(nt_ptr);
+			stress_asm_mb();
+			(void)stress_nt_load32(nt_ptr);
+			stress_asm_mb();
+			(void)stress_nt_load32(nt_ptr);
+			stress_asm_mb();
+
+			(void)stress_nt_load32(nt_ptr);
+			stress_asm_mb();
+			(void)stress_nt_load32(nt_ptr);
+			stress_asm_mb();
+			(void)stress_nt_load32(nt_ptr);
+			stress_asm_mb();
+			(void)stress_nt_load32(nt_ptr);
+			stress_asm_mb();
+		}
+		return;
+	}
+#endif
 	ptr = (uint32_t *)(((uintptr_t)mem) + offset);
 
 	for (i = 0; !thread_terminate && (i < 65536); i++) {
@@ -519,16 +541,43 @@ static void HOT OPTIMIZE3 stress_memthrash_spinread(
 	}
 }
 
-static void HOT OPTIMIZE3 stress_memthrash_spinwrite(
+static void OPTIMIZE3 stress_memthrash_spinwrite(
 	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	uint32_t i;
 	volatile uint32_t *ptr;
 	const size_t size = mem_size - (8 * sizeof(*ptr));
-	const size_t offset = (stress_mwc32() % size) & ~(size_t)3;
+	const size_t offset = stress_mwc32modn(size) & ~(size_t)3;
 
 	(void)context;
+
+#if defined(HAVE_NT_STORE32)
+	if (stress_cpu_x86_has_sse2()) {
+		uint32_t *nt_ptr = (uint32_t *)(((uintptr_t)mem) + offset);
+
+		for (i = 0; !thread_terminate && (i < 65536); i++) {
+			stress_nt_store32(nt_ptr, i);
+			stress_asm_mb();
+			stress_nt_store32(nt_ptr, i);
+			stress_asm_mb();
+			stress_nt_store32(nt_ptr, i);
+			stress_asm_mb();
+			stress_nt_store32(nt_ptr, i);
+			stress_asm_mb();
+
+			stress_nt_store32(nt_ptr, i);
+			stress_asm_mb();
+			stress_nt_store32(nt_ptr, i);
+			stress_asm_mb();
+			stress_nt_store32(nt_ptr, i);
+			stress_asm_mb();
+			stress_nt_store32(nt_ptr, i);
+			stress_asm_mb();
+		}
+		return;
+	}
+#endif
 	ptr = (uint32_t *)(((uintptr_t)mem) + offset);
 
 	for (i = 0; !thread_terminate && (i < 65536); i++) {
@@ -544,7 +593,7 @@ static void HOT OPTIMIZE3 stress_memthrash_spinwrite(
 	}
 }
 
-static void HOT OPTIMIZE3 stress_memthrash_tlb(
+static void OPTIMIZE3 stress_memthrash_tlb(
 	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
@@ -584,7 +633,8 @@ static void OPTIMIZE3 TARGET_CLONES stress_memthrash_swapfwdrev(
 	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
-	uint64_t *fwd, *rev, *end = (uint64_t *)((uintptr_t)mem + mem_size);
+	register uint64_t *fwd, *rev;
+	uint64_t *const end = (uint64_t *)((uintptr_t)mem + mem_size);
 
 	(void)context;
 	for (fwd = (uint64_t *)mem, rev = end - 1; fwd < end; rev--, fwd++) {
@@ -603,6 +653,24 @@ static void OPTIMIZE3 TARGET_CLONES stress_memthrash_swapfwdrev(
 	}
 }
 
+static void OPTIMIZE3 TARGET_CLONES stress_memthrash_reverse(
+	const stress_memthrash_context_t *context,
+	const size_t mem_size)
+{
+	register uint8_t *fwd = (uint8_t *)mem;
+	register uint8_t *end = (uint8_t *)mem + mem_size;
+	register uint8_t *rev = end;
+
+	(void)context;
+	while (fwd < end) {
+		register uint8_t tmp;
+
+		tmp = *fwd;
+		*(fwd++) = *(--rev);
+		*rev = tmp;
+	}
+}
+
 #if defined(HAVE_MEMTHRASH_NUMA)
 static void OPTIMIZE3 TARGET_CLONES stress_memthrash_numa(
 	const stress_memthrash_context_t *context,
@@ -611,21 +679,25 @@ static void OPTIMIZE3 TARGET_CLONES stress_memthrash_numa(
 	uint8_t *ptr;
 	const uint8_t *end = (uint8_t *)((uintptr_t)mem + mem_size);
 	const size_t page_size = context->args->page_size;
-	unsigned long node;
+	stress_numa_mask_t *numa_mask = context->numa_mask;
+	unsigned long int node;
 
-	if (context->numa_nodes < 1)
+	if (!numa_mask)
 		return;
 
-	node = (unsigned long)stress_mwc32() % context->numa_nodes;
-	(void)memset(context->numa_node_mask, 0, context->numa_node_mask_size);
+	node = (unsigned long int)stress_mwc32modn((uint32_t)numa_mask->nodes);
+	(void)shim_memset(numa_mask->mask, 0, numa_mask->mask_size);
 
 	for (ptr = (uint8_t *)mem; ptr < end; ptr += page_size) {
-		STRESS_SETBIT(context->numa_node_mask, node);
+		STRESS_SETBIT(numa_mask->mask, (unsigned long int)node);
 
-		(void)shim_mbind((void *)ptr, page_size, MPOL_PREFERRED, context->numa_node_mask, context->max_numa_nodes, 0);
-		STRESS_CLRBIT(context->numa_node_mask, node);
+		if (stress_mwc1())
+			(void)shim_mbind((void *)ptr, page_size, MPOL_PREFERRED, numa_mask->mask, numa_mask->max_nodes, 0);
+		else
+			(void)shim_mbind((void *)ptr, page_size, MPOL_BIND, numa_mask->mask, numa_mask->max_nodes, MPOL_MF_MOVE);
+		STRESS_CLRBIT(numa_mask->mask, (unsigned long int)node);
 		node++;
-		if (node >= context->numa_nodes)
+		if (node >= numa_mask->nodes)
 			node = 0;
 	}
 }
@@ -656,12 +728,17 @@ static const stress_memthrash_method_info_t memthrash_methods[] = {
 	{ "memmove",	stress_memthrash_memmove },
 	{ "memset",	stress_memthrash_memset },
 	{ "memset64",	stress_memthrash_memset64 },
+#if defined(HAVE_ASM_X86_REP_STOSD) &&	\
+    !defined(__ILP32__)
+	{ "memsetstosd",stress_memtrash_memsetstosd },
+#endif
 	{ "mfence",	stress_memthrash_mfence },
 #if defined(HAVE_MEMTHRASH_NUMA)
 	{ "numa",	stress_memthrash_numa },
 #endif
 	{ "prefetch",	stress_memthrash_prefetch },
 	{ "random",	stress_memthrash_random },
+	{ "reverse",	stress_memthrash_reverse },
 	{ "spinread",	stress_memthrash_spinread },
 	{ "spinwrite",	stress_memthrash_spinwrite },
 	{ "swap",	stress_memthrash_swap },
@@ -688,7 +765,7 @@ static void stress_memthrash_random(const stress_memthrash_context_t *context, s
 {
 	/* loop until we find a good candidate */
 	for (;;) {
-		size_t i = stress_mwc8() % SIZEOF_ARRAY(memthrash_methods);
+		size_t i = stress_mwc8modn((uint8_t)SIZEOF_ARRAY(memthrash_methods));
 		const stress_memthrash_func_t func = (stress_memthrash_func_t)memthrash_methods[i].func;
 
 		/* Don't run stress_memthrash_random/all to avoid recursion */
@@ -700,31 +777,6 @@ static void stress_memthrash_random(const stress_memthrash_context_t *context, s
 	}
 }
 
-/*
- *  stress_set_memthrash_method()
- *	set the default memthresh method
- */
-static int stress_set_memthrash_method(const char *name)
-{
-	size_t i;
-
-	for (i = 0; i < SIZEOF_ARRAY(memthrash_methods); i++) {
-		const stress_memthrash_method_info_t *info = &memthrash_methods[i];
-		if (!strcmp(memthrash_methods[i].name, name)) {
-			stress_set_setting("memthrash-method", TYPE_ID_UINTPTR_T, &info);
-			return 0;
-		}
-	}
-
-	(void)fprintf(stderr, "memthrash-method must be one of:");
-	for (i = 0; i < SIZEOF_ARRAY(memthrash_methods); i++) {
-		(void)fprintf(stderr, " %s", memthrash_methods[i].name);
-	}
-	(void)fprintf(stderr, "\n");
-
-	return -1;
-}
-
 static void stress_memthrash_find_primes(void)
 {
 	size_t i;
@@ -735,7 +787,7 @@ static void stress_memthrash_find_primes(void)
 
 		stress_memthrash_primes[i].mem_size = mem_size;
 		stress_memthrash_primes[i].prime_stride =
-			(size_t)stress_get_prime64((uint64_t)cache_lines) * STRESS_CACHE_LINE_SIZE;
+			(size_t)stress_get_next_prime64((uint64_t)cache_lines) * STRESS_CACHE_LINE_SIZE;
 	}
 }
 
@@ -744,10 +796,9 @@ static void stress_memthrash_find_primes(void)
  */
 static void *stress_memthrash_func(void *ctxt)
 {
-	static void *nowt = NULL;
 	const stress_memthrash_context_t *context = (stress_memthrash_context_t *)ctxt;
 	const stress_memthrash_func_t func = context->memthrash_method->func;
-	const stress_args_t *args = context->args;
+	stress_args_t *args = context->args;
 
 	/*
 	 *  Block all signals, let controlling thread
@@ -755,11 +806,11 @@ static void *stress_memthrash_func(void *ctxt)
 	 */
 	(void)sigprocmask(SIG_BLOCK, &set, NULL);
 
-	while (!thread_terminate && keep_stressing(args)) {
+	while (!thread_terminate && stress_continue(args)) {
 		size_t j;
 
 		for (j = MATRIX_SIZE_MIN_SHIFT; j <= MATRIX_SIZE_MAX_SHIFT &&
-		     !thread_terminate && keep_stressing(args); j++) {
+		     !thread_terminate && stress_continue(args); j++) {
 			size_t mem_size = 1 << (2 * j);
 			size_t i;
 
@@ -767,14 +818,11 @@ static void *stress_memthrash_func(void *ctxt)
 				if (func == memthrash_methods[i].func)
 					break;
 			func(context, mem_size);
-			inc_counter(args);
-			shim_sched_yield();
+			stress_bogo_inc(args);
+			(void)shim_sched_yield();
 		}
 	}
-
-	/* Wake parent up, all done! */
-	(void)kill(args->pid, SIGALRM);
-	return &nowt;
+	return &g_nowt;
 }
 
 static inline uint32_t stress_memthrash_max(
@@ -816,7 +864,7 @@ static void stress_memthrash_sigalrm_handler(int signum)
 	thread_terminate = true;
 }
 
-static int stress_memthrash_child(const stress_args_t *args, void *ctxt)
+static int stress_memthrash_child(stress_args_t *args, void *ctxt)
 {
 	stress_memthrash_context_t *context = (stress_memthrash_context_t *)ctxt;
 	const uint32_t max_threads = context->max_threads;
@@ -824,12 +872,7 @@ static int stress_memthrash_child(const stress_args_t *args, void *ctxt)
 	int ret;
 	stress_pthread_info_t *pthread_info;
 
-	int flags = MAP_PRIVATE | MAP_ANONYMOUS;
-#if defined(MAP_POPULATE)
-	flags |= MAP_POPULATE;
-#endif
-
-	pthread_info = calloc(max_threads, sizeof(*pthread_info));
+	pthread_info = (stress_pthread_info_t *)calloc(max_threads, sizeof(*pthread_info));
 	if (!pthread_info) {
 		pr_inf_skip("%s: failed to allocate pthread information array, skipping stressor\n",
 			args->name);
@@ -840,22 +883,22 @@ static int stress_memthrash_child(const stress_args_t *args, void *ctxt)
 
 
 mmap_retry:
-	mem = mmap(NULL, MEM_SIZE, PROT_READ | PROT_WRITE, flags, -1, 0);
+	mem = stress_mmap_populate(NULL, MEM_SIZE, PROT_READ | PROT_WRITE,
+			MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 	if (mem == MAP_FAILED) {
-#if defined(MAP_POPULATE)
-		flags &= ~MAP_POPULATE;	/* Less aggressive, more OOMable */
-#endif
-		if (!keep_stressing_flag()) {
+		if (!stress_continue_flag()) {
 			pr_dbg("%s: mmap failed: %d %s\n",
 				args->name, errno, strerror(errno));
 			free(pthread_info);
 			return EXIT_NO_RESOURCE;
 		}
 		(void)shim_usleep(100000);
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			goto reap_mem;
 		goto mmap_retry;
 	}
+	stress_set_vma_anon_name(mem, MEM_SIZE, "memthrash-data");
+	(void)stress_madvise_mergeable(mem, MEM_SIZE);
 
 	for (i = 0; i < max_threads; i++) {
 		pthread_info[i].ret = pthread_create(&pthread_info[i].pthread,
@@ -872,7 +915,7 @@ mmap_retry:
 				args->name, ret, strerror(ret));
 			goto reap;
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			goto reap;
 	}
 	/* Wait for SIGALRM or SIGINT/SIGHUP etc */
@@ -901,40 +944,35 @@ reap_mem:
  *  stress_memthrash()
  *	stress by creating pthreads
  */
-static int stress_memthrash(const stress_args_t *args)
+static int stress_memthrash(stress_args_t *args)
 {
 	stress_memthrash_context_t context;
+	size_t memthrash_method = 0;
 	int rc;
 
+	if (stress_sigchld_set_handler(args) < 0)
+		return EXIT_NO_RESOURCE;
+
 	stress_memthrash_find_primes();
 
 	context.args = args;
 	context.total_cpus = (uint32_t)stress_get_processors_online();
-	context.memthrash_method = &memthrash_methods[0];
 	context.max_threads = stress_memthrash_max(args->num_instances, context.total_cpus);
 #if defined(HAVE_MEMTHRASH_NUMA)
 	{
-		const size_t numa_bits = sizeof(unsigned long) * 8;
-		size_t numa_elements;
-
-		context.numa_nodes = stress_numa_count_mem_nodes(&context.max_numa_nodes);
-
-		numa_elements = (context.max_numa_nodes + numa_bits - 1) / numa_bits;
-		numa_elements = numa_elements ? numa_elements: 1;
-		context.numa_node_mask = calloc(context.max_numa_nodes, numa_elements);
-		context.numa_node_mask_size = (size_t)context.max_numa_nodes * numa_elements;
-		if (!context.numa_node_mask) {
-			pr_inf_skip("%s: could not allocate %zd numa elements in numa mask, skipping stressor\n",
-				args->name, numa_elements);
-			return EXIT_NO_RESOURCE;
+		context.numa_mask = stress_numa_mask_alloc();
+		if (!context.numa_mask) {
+			if (args->instance == 0)
+				pr_inf("%s: no NUMA nodes or maximum NUMA nodes, ignoring numa memthrash method\n", args->name);
 		}
 	}
 #endif
 
-	(void)stress_get_setting("memthrash-method", &context.memthrash_method);
+	(void)stress_get_setting("memthrash-method", &memthrash_method);
+	context.memthrash_method = &memthrash_methods[memthrash_method];
 
-	pr_dbg("%s: using method '%s'\n", args->name, context.memthrash_method->name);
 	if (args->instance == 0) {
+		pr_dbg("%s: using method '%s'\n", args->name, context.memthrash_method->name);
 		pr_inf("%s: starting %" PRIu32 " thread%s on each of the %"
 			PRIu32 " stressors on a %" PRIu32 " CPU system\n",
 			args->name, context.max_threads, plural(context.max_threads),
@@ -949,48 +987,48 @@ static int stress_memthrash(const stress_args_t *args)
 
 	(void)sigfillset(&set);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	rc = stress_oomable_child(args, &context, stress_memthrash_child, STRESS_OOMABLE_NORMAL);
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 #if defined(HAVE_MEMTHRASH_NUMA)
-	free(context.numa_node_mask);
+	if (context.numa_mask)
+		stress_numa_mask_free(context.numa_mask);
 #endif
 
 	return rc;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_memthrash_method,	stress_set_memthrash_method },
-	{ 0,			NULL }
+static const char *stress_memthrash_method(const size_t i)
+{
+	return (i < SIZEOF_ARRAY(memthrash_methods)) ? memthrash_methods[i].name : NULL;
+}
+
+static const stress_opt_t opts[] = {
+	{ OPT_memthrash_method, "memthrash-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_memthrash_method },
+	END_OPT,
 };
 
-stressor_info_t stress_memthrash_info = {
+const stressor_info_t stress_memthrash_info = {
 	.stressor = stress_memthrash,
 	.class = CLASS_MEMORY,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.help = help
 };
 #else
 
-static int stress_set_memthrash_method(const char *name)
-{
-	(void)name;
-
-	(void)pr_inf("warning: --memthrash-method not available on this system\n");
-	return 0;
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_memthrash_method,	stress_set_memthrash_method },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_memthrash_method, "memthrash-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_unimplemented_method },
+	END_OPT,
 };
 
-stressor_info_t stress_memthrash_info = {
+const stressor_info_t stress_memthrash_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_MEMORY,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.help = help,
 	.unimplemented_reason = "built without pthread support"
 };
