@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,11 @@
  *
  */
 #include "stress-ng.h"
+#include "core-madvise.h"
+#include "core-mincore.h"
+#include "core-mmap.h"
+#include "core-numa.h"
+#include "core-out-of-memory.h"
 
 #define DEFAULT_MREMAP_BYTES	(256 * MB)
 #define MIN_MREMAP_BYTES	(4 * KB)
@@ -25,34 +30,18 @@
 
 static const stress_help_t help[] = {
 	{ NULL,	"mremap N",	  "start N workers stressing mremap" },
-	{ NULL,	"mremap-ops N",	  "stop after N mremap bogo operations" },
 	{ NULL,	"mremap-bytes N", "mremap N bytes maximum for each stress iteration" },
-	{ NULL, "mremap-lock",	  "mlock remap pages, force pages to be unswappable" },
+	{ NULL, "mremap-mlock",	  "mlock remap pages, force pages to be unswappable" },
+	{ NULL, "mremap-numa",	  "bind memory mappings to randomly selected NUMA nodes" },
+	{ NULL,	"mremap-ops N",	  "stop after N mremap bogo operations" },
 	{ NULL,	NULL,		  NULL }
 };
 
-static int stress_set_mremap_bytes(const char *opt)
-{
-	size_t mremap_bytes;
-
-	mremap_bytes = (size_t)stress_get_uint64_byte_memory(opt, 1);
-	stress_check_range_bytes("mremap-bytes", mremap_bytes,
-		MIN_MREMAP_BYTES, MAX_MREMAP_BYTES);
-	return stress_set_setting("mremap-bytes", TYPE_ID_SIZE_T, &mremap_bytes);
-}
-
-static int stress_set_mremap_mlock(const char *opt)
-{
-	bool mremap_mlock = true;
-
-	(void)opt;
-	return stress_set_setting("mremap-mlock", TYPE_ID_BOOL, &mremap_mlock);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_mremap_bytes,	stress_set_mremap_bytes },
-	{ OPT_mremap_mlock,	stress_set_mremap_mlock },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_mremap_bytes, "mremap-bytes", TYPE_ID_SIZE_T_BYTES_VM, MIN_MREMAP_BYTES, MAX_MREMAP_BYTES, NULL },
+	{ OPT_mremap_mlock, "mremap-mlock", TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mremap_numa,  "mremap-numa",  TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_MREMAP) &&	\
@@ -78,7 +67,7 @@ static inline void *rand_mremap_addr(const size_t sz, int flags)
 	if (addr == MAP_FAILED)
 		return NULL;
 
-	(void)munmap(addr, sz);
+	(void)stress_munmap_retry_enomem(addr, sz);
 
 	/*
 	 * At this point, we know that we can remap to this addr
@@ -94,14 +83,17 @@ static inline void *rand_mremap_addr(const size_t sz, int flags)
  *	try and remap old size to new size
  */
 static int try_remap(
-	const stress_args_t *args,
+	stress_args_t *args,
 	uint8_t **buf,
 	const size_t old_sz,
 	const size_t new_sz,
-	const bool mremap_mlock)
+	const bool mremap_mlock,
+	double *duration,
+	double *count)
 {
 	uint8_t *newbuf;
 	int retry, flags = 0;
+	static int metrics_counter = 0;
 #if defined(MREMAP_MAYMOVE)
 	const int maymove = MREMAP_MAYMOVE;
 #else
@@ -116,14 +108,18 @@ static int try_remap(
 #endif
 
 	for (retry = 0; retry < 100; retry++) {
+		double t = 0.0;
+
 #if defined(MREMAP_FIXED)
 		void *addr = rand_mremap_addr(new_sz + args->page_size, flags);
 #endif
-		if (!keep_stressing_flag()) {
-			(void)munmap(*buf, old_sz);
+		if (UNLIKELY(!stress_continue_flag())) {
+			(void)stress_munmap_retry_enomem(*buf, old_sz);
 			*buf = 0;
 			return 0;
 		}
+		if (UNLIKELY(metrics_counter == 0))
+			t = stress_time_now();
 #if defined(MREMAP_FIXED)
 		if (addr) {
 			newbuf = mremap(*buf, old_sz, new_sz, flags, addr);
@@ -133,7 +129,11 @@ static int try_remap(
 #else
 		newbuf = mremap(*buf, old_sz, new_sz, flags);
 #endif
-		if (newbuf && newbuf != MAP_FAILED) {
+		if (newbuf && (newbuf != MAP_FAILED)) {
+			if (UNLIKELY(metrics_counter == 0)) {
+				(*duration) += stress_time_now() - t;
+				(*count) += 1.0;
+			}
 			*buf = newbuf;
 
 #if defined(MREMAP_DONTUNMAP)
@@ -142,11 +142,18 @@ static int try_remap(
 			 *  followed by an unmap of the old mapping for
 			 *  some more exercise
 			 */
+			if (UNLIKELY(metrics_counter == 0))
+				t = stress_time_now();
 			newbuf = mremap(*buf, new_sz, new_sz,
 					MREMAP_DONTUNMAP | MREMAP_MAYMOVE);
-			if (newbuf && newbuf != MAP_FAILED) {
+			if (newbuf && (newbuf != MAP_FAILED)) {
+				if (UNLIKELY(metrics_counter == 0)) {
+					(*duration) += stress_time_now() - t;
+					(*count) += 1.0;
+				}
+
 				if (*buf)
-					(void)munmap(*buf, new_sz);
+					(void)stress_munmap_retry_enomem(*buf, new_sz);
 				*buf = newbuf;
 			}
 #endif
@@ -157,6 +164,9 @@ static int try_remap(
 #else
 			(void)mremap_mlock;
 #endif
+			if (metrics_counter++ > 500)
+				metrics_counter = 0;
+
 			return 0;
 		}
 
@@ -185,12 +195,18 @@ static int try_remap(
 	return -1;
 }
 
-static int stress_mremap_child(const stress_args_t *args, void *context)
+static int stress_mremap_child(stress_args_t *args, void *context)
 {
 	size_t new_sz, sz, mremap_bytes = DEFAULT_MREMAP_BYTES;
 	int flags = MAP_PRIVATE | MAP_ANONYMOUS;
 	const size_t page_size = args->page_size;
 	bool mremap_mlock = false;
+	bool mremap_numa = false;
+	double duration = 0.0, count = 0.0, rate;
+	int ret = EXIT_SUCCESS;
+#if defined(HAVE_LINUX_MEMPOLICY_H)
+	stress_numa_mask_t *numa_mask = NULL;
+#endif
 
 #if defined(MAP_POPULATE)
 	flags |= MAP_POPULATE;
@@ -211,15 +227,42 @@ static int stress_mremap_child(const stress_args_t *args, void *context)
 	new_sz = sz = mremap_bytes & ~(page_size - 1);
 
 	(void)stress_get_setting("mremap-mlock", &mremap_mlock);
+	(void)stress_get_setting("mremap-numa", &mremap_numa);
 
+	if (mremap_numa) {
+#if defined(HAVE_LINUX_MEMPOLICY_H)
+		if (stress_numa_nodes() > 1) {
+			numa_mask = stress_numa_mask_alloc();
+			if (!numa_mask) {
+				pr_inf("%s: cannot allocate NUMA mask, disabling --mremap-numa\n",
+					args->name);
+				mremap_numa = false;
+			}
+		} else {
+			if (args->instance == 0) {
+				pr_inf("%s: only 1 NUMA node available, disabling --mremap-numa\n",
+					args->name);
+				mremap_numa = false;
+			}
+		}
+#else
+		if (args->instance == 0)
+			pr_inf("%s: --mremap-numa selected but not supported by this system, disabling option\n",
+				args->name);
+		mremap_numa = false;
+#endif
+	}
+
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		uint8_t *buf = NULL, *ptr;
 		size_t old_sz;
 
-		if (!keep_stressing_flag())
-			break;
+		if (UNLIKELY(!stress_continue_flag()))
+			goto deinit;
 
 		buf = mmap(NULL, new_sz, PROT_READ | PROT_WRITE, flags, -1, 0);
 		if (buf == MAP_FAILED) {
@@ -229,39 +272,51 @@ static int stress_mremap_child(const stress_args_t *args, void *context)
 #endif
 			continue;	/* Try again */
 		}
+#if defined(HAVE_LINUX_MEMPOLICY_H)
+		if (mremap_numa)
+			stress_numa_randomize_pages(numa_mask, buf, page_size, sz);
+#endif
 		(void)stress_madvise_random(buf, new_sz);
+		(void)stress_madvise_mergeable(buf, new_sz);
 		(void)stress_mincore_touch_pages(buf, mremap_bytes);
 
 		/* Ensure we can write to the mapped pages */
 		if (g_opt_flags & OPT_FLAGS_VERIFY) {
 			stress_mmap_set(buf, new_sz, page_size);
-			if (stress_mmap_check(buf, sz, page_size) < 0) {
+			if (UNLIKELY(stress_mmap_check(buf, sz, page_size) < 0)) {
 				pr_fail("%s: mmap'd region of %zu "
 					"bytes does not contain expected data\n",
 					args->name, sz);
-				(void)munmap(buf, new_sz);
-				return EXIT_FAILURE;
+				(void)stress_munmap_retry_enomem(buf, new_sz);
+				ret = EXIT_FAILURE;
+				goto deinit;
 			}
 		}
 
 		old_sz = new_sz;
 		new_sz >>= 1;
 		while (new_sz > page_size) {
-			if (try_remap(args, &buf, old_sz, new_sz, mremap_mlock) < 0) {
-				(void)munmap(buf, old_sz);
-				return EXIT_FAILURE;
+			if (UNLIKELY(try_remap(args, &buf, old_sz, new_sz, mremap_mlock, &duration, &count) < 0)) {
+				(void)stress_munmap_retry_enomem(buf, old_sz);
+				ret = EXIT_FAILURE;
+				goto deinit;
 			}
-			if (!keep_stressing(args))
-				return EXIT_SUCCESS;
+			if (UNLIKELY(!stress_continue(args)))
+				goto deinit;
+#if defined(HAVE_LINUX_MEMPOLICY_H)
+			if (mremap_numa)
+				stress_numa_randomize_pages(numa_mask, buf, page_size, new_sz);
+#endif
 			(void)stress_madvise_random(buf, new_sz);
 			if (g_opt_flags & OPT_FLAGS_VERIFY) {
-				if (stress_mmap_check(buf, new_sz, page_size) < 0) {
+				if (UNLIKELY(stress_mmap_check(buf, new_sz, page_size) < 0)) {
 					pr_fail("%s: mremap'd region "
 						"of %zu bytes does "
 						"not contain expected data\n",
 						args->name, sz);
-					(void)munmap(buf, new_sz);
-					return EXIT_FAILURE;
+					(void)stress_munmap_retry_enomem(buf, new_sz);
+					ret = EXIT_FAILURE;
+					goto deinit;
 				}
 			}
 			old_sz = new_sz;
@@ -270,12 +325,17 @@ static int stress_mremap_child(const stress_args_t *args, void *context)
 
 		new_sz <<= 1;
 		while (new_sz < mremap_bytes) {
-			if (try_remap(args, &buf, old_sz, new_sz, mremap_mlock) < 0) {
-				(void)munmap(buf, old_sz);
-				return EXIT_FAILURE;
+			if (UNLIKELY(try_remap(args, &buf, old_sz, new_sz, mremap_mlock, &duration, &count) < 0)) {
+				(void)stress_munmap_retry_enomem(buf, old_sz);
+				ret = EXIT_FAILURE;
+				goto deinit;
 			}
-			if (!keep_stressing(args))
-				return EXIT_SUCCESS;
+			if (UNLIKELY(!stress_continue(args)))
+				goto deinit;
+#if defined(HAVE_LINUX_MEMPOLICY_H)
+			if (mremap_numa)
+				stress_numa_randomize_pages(numa_mask, buf, page_size, new_sz);
+#endif
 			(void)stress_madvise_random(buf, new_sz);
 			old_sz = new_sz;
 			new_sz <<= 1;
@@ -283,48 +343,60 @@ static int stress_mremap_child(const stress_args_t *args, void *context)
 
 		/* Invalid remap flags */
 		ptr = mremap(buf, old_sz, old_sz, ~0);
-		if (ptr && ptr != MAP_FAILED)
+		if (ptr && (ptr != MAP_FAILED))
 			buf = ptr;
-		ptr = mremap(buf, old_sz, old_sz, MREMAP_FIXED | MREMAP_MAYMOVE);
-		if (ptr && ptr != MAP_FAILED)
+		ptr = mremap(buf, old_sz, old_sz, MREMAP_FIXED | MREMAP_MAYMOVE, (void *)~(uintptr_t)0);
+		if (ptr && (ptr != MAP_FAILED))
 			buf = ptr;
 #if defined(MREMAP_MAYMOVE)
 		/* Invalid new size */
 		ptr = mremap(buf, old_sz, 0, MREMAP_MAYMOVE);
-		if (ptr && ptr != MAP_FAILED)
+		if (ptr && (ptr != MAP_FAILED))
 			buf = ptr;
 #endif
-		(void)munmap(buf, old_sz);
+		(void)stress_munmap_retry_enomem(buf, old_sz);
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
+deinit:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	return EXIT_SUCCESS;
+	rate = (count > 0.0) ? duration / count : 0.0;
+	stress_metrics_set(args, 0, "nanosecs per mremap call",
+		rate * STRESS_DBL_NANOSECOND, STRESS_METRIC_HARMONIC_MEAN);
+
+#if defined(HAVE_LINUX_MEMPOLICY_H)
+	if (numa_mask)
+		stress_numa_mask_free(numa_mask);
+#endif
+
+	return ret;
 }
 
 /*
  *  stress_mremap()
  *	stress mmap
  */
-static int stress_mremap(const stress_args_t *args)
+static int stress_mremap(stress_args_t *args)
 {
 	return stress_oomable_child(args, NULL, stress_mremap_child, STRESS_OOMABLE_NORMAL);
 }
 
-stressor_info_t stress_mremap_info = {
+const stressor_info_t stress_mremap_info = {
 	.stressor = stress_mremap,
 	.class = CLASS_VM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
-stressor_info_t stress_mremap_info = {
-	.stressor = stress_not_implemented,
+const stressor_info_t stress_mremap_info = {
+	.stressor = stress_unimplemented,
 	.class = CLASS_VM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
-	.help = help
+	.opts = opts,
+	.verify = VERIFY_OPTIONAL,
+	.help = help,
+	.unimplemented_reason = "built without mremap() system call support"
 };
 #endif
