@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
 
 #if defined(HAVE_LINUX_OPENAT2_H)
 #include <linux/openat2.h>
@@ -27,7 +29,7 @@
 #include <utime.h>
 #endif
 
-typedef int (*stress_open_func_t)(const stress_args_t *args, const pid_t pid, double *duration, double *count);
+typedef int (*stress_open_func_t)(stress_args_t *args, const char *temp_dir, const pid_t pid, double *duration, double *count);
 
 static const stress_help_t help[] = {
 	{ "o N", "open N",		"start N workers exercising open/close" },
@@ -145,11 +147,6 @@ static int open_flags[] = {
 #endif
 };
 
-static int stress_set_open_fd(const char *opt)
-{
-        return stress_set_setting_true("open-fd", opt);
-}
-
 static size_t stress_get_max_fds(void)
 {
 	const size_t max_size = (size_t)-1;
@@ -177,21 +174,22 @@ static size_t stress_get_max_fds(void)
 	return max_fds;
 }
 
-static int stress_set_open_max(const char *opt)
+static void stress_open_max(const char *opt_name, const char *opt_arg, stress_type_id_t *type_id, void *value)
 {
-	size_t open_max;
-	size_t max_fds = stress_get_max_fds();
+	size_t *open_max = (size_t *)value;
+	const size_t max_fds = stress_get_max_fds();
 
-	open_max = (size_t)stress_get_uint64_percent(opt, 1, (uint64_t)max_fds,
-			"cannot determine maximum number of file descriptors");
+	(void)opt_name;
 
-        return stress_set_setting("open-max", TYPE_ID_SIZE_T, &open_max);
+	*type_id = TYPE_ID_SIZE_T;
+	*open_max = (size_t)stress_get_uint64_percent(opt_arg, 1, (uint64_t)max_fds,
+                        "cannot determine maximum number of file descriptors");
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_open_fd,	stress_set_open_fd, },
-	{ OPT_open_max,	stress_set_open_max },
-	{ 0,		NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_open_fd,  "open-fd",  TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_open_max,	"open-max", TYPE_ID_CALLBACK, 0, 0, stress_open_max },
+	END_OPT,
 };
 
 #if defined(HAVE_OPENAT) &&	\
@@ -211,13 +209,13 @@ static int obsolete_futimesat(
     defined(HAVE_SYSCALL)
 	/* Try direct system call first */
 	ret = (int)syscall(__NR_futimesat, dir_fd, pathname, tv);
-	if ((ret == 0) || (errno != ENOSYS))
+	if (LIKELY((ret == 0) || (errno != ENOSYS)))
 		return ret;
 #endif
 #if defined(HAVE_FUTIMESAT)
 	/* Try libc variant next */
 	ret = (int)futimesat(dir_fd, pathname, tv);
-	if ((ret == 0) || (errno != ENOSYS))
+	if (LIKELY((ret == 0) || (errno != ENOSYS)))
 		return ret;
 #endif
 	/* Not available */
@@ -244,13 +242,13 @@ static int obsolete_futimes(const int fd, const struct timeval tv[2])
     defined(HAVE_SYSCALL)
 	/* Try direct system call first */
 	ret = (int)syscall(__NR_futimes, fd, tv);
-	if ((ret == 0) || (errno != ENOSYS))
+	if (LIKELY((ret == 0) || (errno != ENOSYS)))
 		return ret;
 #endif
 #if defined(HAVE_FUTIMES)
 	/* Try libc variant next */
 	ret = (int)futimes(fd, tv);
-	if ((ret == 0) || (errno != ENOSYS))
+	if (LIKELY((ret == 0) || (errno != ENOSYS)))
 		return ret;
 #endif
 	/* Not available */
@@ -279,7 +277,7 @@ static inline int open_arg2(
 #else
 	fd = open(pathname, flags);
 #endif
-	if (fd >= 0) {
+	if (LIKELY(fd >= 0)) {
 		(*duration) += stress_time_now() - t;
 		(*count) += 1.0;
 		(void)obsolete_futimes(fd, NULL);
@@ -306,7 +304,7 @@ static inline int open_arg3(
 	fd = open(pathname, flags, mode);
 #endif
 
-	if (fd >= 0) {
+	if (LIKELY(fd >= 0)) {
 		struct timeval tv[2];
 
 		(*duration) += stress_time_now() - t;
@@ -338,21 +336,22 @@ static inline int open_arg3(
 
 #if defined(O_CREAT)
 static int open_flag_perm(
-	const stress_args_t *args,
+	stress_args_t *args,
+	const char *temp_dir,
 	const pid_t pid,
 	double *duration,
 	double *count)
 {
-	static size_t index = 0;
+	static size_t idx = 0;
 	int fd;
 	const int mode = S_IRUSR | S_IWUSR;
-	const int flags = open_perms[index];
+	const int flags = open_perms[idx];
 	char filename[PATH_MAX];
 
 	(void)args;
 
-	(void)snprintf(filename, sizeof(filename), "stress-open-%" PRIdMAX "-%" PRIu32,
-		(intmax_t)pid, stress_mwc32());
+	(void)snprintf(filename, sizeof(filename), "%s/stress-open-%" PRIdMAX "-%" PRIu32,
+		temp_dir, (intmax_t)pid, stress_mwc32());
 
 	if (UNLIKELY((open_count == 0) || (!open_perms))) {
 		fd = open_arg3(filename, O_CREAT | O_RDWR, mode, duration, count);
@@ -366,30 +365,32 @@ static int open_flag_perm(
 			(void)mkdir(filename, mode);
 		} else {
 			fd = open_arg3(filename, O_CREAT | O_RDWR, mode, duration, count);
-			if (fd >= 0)
+			if (LIKELY(fd >= 0))
 				(void)close(fd);
 		}
 	}
 #else
 	fd = open_arg3(filename, O_CREAT | O_RDWR, mode, duration, count);
-	if (fd >= 0)
+	if (LIKELY(fd >= 0))
 		(void)close(fd);
 #endif
 	fd = open_arg3(filename, flags, mode, duration, count);
 #if defined(O_DIRECTORY)
-	if (flags & O_DIRECTORY)
+	if (LIKELY(flags & O_DIRECTORY))
 		(void)shim_rmdir(filename);
 #endif
 	(void)shim_unlink(filename);
-	index++;
-	index %= open_count;
+	idx++;
+	if (UNLIKELY(idx >= open_count))
+		idx = 0;
 
 	return fd;
 }
 #endif
 
 static int open_dev_zero_rd(
-	const stress_args_t *args,
+	stress_args_t *args,
+	const char *temp_dir,
 	const pid_t pid,
 	double *duration,
 	double *count)
@@ -397,6 +398,7 @@ static int open_dev_zero_rd(
 	int flags = 0;
 
 	(void)args;
+	(void)temp_dir,
 	(void)pid;
 
 #if defined(O_ASYNC)
@@ -424,7 +426,8 @@ static int open_dev_zero_rd(
 }
 
 static int open_dev_null_wr(
-	const stress_args_t *args,
+	stress_args_t *args,
+	const char *temp_dir,
 	const pid_t pid,
 	double *duration,
 	double *count)
@@ -432,6 +435,7 @@ static int open_dev_null_wr(
 	int flags = 0;
 
 	(void)args;
+	(void)temp_dir;
 	(void)pid;
 
 #if defined(O_ASYNC)
@@ -463,14 +467,16 @@ static int open_dev_null_wr(
 
 #if defined(O_TMPFILE)
 static int open_tmp_rdwr(
-	const stress_args_t *args,
+	stress_args_t *args,
+	const char *temp_dir,
 	const pid_t pid,
 	double *duration,
 	double *count)
 {
-	int flags = 0;
+	int fd, flags = O_TMPFILE;
 
 	(void)args;
+	(void)temp_dir;
 	(void)pid;
 
 #if defined(O_TRUNC)
@@ -487,18 +493,26 @@ static int open_tmp_rdwr(
 #endif
 	flags &= stress_mwc32();
 	flags |= O_TMPFILE | O_RDWR;
-	return open_arg3("/tmp", flags, S_IRUSR | S_IWUSR, duration, count);
+
+	/* try temp_dir first, some filesystems do not support this */
+	fd = open_arg3(temp_dir, flags, S_IRUSR | S_IWUSR, duration, count);
+	if (fd == -1)
+		fd = open_arg3("/tmp", flags, S_IRUSR | S_IWUSR, duration, count);
+
+	return fd;
 }
 #endif
 
 #if defined(O_TMPFILE)
 static int open_tmpfile_no_rdwr(
-	const stress_args_t *args,
+	stress_args_t *args,
+	const char *temp_dir,
 	const pid_t pid,
 	double *duration,
 	double *count)
 {
 	(void)args;
+	(void)temp_dir;
 	(void)pid;
 
 	/* Force -EINVAL, need O_WRONLY or O_RDWR to succeed */
@@ -510,12 +524,14 @@ static int open_tmpfile_no_rdwr(
     defined(O_RDWR) &&			\
     defined(N_NOCTTY)
 static int open_pt(
-	const stress_args_t *args,
+	stress_args_t *args,
+	const char *temp_dir,
 	const pid_t pid,
 	double *duration,
 	double *count)
 {
 	(void)args;
+	(void)temp_dir;
 	(void)pid;
 
 	return posix_openpt(O_RDWR | O_NOCTTY);
@@ -525,12 +541,14 @@ static int open_pt(
 #if defined(O_TMPFILE) &&	\
     defined(O_EXCL)
 static int open_tmp_rdwr_excl(
-	const stress_args_t *args,
+	stress_args_t *args,
+	const char *temp_dir,
 	const pid_t pid,
 	double *duration,
 	double *count)
 {
 	(void)args;
+	(void)temp_dir;
 	(void)pid;
 
 	return open_arg3("/tmp", O_TMPFILE | O_EXCL | O_RDWR, S_IRUSR | S_IWUSR, duration, count);
@@ -539,7 +557,8 @@ static int open_tmp_rdwr_excl(
 
 #if defined(O_DIRECTORY)
 static int open_dir(
-	const stress_args_t *args,
+	stress_args_t *args,
+	const char *temp_dir,
 	const pid_t pid,
 	double *duration,
 	double *count)
@@ -547,19 +566,21 @@ static int open_dir(
 	(void)args;
 	(void)pid;
 
-	return open_arg2(".", O_DIRECTORY | O_RDONLY, duration, count);
+	return open_arg2(temp_dir, O_DIRECTORY | O_RDONLY, duration, count);
 }
 #endif
 
 #if defined(O_DIRECTORY) &&	\
     defined(__linux__)
 static int open_dir_proc_self_fd(
-	const stress_args_t *args,
+	stress_args_t *args,
+	const char *temp_dir,
 	const pid_t pid,
 	double *duration,
 	double *count)
 {
 	(void)args;
+	(void)temp_dir;
 	(void)pid;
 
 	return open_arg2("/proc/self/fd", O_DIRECTORY | O_RDONLY, duration, count);
@@ -568,7 +589,8 @@ static int open_dir_proc_self_fd(
 
 #if defined(O_PATH)
 static int open_path(
-	const stress_args_t *args,
+	stress_args_t *args,
+	const char *temp_dir,
 	const pid_t pid,
 	double *duration,
 	double *count)
@@ -576,13 +598,14 @@ static int open_path(
 	(void)args;
 	(void)pid;
 
-	return open_arg2(".", O_DIRECTORY | O_PATH, duration, count);
+	return open_arg2(temp_dir, O_DIRECTORY | O_PATH, duration, count);
 }
 #endif
 
 #if defined(O_CREAT)
 static int open_create_eisdir(
-	const stress_args_t *args,
+	stress_args_t *args,
+	const char *temp_dir,
 	const pid_t pid,
 	double *duration,
 	double *count)
@@ -590,14 +613,15 @@ static int open_create_eisdir(
 	(void)args;
 	(void)pid;
 
-	return open_arg3(".", O_CREAT, S_IRUSR | S_IWUSR, duration, count);
+	return open_arg3(temp_dir, O_CREAT, S_IRUSR | S_IWUSR, duration, count);
 }
 #endif
 
 #if defined(O_DIRECT) &&	\
     defined(O_CREAT)
 static int open_direct(
-	const stress_args_t *args,
+	stress_args_t *args,
+	const char *temp_dir,
 	const pid_t pid,
 	double *duration,
 	double *count)
@@ -606,22 +630,22 @@ static int open_direct(
 	int fd;
 	double t;
 
-	(void)snprintf(filename, sizeof(filename), "stress-open-%" PRIdMAX "-%" PRIu32,
-		(intmax_t)pid, stress_mwc32());
+	(void)snprintf(filename, sizeof(filename), "%s/stress-open-%" PRIdMAX "-%" PRIu32,
+		temp_dir, (intmax_t)pid, stress_mwc32());
 
 	t = stress_time_now();
 	fd = open(filename, O_RDWR | O_CREAT | O_TRUNC | O_DIRECT, S_IRUSR | S_IWUSR);
-	if (fd >= 0) {
+	if (LIKELY(fd >= 0)) {
 		(*duration) += stress_time_now() - t;
 		(*count) += 1.0;
 	} else {
 		int ret;
 		struct stat statbuf;
 
-		ret = stat(filename, &statbuf);
-		if (ret == 0) {
-			pr_inf("%s: open with O_DIRECT failed but file '%s' was created\n",
-				args->name, filename);
+		ret = shim_stat(filename, &statbuf);
+		if (UNLIKELY(ret == 0)) {
+			pr_inf("%s: open with O_DIRECT failed but file '%s' was created%s\n",
+				args->name, filename, stress_get_fs_type(filename));
 		}
 	}
 	(void)shim_unlink(filename);
@@ -633,7 +657,8 @@ static int open_direct(
     defined(AT_FDCWD) &&	\
     defined(O_CREAT)
 static int open_with_openat_cwd(
-	const stress_args_t *args,
+	stress_args_t *args,
+	const char *temp_dir,
 	const pid_t pid,
 	double *duration,
 	double *count)
@@ -653,12 +678,12 @@ static int open_with_openat_cwd(
 	if (chdir(temp_path) < 0)
 		return -1;
 
-	(void)snprintf(filename, sizeof(filename), "stress-open-%" PRIdMAX "-%" PRIu32,
-		(intmax_t)pid, stress_mwc32());
+	(void)snprintf(filename, sizeof(filename), "%s/stress-open-%" PRIdMAX "-%" PRIu32,
+		temp_dir, (intmax_t)pid, stress_mwc32());
 
 	t = stress_time_now();
 	fd = openat(AT_FDCWD, filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);
-	if (fd >= 0) {
+	if (LIKELY(fd >= 0)) {
 		struct timeval tv[2];
 
 		(*duration) += stress_time_now() - t;
@@ -669,6 +694,11 @@ static int open_with_openat_cwd(
 		/* Exercise illegal times */
 		tv[0].tv_usec = 1000001;
 		tv[0].tv_sec = 0;
+		tv[1].tv_usec = 0;
+		tv[1].tv_sec = 1;
+		(void)obsolete_futimesat(AT_FDCWD, filename, tv);
+		tv[0].tv_usec = 0;
+		tv[0].tv_sec = 1;
 		tv[1].tv_usec = 1000001;
 		tv[1].tv_sec = 0;
 		(void)obsolete_futimesat(AT_FDCWD, filename, tv);
@@ -687,26 +717,30 @@ static int open_with_openat_cwd(
     defined(O_DIRECTORY) &&	\
     defined(O_CREAT)
 static int open_with_openat_dir_fd(
-	const stress_args_t *args,
+	stress_args_t *args,
+	const char *temp_dir,
 	const pid_t pid,
 	double *duration,
 	double *count)
 {
 	char filename[PATH_MAX];
 	int fd, dir_fd;
+	const uint32_t rnd32 = stress_mwc32();
 
 	(void)args;
 
 	(void)snprintf(filename, sizeof(filename), "stress-open-%" PRIdMAX "-%" PRIu32,
-		(intmax_t)pid, stress_mwc32());
+		(intmax_t)pid, rnd32);
 
-	dir_fd = open_arg2(".", O_DIRECTORY | O_PATH, duration, count);
+	dir_fd = open_arg2(temp_dir, O_DIRECTORY | O_PATH, duration, count);
 	if (dir_fd < 0)
 		return -1;
 
 	fd = openat(dir_fd, filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);
-	if (fd >= 0) {
+	if (LIKELY(fd >= 0)) {
 		(void)obsolete_futimesat(dir_fd, filename, NULL);
+		(void)snprintf(filename, sizeof(filename), "%s/stress-open-%" PRIdMAX "-%" PRIu32,
+			temp_dir, (intmax_t)pid, rnd32);
 		(void)shim_unlink(filename);
 	}
 	(void)close(dir_fd);
@@ -721,7 +755,8 @@ static int open_with_openat_dir_fd(
     defined(HAVE_SYSCALL) &&		\
     defined(O_CREAT)
 static int open_with_openat2_cwd(
-	const stress_args_t *args,
+	stress_args_t *args,
+	const char *temp_dir,
 	const pid_t pid,
 	double *duration,
 	double *count)
@@ -751,7 +786,6 @@ static int open_with_openat2_cwd(
 
 	char cwd[PATH_MAX];
 	char filename[PATH_MAX];
-	const char *temp_path = stress_get_temp_path();
 	int fd = -1;
 	size_t i = 0;
 	static size_t j;
@@ -759,11 +793,9 @@ static int open_with_openat2_cwd(
 
 	(void)args;
 
-	if (!temp_path)
-		return -1;
 	if (!getcwd(cwd, sizeof(cwd)))
 		return -1;
-	if (chdir(temp_path) < 0)
+	if (chdir(temp_dir) < 0)
 		return -1;
 	(void)snprintf(filename, sizeof(filename), "stress-open-%" PRIdMAX "-%" PRIu32,
 		(intmax_t)pid, stress_mwc32());
@@ -775,17 +807,17 @@ static int open_with_openat2_cwd(
 	for (i = 0; i < SIZEOF_ARRAY(resolve_flags); i++) {
 		double t;
 
-		(void)memset(&how, 0, sizeof(how));
+		(void)shim_memset(&how, 0, sizeof(how));
 		how.flags = O_CREAT | O_RDWR;
 		how.mode = S_IRUSR | S_IWUSR;
 		how.resolve = resolve_flags[j++];
 
-		if (j >= SIZEOF_ARRAY(resolve_flags))
+		if (UNLIKELY(j >= SIZEOF_ARRAY(resolve_flags)))
 			j = 0;
 
 		/* Exercise illegal usize field */
 		fd = (int)syscall(__NR_openat2, AT_FDCWD, filename, &how, 0);
-		if (fd >= 0) {
+		if (UNLIKELY(fd >= 0)) {
 			/* Unxexpected, but handle it anyhow */
 			(void)shim_unlink(filename);
 			break;
@@ -793,7 +825,7 @@ static int open_with_openat2_cwd(
 
 		t = stress_time_now();
 		fd = (int)syscall(__NR_openat2, AT_FDCWD, filename, &how, sizeof(how));
-		if (fd >= 0) {
+		if (LIKELY(fd >= 0)) {
 			(*duration) += stress_time_now() - t;
 			(*count) += 1.0;
 			(void)shim_unlink(filename);
@@ -808,7 +840,8 @@ static int open_with_openat2_cwd(
 
 #if defined(__linux__)
 static int open_with_open_proc_self_fd(
-	const stress_args_t *args,
+	stress_args_t *args,
+	const char *temp_dir,
 	const pid_t pid,
 	double *duration,
 	double *count)
@@ -817,11 +850,12 @@ static int open_with_open_proc_self_fd(
 	double t;
 
 	(void)args;
+	(void)temp_dir;
 	(void)pid;
 
 	t = stress_time_now();
 	fd = open("/proc/self/fd/0", O_RDONLY);
-	if (fd >= 0) {
+	if (LIKELY(fd >= 0)) {
 		(*duration) += stress_time_now() - t;
 		(*count) += 1.0;
 	}
@@ -830,7 +864,8 @@ static int open_with_open_proc_self_fd(
 #endif
 
 static int open_dup(
-	const stress_args_t *args,
+	stress_args_t *args,
+	const char *temp_dir,
 	const pid_t pid,
 	double *duration,
 	double *count)
@@ -839,11 +874,12 @@ static int open_dup(
 	double t;
 
 	(void)args;
+	(void)temp_dir;
 	(void)pid;
 
 	t = stress_time_now();
 	fd = dup(STDOUT_FILENO);
-	if (fd >= 0) {
+	if (LIKELY(fd >= 0)) {
 		(*duration) += stress_time_now() - t;
 		(*count) += 1.0;
 	}
@@ -853,7 +889,8 @@ static int open_dup(
 #if defined(O_CREAT) &&	\
     defined(O_TRUNC)
 static int open_rdonly_trunc(
-	const stress_args_t *args,
+	stress_args_t *args,
+	const char *temp_dir,
 	const pid_t pid,
 	double *duration,
 	double *count)
@@ -863,8 +900,8 @@ static int open_rdonly_trunc(
 
 	(void)args;
 
-	(void)snprintf(filename, sizeof(filename), "stress-open-%" PRIdMAX "-%" PRIu32,
-		(intmax_t)pid, stress_mwc32());
+	(void)snprintf(filename, sizeof(filename), "%s/stress-open-%" PRIdMAX "-%" PRIu32,
+		temp_dir, (intmax_t)pid, stress_mwc32());
 
 	fd = open_arg3(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR, duration, count);
 	if (fd < 0)
@@ -878,6 +915,45 @@ static int open_rdonly_trunc(
 }
 #endif
 
+#if defined(O_CREAT)
+static int open_modes(
+	stress_args_t *args,
+	const char *temp_dir,
+	const pid_t pid,
+	double *duration,
+	double *count)
+{
+	char filename[PATH_MAX];
+	int fd;
+	int mode_mask = 0777;
+	static int mode = 0;
+
+#if defined(S_ISVTX)
+	mode_mask |= S_ISVTX;
+#endif
+#if defined(S_ISGID)
+	mode_mask |= S_ISGID;
+#endif
+#if defined(S_ISUID)
+	mode_mask |= S_ISUID;
+#endif
+
+	(void)args;
+
+	(void)snprintf(filename, sizeof(filename), "%s/stress-open-%" PRIdMAX "-%" PRIu32,
+		temp_dir, (intmax_t)pid, stress_mwc32());
+
+	fd = open_arg3(filename, O_CREAT | O_RDWR, mode, duration, count);
+	mode++;
+	mode &= mode_mask;
+	if (UNLIKELY(fd < 0))
+		return fd;
+
+	(void)shim_unlink(filename);
+	return fd;
+}
+#endif
+
 static stress_open_func_t open_funcs[] = {
 #if defined(O_CREAT)
 	open_flag_perm,
@@ -943,6 +1019,9 @@ static stress_open_func_t open_funcs[] = {
     defined(O_TRUNC)
 	open_rdonly_trunc,
 #endif
+#if defined(O_CREAT)
+	open_modes,
+#endif
 };
 
 /*
@@ -978,10 +1057,10 @@ static void stress_fd_dir(const char *path, double *duration, double *count)
  *  stress_open()
  *	stress system by rapid open/close calls
  */
-static int stress_open(const stress_args_t *args)
+static int stress_open(stress_args_t *args)
 {
-	int *fds;
-	char path[PATH_MAX];
+	int *fds, ret;
+	char path[PATH_MAX], temp_dir[PATH_MAX];
 	const size_t max_size = (size_t)-1;
 	size_t open_max = stress_get_max_file_limit();
 	size_t i, sz;
@@ -999,6 +1078,11 @@ static int stress_open(const stress_args_t *args)
 	if (sizeof(void *) == 4)
 		open_max = STRESS_MINIMUM(open_max, 65536);
 
+	ret = stress_temp_dir_mk_args(args);
+	if (ret < 0)
+		return stress_exit_status(-ret);
+	(void)stress_temp_dir_args(args, temp_dir, sizeof(temp_dir));
+
 	(void)stress_get_setting("open-max", &open_max);
 	(void)stress_get_setting("open-fd", &open_fd);
 
@@ -1008,21 +1092,31 @@ static int stress_open(const stress_args_t *args)
 	/* Limit to max int (fd value) */
 	if (open_max > INT_MAX)
 		open_max = INT_MAX;
+	if (open_max < 1)
+		open_max = 1;
 
-	if (!args->instance)
-		pr_inf("%s: using a maximum of %zd file descriptors\n", args->name, open_max);
 
 	sz = open_max * sizeof(*fds);
 	fds = (int *)mmap(NULL, sz, PROT_READ | PROT_WRITE,
-		MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
+			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (fds == MAP_FAILED) {
-		pr_inf_skip("%s: cannot mmap %zd file descriptors\n", args->name, open_max);
-		return EXIT_NO_RESOURCE;
+		/* shrink */
+		open_max = 1024 * 1024;
+		sz = open_max * sizeof(*fds);
+		fds = (int *)mmap(NULL, sz, PROT_READ | PROT_WRITE,
+				MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
+		if (fds == MAP_FAILED) {
+			pr_inf_skip("%s: cannot mmap %zd file descriptors\n", args->name, open_max);
+			return EXIT_NO_RESOURCE;
+		}
 	}
+	if (!args->instance)
+		pr_inf("%s: using a maximum of %zd file descriptors\n", args->name, open_max);
+	stress_set_vma_anon_name(fds, sz, "fds");
 
 	if (open_fd) {
 		(void)snprintf(path, sizeof(path), "/proc/%" PRIdMAX "/fd", (intmax_t)mypid);
-		if ((stat(path, &statbuf) == 0) &&
+		if ((shim_stat(path, &statbuf) == 0) &&
 		    ((statbuf.st_mode & S_IFMT) == S_IFDIR)) {
 			pid = fork();
 
@@ -1037,33 +1131,34 @@ static int stress_open(const stress_args_t *args)
 		all_open_flags |= open_flags[i];
 	open_count = stress_flag_permutation(all_open_flags, &open_perms);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		size_t n;
-		int ret;
 		unsigned int min_fd = UINT_MAX, max_fd = 0;
 
 		for (i = 0; i < open_max; i++) {
 			for (;;) {
 				int idx;
 
-				if (!keep_stressing(args)) {
+				if (UNLIKELY(!stress_continue(args))) {
 					if (pid > 1)
-						(void)kill(pid, SIGKILL);
+						(void)stress_kill_pid(pid);
 					goto close_all;
 				}
 
 				idx = stress_mwc32modn(SIZEOF_ARRAY(open_funcs));
-				fds[i] = open_funcs[idx](args, mypid, &duration, &count);
+				fds[i] = open_funcs[idx](args, temp_dir, mypid, &duration, &count);
 
 				if (fds[i] >= 0)
 					break;
 
 				/* Check if we hit the open file limit */
-				if ((errno == EMFILE) || (errno == ENFILE)) {
+				if (UNLIKELY((errno == EMFILE) || (errno == ENFILE))) {
 					if (pid > 1)
-						(void)kill(pid, SIGKILL);
+						(void)stress_kill_pid(pid);
 					goto close_all;
 				}
 
@@ -1077,10 +1172,10 @@ static int stress_open(const stress_args_t *args)
 			}
 			stress_read_fdinfo(mypid, fds[i]);
 
-			if ((i & 8191) == 8191)
+			if (UNLIKELY((i & 8191) == 8191))
 				sync();
 
-			inc_counter(args);
+			stress_bogo_inc(args);
 		}
 close_all:
 		n = i;
@@ -1096,30 +1191,28 @@ close_all:
 			for (i = 0; i < n; i++)
 				(void)close(fds[i]);
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
+	(void)stress_temp_dir_rm_args(args);
 	(void)munmap((void *)fds, sz);
 	if (open_perms)
 		free(open_perms);
 
-	if (pid > 1) {
-		int status;
-
-		(void)kill(pid, SIGKILL);
-		(void)waitpid(pid, &status, 0);
-	}
+	if (pid > 1)
+		(void)stress_kill_pid_wait(pid, NULL);
 
 	rate = (count > 0.0) ? duration / count: 0.0;
-	stress_metrics_set(args, 0, "nanosecs per open", rate * 1000000000);
+	stress_metrics_set(args, 0, "nanosecs per open",
+		rate * 1000000000.0, STRESS_METRIC_HARMONIC_MEAN);
 
 	return EXIT_SUCCESS;
 }
 
-stressor_info_t stress_open_info = {
+const stressor_info_t stress_open_info = {
 	.stressor = stress_open,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.help = help
 };
