@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,6 +21,8 @@
 #include "core-affinity.h"
 #include "core-builtin.h"
 
+#include <sys/ioctl.h>
+
 static const stress_help_t help[] = {
 	{ "p N", "pipe N",		"start N workers exercising pipe I/O" },
 	{ NULL,	"pipe-data-size N",	"set pipe size of each pipe write to N bytes" },
@@ -36,46 +38,12 @@ static const stress_help_t help[] = {
 #define PIPE_BUF	(4096)
 #endif
 
-static int stress_set_pipe_vmsplice(const char *opt)
-{
-	return stress_set_setting_true("pipe-vmsplice", opt);
-}
-
-#if defined(F_SETPIPE_SZ)
-/*
- *  stress_set_pipe_size()
- *	set pipe size in bytes
- */
-static int stress_set_pipe_size(const char *opt)
-{
-	size_t pipe_size;
-
-	pipe_size = (size_t)stress_get_uint64_byte(opt);
-	stress_check_range_bytes("pipe-size", pipe_size, 4096, 1024 * 1024);
-	return stress_set_setting("pipe-size", TYPE_ID_SIZE_T, &pipe_size);
-}
-#endif
-
-/*
- *  stress_set_pipe_size()
- *	set pipe data write size in bytes
- */
-static int stress_set_pipe_data_size(const char *opt)
-{
-	size_t pipe_data_size;
-
-	pipe_data_size = (size_t)stress_get_uint64_byte(opt);
-	stress_check_range_bytes("pipe-data-size", pipe_data_size,
-		8, stress_get_page_size());
-	return stress_set_setting("pipe-data-size", TYPE_ID_SIZE_T, &pipe_data_size);
-}
-
 static size_t pipe_get_size(const int fd)
 {
 #if defined(F_GETPIPE_SZ)
 	ssize_t sz;
 
-	if ((sz = fcntl(fd, F_GETPIPE_SZ)) < 0)
+	if (UNLIKELY((sz = fcntl(fd, F_GETPIPE_SZ)) < 0))
 		return PIPE_BUF;
 	return sz;
 #else
@@ -97,26 +65,26 @@ static void pipe_change_size(
 #if defined(F_GETPIPE_SZ)
 	ssize_t sz;
 #endif
-	if (!pipe_size)
+	if (UNLIKELY(!pipe_size))
 		return;
 
 #if !(defined(HAVE_PIPE2) &&	\
     defined(O_DIRECT))
-	if (pipe_size < args->page_size)
+	if (UNLIKELY(pipe_size < args->page_size))
 		return;
 #endif
-	if (fcntl(fd, F_SETPIPE_SZ, pipe_size) < 0) {
+	if (UNLIKELY(fcntl(fd, F_SETPIPE_SZ, pipe_size) < 0)) {
 		pr_err("%s: cannot set pipe size, keeping "
 			"default pipe size, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 	}
 #if defined(F_GETPIPE_SZ)
 	/* Sanity check size */
-	if ((sz = fcntl(fd, F_GETPIPE_SZ)) < 0) {
+	if (UNLIKELY((sz = fcntl(fd, F_GETPIPE_SZ)) < 0)) {
 		pr_err("%s: cannot get pipe size, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 	} else {
-		if ((size_t)sz != pipe_size) {
+		if (UNLIKELY((size_t)sz != pipe_size)) {
 			pr_err("%s: cannot set desired pipe size, "
 				"pipe %zd, defaulting to size %zd\n",
 				args->name, pipe_size, sz);
@@ -250,8 +218,8 @@ static int stress_pipe_read_splice(
 #endif
 	struct iovec iov;
 	size_t offset = 0;
-	size_t nbufs = buf_size / pipe_data_size;
-	size_t offset_end = nbufs * pipe_data_size;
+	const size_t nbufs = buf_size / pipe_data_size;
+	const size_t offset_end = nbufs * pipe_data_size;
 
 	iov.iov_len = pipe_data_size;
 
@@ -260,7 +228,7 @@ static int stress_pipe_read_splice(
 
 		iov.iov_base = buf + offset;
 		offset += pipe_data_size;
-		if (offset >= offset_end)
+		if (UNLIKELY(offset >= offset_end))
 			offset = 0;
 		n = vmsplice(fd, &iov, 1, 0);
 		if (UNLIKELY(n <= 0)) {
@@ -312,8 +280,8 @@ static int stress_pipe_read_splice_verify(
 	uint32_t *buf32;
 	struct iovec iov;
 	size_t offset = 0;
-	size_t nbufs = buf_size / pipe_data_size;
-	size_t offset_end = nbufs * pipe_data_size;
+	const size_t nbufs = buf_size / pipe_data_size;
+	const size_t offset_end = nbufs * pipe_data_size;
 
 	iov.iov_len = pipe_data_size;
 
@@ -323,7 +291,7 @@ static int stress_pipe_read_splice_verify(
 		iov.iov_base = buf + offset;
 		buf32 = (uint32_t *)iov.iov_base;
 		offset += pipe_data_size;
-		if (offset >= offset_end)
+		if (UNLIKELY(offset >= offset_end))
 			offset = 0;
 		n = vmsplice(fd, &iov, 1, 0);
 		if (UNLIKELY(n <= 0)) {
@@ -462,8 +430,8 @@ static int stress_pipe_write_splice(
 	register uint64_t bytes = 0;
 	struct iovec iov;
 	size_t offset = 0;
-	size_t nbufs = buf_size / pipe_data_size;
-	size_t offset_end = nbufs * pipe_data_size;
+	const size_t nbufs = buf_size / pipe_data_size;
+	const size_t offset_end = nbufs * pipe_data_size;
 
 	iov.iov_len = pipe_data_size;
 
@@ -472,7 +440,7 @@ static int stress_pipe_write_splice(
 
 		iov.iov_base = buf + offset;
 		offset += pipe_data_size;
-		if (offset >= offset_end)
+		if (UNLIKELY(offset >= offset_end))
 			offset = 0;
 		ret = vmsplice(fd, &iov, 1, 0);
 		if (UNLIKELY(ret <= 0)) {
@@ -513,8 +481,8 @@ static int stress_pipe_write_splice_verify(
 	register uint64_t bytes = 0;
 	struct iovec iov;
 	size_t offset = 0;
-	size_t nbufs = buf_size / pipe_data_size;
-	size_t offset_end = nbufs * pipe_data_size;
+	const size_t nbufs = buf_size / pipe_data_size;
+	const size_t offset_end = nbufs * pipe_data_size;
 
 	iov.iov_len = pipe_data_size;
 
@@ -526,7 +494,7 @@ static int stress_pipe_write_splice_verify(
 		buf32 = (uint32_t *)iov.iov_base;
 		*buf32 = val++;
 		offset += pipe_data_size;
-		if (offset >= offset_end)
+		if (UNLIKELY(offset >= offset_end))
 			offset = 0;
 		ret = vmsplice(fd, &iov, 1, 0);
 		if (UNLIKELY(ret <= 0)) {
@@ -633,6 +601,8 @@ static int stress_pipe(stress_args_t *args)
 		(void)close(pipefds[1]);
 		return EXIT_NO_RESOURCE;
 	}
+	stress_set_vma_anon_name(buf_rd, buf_rd_size, "read-buffer");
+
 	/* round to nearest whole page */
 	buf_wr_size = ((pipe_wr_size + page_size - 1) & ~(page_size - 1)) * 2;
 	buf_wr = (char *)stress_mmap_populate(NULL, buf_wr_size,
@@ -646,8 +616,11 @@ static int stress_pipe(stress_args_t *args)
 		(void)close(pipefds[1]);
 		return EXIT_NO_RESOURCE;
 	}
+	stress_set_vma_anon_name(buf_wr, buf_wr_size, "write-buffer");
 	stress_rndbuf(buf_wr, buf_wr_size);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
 	parent_cpu = stress_get_cpu();
@@ -714,7 +687,7 @@ again:
 		duration = stress_time_now() - t;
 		rate = (duration > 0.0) ? ((double)bytes / duration) / (double)MB : 0.0;
 		stress_metrics_set(args, 0, "MB per sec pipe write rate",
-			rate, STRESS_HARMONIC_MEAN);
+			rate, STRESS_METRIC_HARMONIC_MEAN);
 
 		(void)close(pipefds[1]);
 		(void)shim_kill(pid, SIGPIPE);
@@ -734,19 +707,20 @@ finish:
 	return rc;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
+static const stress_opt_t opts[] = {
 #if defined(F_SETPIPE_SZ)
-	{ OPT_pipe_size,	stress_set_pipe_size },
+	{ OPT_pipe_size,      "pipe-size",      TYPE_ID_SIZE_T_BYTES_VM, 4096, 1024 * 1024, NULL },
 #endif
-	{ OPT_pipe_data_size,	stress_set_pipe_data_size },
-	{ OPT_pipe_vmsplice,	stress_set_pipe_vmsplice },
-	{ 0,			NULL }
+	/* FIXME: was min = 8, max = stress_get_page_size() */
+	{ OPT_pipe_data_size, "pipe-data-size", TYPE_ID_SIZE_T_BYTES_VM, 8, 4096, NULL },
+	{ OPT_pipe_vmsplice,  "pipe-vmsplice",  TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
 };
 
-stressor_info_t stress_pipe_info = {
+const stressor_info_t stress_pipe_info = {
 	.stressor = stress_pipe,
-	.class = CLASS_PIPE_IO | CLASS_MEMORY | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.class = CLASS_PIPE_IO | CLASS_MEMORY | CLASS_OS | CLASS_IPC,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
