@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2014-2021 Canonical, Ltd.
- * Copyright (C) 2021-2024 Colin Ian King.
+ * Copyright (C) 2021-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,6 +23,9 @@
 #include "core-pthread.h"
 #include "core-put.h"
 
+#include <ctype.h>
+#include <sys/ioctl.h>
+
 #if defined(HAVE_ASM_MTRR_H)
 #include <asm/mtrr.h>
 #endif
@@ -93,7 +96,7 @@ static int stress_dirent_proc_prune(struct dirent **dlist, const int n)
 
 			if (stress_is_dot_filename(dlist[i]->d_name))
 				ignore = true;
-			else if (isdigit((int)dlist[i]->d_name[0])) {
+			else if (isdigit((unsigned char)dlist[i]->d_name[0])) {
 				/* only allow a small numeric files.. */
 				ignore = (digit_count > 1);
 				digit_count++;
@@ -145,10 +148,8 @@ static uint32_t path_sum(const char *path)
 
 static int mixup_sort(const struct dirent **d1, const struct dirent **d2)
 {
-	uint32_t s1, s2;
-
-	s1 = path_sum((*d1)->d_name);
-	s2 = path_sum((*d2)->d_name);
+	register const uint32_t s1 = path_sum((*d1)->d_name);
+	register const uint32_t s2 = path_sum((*d2)->d_name);
 
 	if (s1 == s2)
 		return 0;
@@ -162,7 +163,8 @@ static int mixup_sort(const struct dirent **d1, const struct dirent **d2)
  */
 static void stress_proc_self_mem(stress_args_t *args, const int fd)
 {
-	uint8_t *page, *buf, rnd = stress_mwc8();
+	uint8_t *page, *buf;
+	const uint8_t rnd = stress_mwc8();
 	const size_t page_size = stress_get_page_size();
 	off_t offset;
 
@@ -176,12 +178,13 @@ static void stress_proc_self_mem(stress_args_t *args, const int fd)
 		free(buf);
 		return;
 	}
+	stress_set_vma_anon_name(page, page_size, "proc-self-mem");
 	offset = (off_t)(uintptr_t)page;
 
 	(void)shim_memset(page, rnd, page_size);
 	if (lseek(fd, offset, SEEK_SET) == offset) {
 		ssize_t ret;
-		size_t mem_offset = stress_mwc32modn((uint32_t)page_size);
+		const size_t mem_offset = stress_mwc32modn((uint32_t)page_size);
 
 		ret = read(fd, buf, page_size);
 		if ((ret == (ssize_t)page_size) &&
@@ -191,7 +194,7 @@ static void stress_proc_self_mem(stress_args_t *args, const int fd)
 		}
 	}
 
-	(void)munmap(page, page_size);
+	(void)munmap((void *)page, page_size);
 	free(buf);
 }
 
@@ -268,18 +271,18 @@ static inline void stress_proc_rw(
 
 	while ((loops == -1) || (loops > 0)) {
 		double t_start;
-		bool timeout = false;
 		uint8_t *ptr;
 		struct stat statbuf;
-		bool writeable = true;
 		size_t len;
 		ssize_t i;
+		bool timeout = false;
+		bool writeable = true;
 		bool skip_fifo = true;
 
 		ret = shim_pthread_spin_lock(&lock);
 		if (ret)
 			return;
-		(void)shim_strlcpy(path, proc_path, sizeof(path));
+		(void)shim_strscpy(path, proc_path, sizeof(path));
 		(void)shim_pthread_spin_unlock(&lock);
 
 		if (!*path || !stress_continue_flag())
@@ -287,7 +290,7 @@ static inline void stress_proc_rw(
 
 		if (!strncmp(path, "/proc/self", 10))
 			writeable = false;
-		if (!strncmp(path, "/proc", 5) && isdigit(path[5]))
+		if (!strncmp(path, "/proc", 5) && isdigit((unsigned char)path[5]))
 			writeable = false;
 
 		t_start = stress_time_now();
@@ -455,7 +458,7 @@ mmap_test:
 		/*
 		 *  mmap it
 		 */
-		ptr = mmap(NULL, page_size, PROT_READ,
+		ptr = (uint8_t *)mmap(NULL, page_size, PROT_READ,
 #if defined(MAP_POPULATE)
 			MAP_POPULATE |
 #endif
@@ -463,7 +466,7 @@ mmap_test:
 			fd, 0);
 		if (ptr != MAP_FAILED) {
 			stress_uint8_put(*ptr);
-			(void)munmap(ptr, page_size);
+			(void)munmap((void *)ptr, page_size);
 		}
 
 		if ((stress_time_now() - t_start) > threshold)
@@ -482,7 +485,8 @@ mmap_test:
 			goto timeout_close;
 #endif
 
-#if defined(HAVE_POLL_H)
+#if defined(HAVE_POLL_H) &&	\
+    defined(HAVE_POLL)
 		{
 			struct pollfd fds[1];
 
@@ -512,7 +516,7 @@ mmap_test:
 			fds[0].events = POLLIN;
 			fds[0].revents = 0;
 
-			VOID_RET(int, ppoll(fds, 1, &ts, &sigmask));
+			VOID_RET(int, shim_ppoll(fds, 1, &ts, &sigmask));
 		}
 #endif
 
@@ -600,7 +604,6 @@ timeout_close:
  */
 static void *stress_proc_rw_thread(void *ctxt_ptr)
 {
-	static void *nowt = NULL;
 	const stress_ctxt_t *ctxt = (stress_ctxt_t *)ctxt_ptr;
 
 	/*
@@ -616,7 +619,7 @@ static void *stress_proc_rw_thread(void *ctxt_ptr)
 			break;
 	}
 
-	return &nowt;
+	return &g_nowt;
 }
 
 /*
@@ -654,6 +657,7 @@ static void stress_proc_dir(
 	/* Non-directories files first */
 	for (i = 0; (i < n) && stress_continue_flag(); i++) {
 		struct dirent *d = dlist[i];
+		unsigned char type;
 
 		if (stress_is_dot_filename(d->d_name)) {
 			free(d);
@@ -661,11 +665,12 @@ static void stress_proc_dir(
 			continue;
 		}
 
-		if ((d->d_type == DT_REG) || (d->d_type == DT_LNK)) {
+		type = shim_dirent_type(path, d);
+		if ((type == SHIM_DT_REG) || (type == SHIM_DT_LNK)) {
 			ret = shim_pthread_spin_lock(&lock);
 			if (!ret) {
 				(void)stress_mk_filename(tmp, sizeof(tmp), path, d->d_name);
-				(void)shim_strlcpy(proc_path, tmp, sizeof(proc_path));
+				(void)shim_strscpy(proc_path, tmp, sizeof(proc_path));
 				(void)shim_pthread_spin_unlock(&lock);
 
 				stress_proc_rw(ctxt, loops);
@@ -685,7 +690,7 @@ static void stress_proc_dir(
 	for (i = 0; (i < n) && stress_continue_flag(); i++) {
 		struct dirent *d = dlist[i];
 
-		if (d && (d->d_type == DT_DIR)) {
+		if (d && (shim_dirent_type(path, d) == SHIM_DT_DIR)) {
 			(void)stress_mk_filename(tmp, sizeof(tmp), path, d->d_name);
 
 			free(d);
@@ -709,7 +714,7 @@ static char *stress_random_pid(void)
 	int i, n;
 	size_t j;
 
-	(void)shim_strlcpy(path, "/proc/self", sizeof(path));
+	(void)shim_strscpy(path, "/proc/self", sizeof(path));
 
 	n = stress_proc_scandir("/proc", &dlist, NULL, mixup_sort);
 	if (!n) {
@@ -728,7 +733,7 @@ static char *stress_random_pid(void)
 
 		name = dlist[j]->d_name;
 
-		if (isdigit(name[0])) {
+		if (isdigit((unsigned char)name[0])) {
 			(void)stress_mk_filename(path, sizeof(path), "/proc", name);
 			break;
 		}
@@ -767,7 +772,7 @@ static int stress_procfs(stress_args_t *args)
 
 	(void)sigfillset(&set);
 
-	(void)shim_strlcpy(proc_path, "/proc/self", sizeof(proc_path));
+	(void)shim_strscpy(proc_path, "/proc/self", sizeof(proc_path));
 
 	ctxt.args = args;
 	ctxt.writeable = (geteuid() != 0);
@@ -786,6 +791,8 @@ static int stress_procfs(stress_args_t *args)
 				stress_proc_rw_thread, &ctxt);
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -794,20 +801,22 @@ static int stress_procfs(stress_args_t *args)
 		for (i = 0; i < n; i++) {
 			char procfspath[PATH_MAX];
 			const struct dirent *d = dlist[i];
+			unsigned char type;
 
 			if (!stress_continue(args))
 				break;
 
 			stress_mk_filename(procfspath, sizeof(procfspath), "/proc", d->d_name);
-			if ((d->d_type == DT_REG) || (d->d_type == DT_LNK)) {
+			type = shim_dirent_type("/proc", d);
+			if ((type == SHIM_DT_REG) || (type == SHIM_DT_LNK)) {
 				if (!shim_pthread_spin_lock(&lock)) {
-					(void)shim_strlcpy(proc_path, procfspath, sizeof(proc_path));
+					(void)shim_strscpy(proc_path, procfspath, sizeof(proc_path));
 					(void)shim_pthread_spin_unlock(&lock);
 
 					stress_proc_rw(&ctxt, 8);
 					stress_bogo_inc(args);
 				}
-			} else if (d->d_type == DT_DIR) {
+			} else if (type == SHIM_DT_DIR) {
 				stress_proc_dir(&ctxt, procfspath, true, 0);
 			}
 
@@ -827,7 +836,7 @@ static int stress_procfs(stress_args_t *args)
 	if (rc) {
 		pr_dbg("%s: spin lock failed for %s\n", args->name, proc_path);
 	} else {
-		(void)shim_strlcpy(proc_path, "", sizeof(proc_path));
+		(void)shim_strscpy(proc_path, "", sizeof(proc_path));
 		VOID_RET(int, shim_pthread_spin_unlock(&lock));
 	}
 
@@ -846,13 +855,13 @@ static int stress_procfs(stress_args_t *args)
 	return EXIT_SUCCESS;
 }
 
-stressor_info_t stress_procfs_info = {
+const stressor_info_t stress_procfs_info = {
 	.stressor = stress_procfs,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.help = help
 };
 #else
-stressor_info_t stress_procfs_info = {
+const stressor_info_t stress_procfs_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.help = help,
