@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-mincore.h"
+#include "core-out-of-memory.h"
 
 static volatile bool do_jmp = true;
 static sigjmp_buf jmp_env;
@@ -139,7 +141,7 @@ static void MLOCKED_TEXT stress_rlimit_handler(int signum)
 }
 
 
-static int stress_rlimit_child(const stress_args_t *args, void *ctxt)
+static int stress_rlimit_child(stress_args_t *args, void *ctxt)
 {
 	stress_rlimit_context_t *context = (stress_rlimit_context_t *)ctxt;
 	uint8_t *stack;
@@ -156,7 +158,10 @@ static int stress_rlimit_child(const stress_args_t *args, void *ctxt)
 		(void)munmap((void *)stack, STRESS_MINSIGSTKSZ);
 		return EXIT_NO_RESOURCE;
 	}
+	stress_set_vma_anon_name(stack, STRESS_MINSIGSTKSZ, "stack");
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	/* Child rlimit stressor */
@@ -170,11 +175,11 @@ static int stress_rlimit_child(const stress_args_t *args, void *ctxt)
 		 */
 		for (i = 0; i < SIZEOF_ARRAY(resource_ids); i++) {
 			ret = getrlimit(resource_ids[i].resource, &rlim);
-			if (ret < 0)
+			if (UNLIKELY(ret < 0))
 				continue;
 
 			ret = setrlimit(resource_ids[i].resource, &rlim);
-			if (ret < 0) {
+			if (UNLIKELY(ret < 0)) {
 				pr_fail("%s: setrlimit %s failed, errno=%d (%s)\n",
 					args->name, resource_ids[i].name,
 					errno, strerror(errno));
@@ -197,13 +202,13 @@ static int stress_rlimit_child(const stress_args_t *args, void *ctxt)
 		ret = sigsetjmp(jmp_env, 1);
 
 		/* Check for timer overrun */
-		if ((stress_time_now() - context->start) > (double)g_opt_timeout)
+		if (UNLIKELY((stress_time_now() - context->start) > (double)g_opt_timeout))
 			break;
 		/* Check for counter limit reached */
-		if (args->max_ops && get_counter(args) >= args->max_ops)
+		if (UNLIKELY(args->max_ops && (stress_bogo_get(args) >= args->max_ops)))
 			break;
 
-		if (ret == 0) {
+		if (LIKELY(ret == 0)) {
 			uint8_t *ptr;
 			void *oldbrk;
 			int fds[MAX_RLIMIT_NOFILE];
@@ -221,7 +226,7 @@ static int stress_rlimit_child(const stress_args_t *args, void *ctxt)
 				ptr = (uint8_t *)mmap(NULL, MAX_RLIMIT_AS, PROT_READ | PROT_WRITE,
 					MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 				if (ptr != MAP_FAILED)
-					(void)munmap((void *)ptr, MAX_RLIMIT_AS);
+					(void)stress_munmap_retry_enomem((void *)ptr, MAX_RLIMIT_AS);
 				break;
 			case 2:
 				/* Trigger RLIMIT_DATA */
@@ -246,18 +251,15 @@ static int stress_rlimit_child(const stress_args_t *args, void *ctxt)
 				for (i = 0; i < MAX_RLIMIT_NOFILE; i++) {
 					fds[i] = open("/dev/null", O_RDONLY);
 				}
-				for (i = 0; i < MAX_RLIMIT_NOFILE; i++) {
-					if (fds[i] != -1)
-						(void)close(fds[i]);
-				}
+				stress_close_fds(fds, MAX_RLIMIT_NOFILE);
 				break;
 			}
 		} else if (ret == 1) {
-			inc_counter(args);	/* SIGSEGV/SIGILL occurred */
+			stress_bogo_inc(args);	/* SIGSEGV/SIGILL occurred */
 		} else {
 			break;		/* Something went wrong! */
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -271,7 +273,7 @@ static int stress_rlimit_child(const stress_args_t *args, void *ctxt)
  *  stress_rlimit
  *	stress by generating rlimit signals
  */
-static int stress_rlimit(const stress_args_t *args)
+static int stress_rlimit(stress_args_t *args)
 {
 	struct sigaction old_action_xcpu, old_action_xfsz, old_action_segv;
 	size_t i;
@@ -317,7 +319,7 @@ static int stress_rlimit(const stress_args_t *args)
 	return ret;
 }
 
-stressor_info_t stress_rlimit_info = {
+const stressor_info_t stress_rlimit_info = {
 	.stressor = stress_rlimit,
 	.class = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
