@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,7 +19,12 @@
  */
 #include "stress-ng.h"
 #include "core-arch.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
+#include "core-madvise.h"
+#include "core-mincore.h"
+#include "core-out-of-memory.h"
+#include "core-pragma.h"
 
 #if defined(HAVE_LINUX_MEMPOLICY_H)
 #include <linux/mempolicy.h>
@@ -39,6 +44,16 @@ UNEXPECTED
 UNEXPECTED
 #endif
 
+#if !defined(SHM_HUGE_SHIFT)
+#define SHM_HUGE_SHIFT	(26)
+#endif
+#if !defined(SHM_HUGE_2MB)
+#define SHM_HUGE_2MB	(21 << SHM_HUGE_SHIFT)
+#endif
+#if !defined(SHM_HUGE_1GB)
+#define SHM_HUGE_1GB	(30 << SHM_HUGE_SHIFT)
+#endif
+
 #define MIN_SHM_SYSV_BYTES	(1 * MB)
 #define MAX_SHM_SYSV_BYTES	(256 * MB)
 #define DEFAULT_SHM_SYSV_BYTES	(8 * MB)
@@ -50,6 +65,7 @@ UNEXPECTED
 static const stress_help_t help[] = {
 	{ NULL,	"shm-sysv N",		"start N workers that exercise System V shared memory" },
 	{ NULL,	"shm-sysv-bytes N",	"allocate and free N bytes of shared memory per loop" },
+	{ NULL,	"shm-sysv-mlock",	"attempt to mlock pages into memory" },
 	{ NULL,	"shm-sysv-ops N",	"stop after N shared memory bogo operations" },
 	{ NULL,	"shm-sysv-segs N",	"allocate N shared memory segments per iteration" },
 	{ NULL,	NULL,			NULL }
@@ -61,7 +77,7 @@ static const stress_help_t help[] = {
 
 #define KEY_GET_RETRIES		(40)
 #define BITS_PER_BYTE		(8)
-#define NUMA_LONG_BITS		(sizeof(unsigned long) * BITS_PER_BYTE)
+#define NUMA_LONG_BITS		(sizeof(unsigned long int) * BITS_PER_BYTE)
 #if !defined(MPOL_F_ADDR)
 #define MPOL_F_ADDR		(1 << 1)
 #endif
@@ -112,66 +128,52 @@ static const int shm_flags[] = {
 };
 #endif
 
-static int stress_set_shm_sysv_bytes(const char *opt)
-{
-	size_t shm_sysv_bytes;
-
-	shm_sysv_bytes = (size_t)stress_get_uint64_byte(opt);
-	stress_check_range_bytes("shm-sysv-bytes", shm_sysv_bytes,
-		MIN_SHM_SYSV_BYTES, MAX_MEM_LIMIT);
-	return stress_set_setting("shm-sysv-bytes", TYPE_ID_SIZE_T, &shm_sysv_bytes);
-}
-
-static int stress_set_shm_sysv_segments(const char *opt)
-{
-	size_t shm_sysv_segments;
-
-	shm_sysv_segments = (size_t)stress_get_uint64(opt);
-	stress_check_range("shm-sysv-segs", shm_sysv_segments,
-		MIN_SHM_SYSV_SEGMENTS, MAX_SHM_SYSV_SEGMENTS);
-	return stress_set_setting("shm-sysv-segs", TYPE_ID_SIZE_T, &shm_sysv_segments);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_shm_bytes,		stress_set_shm_sysv_bytes },
-	{ OPT_shm_sysv_segments,	stress_set_shm_sysv_segments },
-	{ 0,				NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_shm_sysv_bytes, "shm-sysv-bytes", TYPE_ID_SIZE_T_BYTES_VM, MIN_SHM_SYSV_BYTES, MAX_MEM_LIMIT, NULL },
+	{ OPT_shm_sysv_mlock, "shm-sysv-mlock", TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_shm_sysv_segs,  "shm-sysv-segs",  TYPE_ID_SIZE_T, MIN_SHM_SYSV_SEGMENTS, MAX_SHM_SYSV_SEGMENTS, NULL },
+	END_OPT,
 };
 
-#if defined(HAVE_SHM_SYSV)
+#if defined(HAVE_SYS_IPC_H) &&	\
+    defined(HAVE_SYS_SHM_H) &&	\
+    defined(HAVE_SHM_SYSV)
 
 static void stress_shm_metrics(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const double duration,
 	const double count,
-	const char *syscall,
-	const int index)
+	const char *syscall_name,
+	const int idx)
 {
 	char buffer[40];
 	const double rate = (count > 0.0) ? (duration / count) : 0.0;
 
-	(void)snprintf(buffer, sizeof(buffer), "nanosecs per %s call", syscall);
-	stress_metrics_set(args, index, buffer, STRESS_DBL_NANOSECOND * rate);
+	(void)snprintf(buffer, sizeof(buffer), "nanosecs per %s call", syscall_name);
+	stress_metrics_set(args, idx, buffer,
+		STRESS_DBL_NANOSECOND * rate, STRESS_METRIC_HARMONIC_MEAN);
 }
 
 /*
  *  stress_shm_sysv_check()
  *	simple check if shared memory is sane
  */
-static int stress_shm_sysv_check(
+static int OPTIMIZE3 stress_shm_sysv_check(
 	uint8_t *buf,
 	const size_t sz,
 	const size_t page_size)
 {
-	uint8_t *ptr, *end = buf + sz;
-	uint8_t val;
+	uint8_t *ptr, val;
+	const uint8_t *end = buf + sz;
 
+PRAGMA_UNROLL_N(4)
 	for (val = 0, ptr = buf; ptr < end; ptr += page_size, val++) {
 		*ptr = val;
 	}
 
+PRAGMA_UNROLL_N(4)
 	for (val = 0, ptr = buf; ptr < end; ptr += page_size, val++) {
-		if (*ptr != val)
+		if (UNLIKELY(*ptr != val))
 			return -1;
 
 	}
@@ -193,12 +195,12 @@ static void exercise_shmat(
 
 	/* Exercise shmat syscall on invalid shm_id */
 	addr = shmat(-1, NULL, 0);
-	if (addr != (void *)-1)
+	if (UNLIKELY(addr != (void *)-1))
 		(void)shmdt(addr);
 
 	/* Exercise shmat syscall on invalid flags */
 	addr = shmat(shm_id, NULL, ~0);
-	if (addr != (void *)-1)
+	if (UNLIKELY(addr != (void *)-1))
 		(void)shmdt(addr);
 
 	/* Exercise valid shmat with all possible values of flags */
@@ -229,6 +231,7 @@ static void exercise_shmat(
 		if (addr != (void *)-1) {
 			/* Exercise remap onto itself */
 			void *remap = shmat(shm_id, addr, SHM_REMAP | SHM_RDONLY);
+
 			if (remap != (void *)-1)
 				(void)shmdt(remap);
 			if (addr != remap)
@@ -245,7 +248,7 @@ static void exercise_shmat(
 #if defined(SHM_RND)
 	addr = mmap(NULL, sz, PROT_READ,
 		    MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
-	if (addr != MAP_FAILED) {
+	if (LIKELY(addr != MAP_FAILED)) {
 		(void)munmap(addr, sz);
 		addr = shmat(shm_id, addr, SHM_RND);
 		if (addr != (void *)-1)
@@ -258,9 +261,8 @@ static void exercise_shmat(
 	/* Exercise shmat with SHM_REMAP flag on NULL address */
 #if defined(SHM_REMAP)
 	addr = shmat(shm_id, NULL, SHM_REMAP);
-	if (addr != (void *)-1) {
+	if (addr != (void *)-1)
 		(void)shmdt(addr);
-	}
 #else
 	UNEXPECTED
 #endif
@@ -292,11 +294,11 @@ static void exercise_shmat(
  *  get_bad_shmid()
  *	find invalid shared memory segment id
  */
-static int get_bad_shmid(const stress_args_t *args)
+static int get_bad_shmid(stress_args_t *args)
 {
 	int id = ~0;
 
-	while (keep_stressing(args)) {
+	while (stress_continue(args)) {
 		int ret;
 		struct shmid_ds ds;
 
@@ -316,7 +318,7 @@ static int get_bad_shmid(const stress_args_t *args)
  *  exercise_shmctl()
  *	exercise shmctl syscall with all possible values of arguments
  */
-static void exercise_shmctl(const size_t sz, const stress_args_t *args)
+static void exercise_shmctl(const size_t sz, stress_args_t *args)
 {
 	key_t key;
 	int shm_id;
@@ -353,7 +355,7 @@ static void exercise_shmctl(const size_t sz, const stress_args_t *args)
 		int ret;
 
 		ret = shmctl(shm_id, IPC_STAT, &buf);
-		if ((ret >= 0) && (errno == 0))
+		if (UNLIKELY((ret >= 0) && (errno == 0)))
 			pr_fail("%s: shmctl IPC_STAT unexpectedly succeeded on non-existent shared "
 				"memory segment, errno=%d (%s)\n", args->name, errno, strerror(errno));
 	}
@@ -366,7 +368,7 @@ static void exercise_shmctl(const size_t sz, const stress_args_t *args)
  *  exercise_shmget()
  *	exercise shmget syscall with all possible values of arguments
  */
-static void exercise_shmget(const size_t sz, const char *name, const bool cap_ipc_lock)
+static void exercise_shmget(const size_t sz, const char *name)
 {
 	key_t key;
 	int shm_id;
@@ -387,7 +389,7 @@ static void exercise_shmget(const size_t sz, const char *name, const bool cap_ip
 		 * existing shared memory segment and IPC_EXCL flag
 		 */
 		shm_id2 = shmget(key, sz, IPC_CREAT | IPC_EXCL);
-		if ((shm_id2 >= 0) && (errno == 0)) {
+		if (UNLIKELY((shm_id2 >= 0) && (errno == 0))) {
 			pr_fail("%s: shmget IPC_CREAT unexpectedly succeeded and re-created "
 				"shared memory segment even with IPC_EXCL flag "
 				"specified, errno=%d (%s)\n", name, errno, strerror(errno));
@@ -399,8 +401,8 @@ static void exercise_shmget(const size_t sz, const char *name, const bool cap_ip
 		 * existing shared memory segment but of greater size
 		 */
 		shm_id2 = shmget(key, sz + (1024 * 1024), IPC_CREAT);
-		if ((shm_id2 >= 0) && (errno == 0)) {
-			pr_fail("%s: shmget IPC_RMID unexpectedly succeeded and again "
+		if (UNLIKELY((shm_id2 >= 0) && (errno == 0))) {
+			pr_fail("%s: shmget IPC_CREAT unexpectedly succeeded and again "
 				"created shared memory segment with a greater "
 				"size, errno=%d (%s)\n", name, errno, strerror(errno));
 			(void)shmctl(shm_id2, IPC_RMID, NULL);
@@ -412,8 +414,8 @@ static void exercise_shmget(const size_t sz, const char *name, const bool cap_ip
     /* Exercise shmget on invalid sizes argument*/
 #if defined(SHMMIN)
 	shm_id = shmget(key, SHMMIN - 1, IPC_CREAT);
-	if ((SHMMIN > 0) && (shm_id >= 0)) {
-		pr_fail("%s: shmget IPC_RMID unexpectedly succeeded on invalid value of"
+	if (UNLIKELY((SHMMIN > 0) && (shm_id >= 0))) {
+		pr_fail("%s: shmget IPC_CREAT unexpectedly succeeded on invalid value of"
 			"size argument, errno=%d (%s)\n", name, errno, strerror(errno));
 		(void)shmctl(shm_id, IPC_RMID, NULL);
 	}
@@ -423,27 +425,40 @@ static void exercise_shmget(const size_t sz, const char *name, const bool cap_ip
 
 #if defined(SHMMAX)
 	shm_id = shmget(key, SHMMAX + 1, IPC_CREAT);
-	if (SHMMAX < ~(size_t)0) && (shm_id >= 0)) {
-		pr_fail("%s: shmget IPC_RMID unexpectedly succeeded on invalid value of"
+	if (UNLIKELY((SHMMAX < ~(size_t)0) && (shm_id >= 0))) {
+		pr_fail("%s: shmget IPC_CREAT unexpectedly succeeded on invalid value of"
 			"size argument, errno=%d (%s)\n", name, errno, strerror(errno));
 		(void)shmctl(shm_id, IPC_RMID, NULL);
 	}
-#else
-	/* UNEXPECTED */
-#endif
+#elif defined(__linux__)
+	{
+		char buf[32];
 
-#if defined(SHM_HUGETLB)
-	/* Check shmget cannot succeed without capabilities */
-	if (!cap_ipc_lock) {
-		shm_id = shmget(IPC_PRIVATE, sz, IPC_CREAT | SHM_HUGETLB | SHM_R | SHM_W);
-		if (shm_id >= 0) {
-			pr_fail("%s: shmget IPC_RMID unexpectedly succeeded on without suitable"
-				"capability, errno=%d (%s)\n", name, errno, strerror(errno));
-			(void)shmctl(shm_id, IPC_RMID, NULL);
+		/* Find size from shmmax proc value */
+		if (stress_system_read("/proc/sys/kernel/shmmax", buf, sizeof(buf)) > 0) {
+			size_t shmmax;
+
+			if (sscanf(buf, "%zu", &shmmax) == 1) {
+				shmmax++;
+				if (shmmax > 0) {
+					shm_id = shmget(key, shmmax, IPC_CREAT);
+					if (UNLIKELY(shm_id >= 0)) {
+						pr_fail("%s: shmget IPC_CREAT unexpectedly succeeded on "
+							"invalid value %zu of size argument, errno=%d (%s)\n",
+							name, shmmax, errno, strerror(errno));
+						(void)shmctl(shm_id, IPC_RMID, NULL);
+					}
+				}
+			}
 		}
 	}
-#else
-	(void)cap_ipc_lock;
+#endif
+
+#if defined(SHM_HUGETLB) &&	\
+    defined(SHM_HUGE_2MB)
+	shm_id = shmget(IPC_PRIVATE, sz, IPC_CREAT | SHM_HUGETLB | SHM_HUGE_2MB | SHM_R | SHM_W);
+	if (shm_id >= 0)
+		(void)shmctl(shm_id, IPC_RMID, NULL);
 #endif
 
 #if defined(IPC_PRIVATE)
@@ -455,7 +470,7 @@ static void exercise_shmget(const size_t sz, const char *name, const bool cap_ip
 #endif
 
 	shm_id = shmget(key, sz, IPC_EXCL);
-	if ((shm_id >= 0) && (errno == 0)) {
+	if (UNLIKELY((shm_id >= 0) && (errno == 0))) {
 		pr_fail("%s: shmget IPC_RMID unexpectedly succeeded on non-existent shared "
 			"memory segment, errno=%d (%s)\n", name, errno, strerror(errno));
 		(void)shmctl(shm_id, IPC_RMID, NULL);
@@ -552,11 +567,12 @@ static void stress_shm_sysv_linux_proc_map(const void *addr, const size_t sz)
  *	be reaped cleanly if this process gets prematurely killed.
  */
 static int stress_shm_sysv_child(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const size_t max_sz,
 	const size_t page_size,
-	const size_t shm_sysv_segments)
+	const size_t shm_sysv_segments,
+	const bool shm_sysv_mlock)
 {
 	void *addrs[MAX_SHM_SYSV_SEGMENTS];
 	key_t keys[MAX_SHM_SYSV_SEGMENTS];
@@ -567,14 +583,13 @@ static int stress_shm_sysv_child(
 	bool ok = true;
 	int mask = ~0;
 	uint32_t instances = args->num_instances;
-	const bool cap_ipc_lock = stress_check_capability(SHIM_CAP_IPC_LOCK);
 	const size_t buffer_size = (page_size / sizeof(uint64_t)) + 1;
 	uint64_t *buffer;
 	double shmget_duration = 0.0, shmget_count = 0.0;
 	double shmat_duration = 0.0, shmat_count = 0.0;
 	double shmdt_duration = 0.0, shmdt_count = 0.0;
 
-	buffer = calloc(buffer_size, sizeof(*buffer));
+	buffer = (uint64_t *)calloc(buffer_size, sizeof(*buffer));
 	if (!buffer) {
 		pr_inf_skip("%s: cannot allocate %zu sized buffer, skipping stressor\n",
 			args->name, buffer_size);
@@ -586,19 +601,19 @@ static int stress_shm_sysv_child(
 		return EXIT_FAILURE;
 	}
 
-	(void)memset(addrs, 0, sizeof(addrs));
-	(void)memset(keys, 0, sizeof(keys));
+	(void)shim_memset(addrs, 0, sizeof(addrs));
+	(void)shim_memset(keys, 0, sizeof(keys));
 	for (i = 0; i < MAX_SHM_SYSV_SEGMENTS; i++)
 		shm_ids[i] = -1;
 
 	/* Make sure this is killable by OOM killer */
-	stress_set_oom_adjustment(args->name, true);
+	stress_set_oom_adjustment(args, true);
 
 	do {
 		size_t sz = max_sz;
 		pid_t pid = -1;
 
-		exercise_shmget(sz, args->name, cap_ipc_lock);
+		exercise_shmget(sz, args->name);
 		exercise_shmctl(sz, args);
 
 		for (i = 0; i < shm_sysv_segments; i++) {
@@ -616,7 +631,7 @@ static int stress_shm_sysv_child(
 				sz = shmall;
 			if ((freemem > page_size) && sz > freemem)
 				sz = freemem;
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				goto reap;
 
 			for (count = 0; count < KEY_GET_RETRIES; count++) {
@@ -632,7 +647,7 @@ static int stress_shm_sysv_child(
 					size_t j;
 					unique = true;
 
-					if (!keep_stressing_flag())
+					if (UNLIKELY(!stress_continue_flag()))
 						goto reap;
 
 					/* Get a unique random key */
@@ -643,7 +658,7 @@ static int stress_shm_sysv_child(
 							break;
 						}
 					}
-					if (!keep_stressing_flag())
+					if (UNLIKELY(!stress_continue_flag()))
 						goto reap;
 				} while (!unique);
 
@@ -651,7 +666,7 @@ static int stress_shm_sysv_child(
 				shm_id = shmget(key, sz,
 					IPC_CREAT | IPC_EXCL |
 					S_IRUSR | S_IWUSR | rnd_flag);
-				if (shm_id >= 0) {
+				if (LIKELY(shm_id >= 0)) {
 					shmget_duration += stress_time_now() - t;
 					shmget_count += 1.0;
 					break;
@@ -670,7 +685,21 @@ static int stress_shm_sysv_child(
 					sz = sz / 2;
 				}
 			}
-			if (shm_id < 0) {
+			if (UNLIKELY(shm_id < 0)) {
+				/* Run out of shm segments, just reap and die */
+				if (errno == ENOSPC) {
+					pr_inf_skip("%s: shmget ran out of free space, "
+						"skipping stressor\n", args->name);
+					rc = EXIT_NO_RESOURCE;
+					ok = false;
+					goto reap;
+				}
+				/* Run out of shm space, or existing key, so reap, die, repawn */
+				if ((errno == ENOMEM) || (errno == EEXIST)) {
+					rc = EXIT_SUCCESS;
+					goto reap;
+				}
+				/* Some unexpected failures handler here */
 				ok = false;
 				pr_fail("%s: shmget failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
@@ -681,7 +710,7 @@ static int stress_shm_sysv_child(
 			/* Inform parent of the new shm ID */
 			msg.index = (int)i;
 			msg.shm_id = shm_id;
-			if (write(fd, &msg, sizeof(msg)) < 0) {
+			if (UNLIKELY(write(fd, &msg, sizeof(msg)) < 0)) {
 				pr_err("%s: write failed: errno=%d: (%s)\n",
 					args->name, errno, strerror(errno));
 				rc = EXIT_FAILURE;
@@ -690,39 +719,33 @@ static int stress_shm_sysv_child(
 
 			t = stress_time_now();
 			addr = shmat(shm_id, NULL, 0);
-			if (addr == (char *) -1) {
+			if (UNLIKELY(addr == (char *)-1)) {
 				ok = false;
-				pr_fail("%s: shmat failed on id %d, errno=%d (%s)\n",
-					args->name, shm_id, errno, strerror(errno));
+				pr_fail("%s: shmat on NULL address failed on id %d, (key=%d, size=%zd), errno=%d (%s)\n",
+					args->name, shm_id, (int)key, sz, errno, strerror(errno));
 				rc = EXIT_FAILURE;
 				goto reap;
 			}
+			if (shm_sysv_mlock)
+				(void)shim_mlock(addr, sz);
 			shmat_duration += stress_time_now() - t;
 			shmat_count += 1.0;
 			addrs[i] = addr;
 			shm_ids[i] = shm_id;
 			keys[i] = key;
 
-			if (!keep_stressing(args))
+			if (UNLIKELY(!stress_continue(args)))
 				goto reap;
 			(void)stress_mincore_touch_pages(addr, sz);
 			(void)shim_msync(addr, sz, stress_mwc1() ? MS_ASYNC : MS_SYNC);
 
-#if defined(_POSIX_MEMLOCK_RANGE) &&   \
-    defined(HAVE_MLOCK)
-			/*
-			 *  Exercise mlock on 1st page of shm
-			 */
-			(void)shim_mlock(addr, 4096);
-#endif
-
-			if (!keep_stressing(args))
+			if (UNLIKELY(!stress_continue(args)))
 				goto reap;
 			(void)stress_madvise_random(addr, sz);
 
-			if (!keep_stressing(args))
+			if (UNLIKELY(!stress_continue(args)))
 				goto reap;
-			if (stress_shm_sysv_check(addr, sz, page_size) < 0) {
+			if (UNLIKELY(stress_shm_sysv_check(addr, sz, page_size) < 0)) {
 				ok = false;
 				pr_fail("%s: memory check failed\n", args->name);
 				rc = EXIT_FAILURE;
@@ -746,7 +769,7 @@ static int stress_shm_sysv_child(
 			{
 				struct shmid_ds ds;
 
-				if (shmctl(shm_id, IPC_STAT, &ds) < 0)
+				if (UNLIKELY(shmctl(shm_id, IPC_STAT, &ds) < 0))
 					pr_fail("%s: shmctl IPC_STAT failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 #if defined(SHM_SET)
@@ -764,7 +787,7 @@ static int stress_shm_sysv_child(
 			{
 				struct shminfo s;
 
-				if (shmctl(shm_id, IPC_INFO, (struct shmid_ds *)&s) < 0)
+				if (UNLIKELY(shmctl(shm_id, IPC_INFO, (struct shmid_ds *)&s) < 0))
 					pr_fail("%s: shmctl IPC_INFO failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 			}
@@ -776,7 +799,7 @@ static int stress_shm_sysv_child(
 			{
 				struct shm_info s;
 
-				if (shmctl(shm_id, SHM_INFO, (struct shmid_ds *)&s) < 0)
+				if (UNLIKELY(shmctl(shm_id, SHM_INFO, (struct shmid_ds *)&s) < 0))
 					pr_fail("%s: shmctl SHM_INFO failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 			}
@@ -798,7 +821,7 @@ static int stress_shm_sysv_child(
     defined(__NR_set_mempolicy)
 			{
 				int ret, mode;
-				unsigned long node_mask[NUMA_LONG_BITS];
+				unsigned long int node_mask[NUMA_LONG_BITS];
 
 				ret = shim_get_mempolicy(&mode, node_mask, 1,
 					addrs[i], MPOL_F_ADDR);
@@ -811,7 +834,7 @@ static int stress_shm_sysv_child(
 			stress_shm_sysv_linux_proc_map(addr, sz);
 #endif
 			exercise_shmat(shm_id, sz, buffer);
-			inc_counter(args);
+			stress_bogo_inc(args);
 		}
 
 		pid = fork();
@@ -843,12 +866,8 @@ reap:
 			if (addrs[i]) {
 				double t;
 
-#if defined(_POSIX_MEMLOCK_RANGE) &&   \
-    defined(HAVE_MLOCK)
-				(void)shim_munlock(addrs[i], 4096);
-#endif
 				t = stress_time_now();
-				if (shmdt(addrs[i]) < 0) {
+				if (UNLIKELY(shmdt(addrs[i]) < 0)) {
 					pr_fail("%s: shmdt failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 				} else {
@@ -858,7 +877,7 @@ reap:
 			}
 			if (shm_ids[i] >= 0) {
 				if (shmctl(shm_ids[i], IPC_RMID, NULL) < 0) {
-					if ((errno != EIDRM) && (errno != EINVAL)) {
+					if (UNLIKELY((errno != EIDRM) && (errno != EINVAL))) {
 						pr_fail("%s: shmctl IPC_RMID failed, errno=%d (%s)\n",
 							args->name, errno, strerror(errno));
 					}
@@ -868,7 +887,7 @@ reap:
 			/* Inform parent shm ID is now free */
 			msg.index = (int)i;
 			msg.shm_id = -1;
-			if (write(fd, &msg, sizeof(msg)) < 0) {
+			if (UNLIKELY(write(fd, &msg, sizeof(msg)) < 0)) {
 				pr_dbg("%s: write failed: errno=%d: (%s)\n",
 					args->name, errno, strerror(errno));
 				ok = false;
@@ -883,7 +902,7 @@ reap:
 
 			(void)waitpid(pid, &status, 0);
 		}
-	} while (ok && keep_stressing(args));
+	} while (ok && stress_continue(args));
 
 	/* Inform parent of end of run */
 	msg.index = -1;
@@ -906,7 +925,7 @@ reap:
  *  stress_shm_sysv()
  *	stress SYSTEM V shared memory
  */
-static int stress_shm_sysv(const stress_args_t *args)
+static int stress_shm_sysv(stress_args_t *args)
 {
 	const size_t page_size = args->page_size;
 	size_t orig_sz, sz;
@@ -915,10 +934,13 @@ static int stress_shm_sysv(const stress_args_t *args)
 	ssize_t i;
 	pid_t pid;
 	bool retry = true;
+	bool shm_sysv_mlock = false;
 	uint32_t restarts = 0;
 	size_t shm_sysv_bytes = DEFAULT_SHM_SYSV_BYTES;
 	size_t shm_sysv_segments = DEFAULT_SHM_SYSV_SEGMENTS;
 
+	(void)stress_get_setting("shm-sysv-mlock", &shm_sysv_mlock);
+
 	if (!stress_get_setting("shm-sysv-bytes", &shm_sysv_bytes)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
 			shm_sysv_bytes = MAX_SHM_SYSV_BYTES;
@@ -940,9 +962,11 @@ static int stress_shm_sysv(const stress_args_t *args)
 
 	orig_sz = sz = shm_sysv_bytes & ~(page_size - 1);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
-	while (keep_stressing_flag() && retry) {
+	while (stress_continue_flag() && retry) {
 		if (pipe(pipefds) < 0) {
 			pr_fail("%s: pipe failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
@@ -968,13 +992,13 @@ fork_again:
 			bool get_procinfo = true;
 #endif
 
-			stress_set_oom_adjustment(args->name, false);
+			stress_set_oom_adjustment(args, false);
 			(void)close(pipefds[1]);
 
 			for (i = 0; i < (ssize_t)MAX_SHM_SYSV_SEGMENTS; i++)
 				shm_ids[i] = -1;
 
-			while (keep_stressing_flag()) {
+			while (stress_continue_flag()) {
 				stress_shm_msg_t msg;
 				ssize_t n;
 
@@ -1007,7 +1031,7 @@ fork_again:
 					stress_shm_get_procinfo(&get_procinfo);
 #endif
 			}
-			(void)kill(pid, SIGALRM);
+			(void)shim_kill(pid, SIGALRM);
 			(void)shim_waitpid(pid, &status, 0);
 			if (WIFSIGNALED(status)) {
 				if ((WTERMSIG(status) == SIGKILL) ||
@@ -1042,12 +1066,16 @@ fork_again:
 			 * Nicing the child may OOM it first as this
 			 * doubles the OOM score
 			 */
-			if (nice(5) < 0)
-				pr_dbg("%s: nice of child failed, "
-					"(instance %d)\n", args->name, args->instance);
+			errno = 0;
+			VOID_RET(int, nice(5));
+			if (errno != 0)
+				pr_dbg("%s: nice of child failed, errno=%d (%s) "
+					"(instance %d)\n", args->name,
+					errno, strerror(errno),
+					args->instance);
 
 			(void)close(pipefds[0]);
-			rc = stress_shm_sysv_child(args, pipefds[1], sz, page_size, shm_sysv_segments);
+			rc = stress_shm_sysv_child(args, pipefds[1], sz, page_size, shm_sysv_segments, shm_sysv_mlock);
 			(void)close(pipefds[1]);
 			_exit(rc);
 		}
@@ -1064,18 +1092,19 @@ fork_again:
 	return rc;
 }
 
-stressor_info_t stress_shm_sysv_info = {
+const stressor_info_t stress_shm_sysv_info = {
 	.stressor = stress_shm_sysv,
-	.class = CLASS_VM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.class = CLASS_VM | CLASS_OS | CLASS_IPC,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_shm_sysv_info = {
+const stressor_info_t stress_shm_sysv_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_VM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.class = CLASS_VM | CLASS_OS | CLASS_IPC,
+	.opts = opts,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without System V shared memory shmat() shmdt() system calls"
 };
