@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
+#include "core-out-of-memory.h"
 #include "core-put.h"
 
 #if defined(HAVE_UCONTEXT_H)
@@ -37,41 +40,73 @@ static const stress_help_t help[] = {
 
 #define MMAPSTACK_SIZE		(256 * KB)
 
+/*
+ *  stress_stack_check sanity check list
+ */
+typedef struct stress_stack_check {
+	struct stress_stack_check *prev;	/* Previous item on stack list */
+	struct stress_stack_check *self_addr;	/* Address of this struct to check */
+	uint32_t waste[2];			/* Waste data on stack */
+} stress_stack_check_t;
+
 static ucontext_t c_main, c_test;
 static uint8_t *stack_mmap;		/* mmap'd stack */
 static uintptr_t page_mask;
 static size_t page_size;
+static int check_status;		/* sanity check status */
 
 /*
  *  push values onto file backed mmap'd stack and
  *  force msync on the map'd region if page boundary
  *  has changed
  */
-static void stress_stackmmap_push_msync(void)
+static void stress_stackmmap_push_msync(stress_stack_check_t *prev_check)
 {
 	void *addr = (void *)(((uintptr_t)&addr) & page_mask);
 	static void *laddr;
-	uint32_t waste[2];
+	stress_stack_check_t check;
+	register stress_stack_check_t *ptr;
+	register int i;
+	static char name[] = "stackmmap";
+
+	check.prev = prev_check;
+	check.self_addr = &check;
 
 	/*
 	 * Ensure something is written to the stack that
 	 * won't get optimized away
 	 */
-	waste[0] = stress_mwc32();
-	stress_uint32_put(waste[0]);
-	waste[1] = stress_mwc32();
-	stress_uint32_put(waste[1]);
-	stress_uint64_put((uint64_t)(intptr_t)&waste);
+	check.waste[0] = stress_mwc32();
+	stress_uint32_put(check.waste[0]);
+	check.waste[1] = ~check.waste[0];
+	stress_uint32_put(check.waste[1]);
+	stress_uint64_put((uint64_t)(intptr_t)&check);
 
 	if (addr != laddr) {
-		(void)shim_msync(addr, page_size,
-			(stress_mwc8() & 1) ? MS_ASYNC : MS_SYNC);
+		(void)shim_msync(addr, page_size, stress_mwc1() ? MS_ASYNC : MS_SYNC);
 		laddr = addr;
 	}
-	if (keep_stressing_flag())
-		stress_stackmmap_push_msync();
 
-	stress_uint64_put((uint64_t)waste[1]);
+	for (i = 0, ptr = &check; (i < 256) && ptr; ptr = ptr->prev, i++) {
+		if (UNLIKELY(ptr->self_addr != ptr)) {
+			pr_inf("%s: sanity check address mismatch, got 0x%p, "
+				"expecting 0x%p\n", name, ptr, ptr->self_addr);
+			check_status = EXIT_FAILURE;
+			return;
+		}
+		if (UNLIKELY(ptr->waste[0] != ~(ptr->waste[1]))) {
+			pr_inf("%s: sanity check data mismatch, got 0x%" PRIx32
+				", expecting 0x%" PRIx32 "\n", name,
+				ptr->waste[0], ptr->waste[1]);
+			check_status = EXIT_FAILURE;
+			return;
+		}
+	}
+
+	if (LIKELY(stress_continue_flag()))
+		stress_stackmmap_push_msync(&check);
+
+	stress_uint32_put(check.waste[1]);
 }
 
 /*
@@ -79,19 +114,19 @@ static void stress_stackmmap_push_msync(void)
  */
 static void stress_stackmmap_push_start(void)
 {
-	stress_stackmmap_push_msync();
+	stress_stackmmap_push_msync(NULL);
 }
 
 /*
  *  stress_stackmmap
  *	stress a file memory map'd stack
  */
-static int stress_stackmmap(const stress_args_t *args)
+static int stress_stackmmap(stress_args_t *args)
 {
 	int fd, ret;
 	volatile int rc = EXIT_FAILURE;		/* could be clobbered */
 	char filename[PATH_MAX];
-	uint8_t *stack_sig;
+	NOCLOBBER uint8_t *stack_sig;
 	struct sigaction new_action;
 
 	page_size = args->page_size;
@@ -100,7 +135,7 @@ static int stress_stackmmap(const stress_args_t *args)
 	/* Create file back'd mmaping for the stack */
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0)
-		return exit_status(-ret);
+		return stress_exit_status(-ret);
 	(void)stress_temp_filename_args(args,
 		filename, sizeof(filename), stress_mwc32());
 
@@ -117,7 +152,7 @@ static int stress_stackmmap(const stress_args_t *args)
 		(void)close(fd);
 		goto tidy_dir;
 	}
-	stack_sig = (uint8_t *)mmap(NULL, STRESS_SIGSTKSZ,
+	stack_sig = (uint8_t *)stress_mmap_populate(NULL, STRESS_SIGSTKSZ,
 		PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (stack_sig == MAP_FAILED) {
 		pr_inf_skip("%s: skipping stressor, cannot mmap signal stack, "
@@ -127,6 +162,7 @@ static int stress_stackmmap(const stress_args_t *args)
 		(void)close(fd);
 		goto tidy_dir;
 	}
+	stress_set_vma_anon_name(stack_sig, STRESS_SIGSTKSZ, "altstack-anon");
 
 	stack_mmap = (uint8_t *)mmap(NULL, MMAPSTACK_SIZE,
 		PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
@@ -144,12 +180,13 @@ static int stress_stackmmap(const stress_args_t *args)
 		goto tidy_stack_sig;
 	}
 	(void)close(fd);
+	stress_set_vma_anon_name(stack_mmap, STRESS_SIGSTKSZ, "altstack-file");
 
 	if (shim_madvise((void *)stack_mmap, MMAPSTACK_SIZE, MADV_RANDOM) < 0) {
 		pr_dbg("%s: madvise failed: errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 	}
-	(void)memset(stack_mmap, 0, MMAPSTACK_SIZE);
+	(void)shim_memset(stack_mmap, 0, MMAPSTACK_SIZE);
 
 #if defined(HAVE_MPROTECT)
 	/*
@@ -161,7 +198,7 @@ static int stress_stackmmap(const stress_args_t *args)
 	UNEXPECTED
 #endif
 
-	(void)memset(&c_test, 0, sizeof(c_test));
+	(void)shim_memset(&c_test, 0, sizeof(c_test));
 	if (getcontext(&c_test) < 0) {
 		pr_fail("%s: getcontext failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -171,6 +208,8 @@ static int stress_stackmmap(const stress_args_t *args)
 	c_test.uc_stack.ss_size = MMAPSTACK_SIZE - (page_size * 2);
 	c_test.uc_link = &c_main;
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	/*
@@ -180,14 +219,16 @@ static int stress_stackmmap(const stress_args_t *args)
 	 */
 	do {
 		pid_t pid;
+
+		(void)stress_mwc32();
 again:
-		if (!keep_stressing_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 		pid = fork();
 		if (pid < 0) {
-			if (stress_redo_fork(errno))
+			if (stress_redo_fork(args, errno))
 				goto again;
-			if (!keep_stressing(args))
+			if (UNLIKELY(!stress_continue(args)))
 				goto finish;
 			pr_err("%s: fork failed: errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
@@ -195,25 +236,28 @@ again:
 			int status, waitret;
 
 			/* Parent, wait for child */
-			(void)setpgid(pid, g_pgrp);
 			waitret = shim_waitpid(pid, &status, 0);
 			if (waitret < 0) {
 				if (errno != EINTR)
 					pr_dbg("%s: waitpid(): errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
-				(void)kill(pid, SIGTERM);
-				(void)stress_killpid(pid);
-				(void)shim_waitpid(pid, &status, 0);
+				stress_kill_and_wait(args, pid, SIGTERM, false);
+			} else {
+				if (WIFEXITED(status)) {
+					if (WEXITSTATUS(status) != EXIT_SUCCESS) {
+						rc = status;
+						goto tidy_mmap;
+					}
+				}
 			}
 		} else {
 			/* Child */
 
-			(void)setpgid(0, g_pgrp);
 			stress_parent_died_alarm();
 			(void)sched_settings_apply(true);
 
 			/* Make sure this is killable by OOM killer */
-			stress_set_oom_adjustment(args->name, true);
+			stress_set_oom_adjustment(args, true);
 
 			/*
 			 *  We need to handle SEGV signals when we
@@ -221,11 +265,11 @@ again:
 			 *  an alternative signal handling stack
 			 *  is required because we ran out of stack
 			 */
-			(void)memset(&new_action, 0, sizeof new_action);
+			(void)shim_memset(&new_action, 0, sizeof new_action);
 			new_action.sa_handler = stress_sig_handler_exit;
 			(void)sigemptyset(&new_action.sa_mask);
 			new_action.sa_flags = SA_ONSTACK;
-			if (sigaction(SIGSEGV, &new_action, NULL) < 0)
+			if (UNLIKELY(sigaction(SIGSEGV, &new_action, NULL) < 0))
 				_exit(EXIT_FAILURE);
 
 			/*
@@ -233,16 +277,16 @@ again:
 			 *  to handle segfaults on an overrun
 			 *  mmap'd stack
 			 */
-			if (stress_sigaltstack(stack_sig, STRESS_SIGSTKSZ) < 0)
+			if (UNLIKELY(stress_sigaltstack(stack_sig, STRESS_SIGSTKSZ) < 0))
 				_exit(EXIT_FAILURE);
 
+			check_status = EXIT_SUCCESS;
 			(void)makecontext(&c_test, stress_stackmmap_push_start, 0);
 			(void)swapcontext(&c_main, &c_test);
-
-			_exit(0);
+			_exit(check_status);
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 finish:
 	rc = EXIT_SUCCESS;
@@ -258,15 +302,18 @@ tidy_dir:
 	return rc;
 }
 
-stressor_info_t stress_stackmmap_info = {
+const stressor_info_t stress_stackmmap_info = {
 	.stressor = stress_stackmmap,
 	.class = CLASS_VM | CLASS_MEMORY,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_stackmmap_info = {
-	.stressor = stress_not_implemented,
+const stressor_info_t stress_stackmmap_info = {
+	.stressor = stress_unimplemented,
 	.class = CLASS_VM | CLASS_MEMORY,
-	.help = help
+	.verify = VERIFY_ALWAYS,
+	.help = help,
+	.unimplemented_reason = "built without ucontext.h or swapcontext()"
 };
 #endif
