@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -38,7 +38,7 @@ typedef struct stress_str_args {
 /*
  *  the STR stress test has different classes of string stressors
  */
-typedef void (*stress_str_func)(stress_str_args_t *info);
+typedef size_t (*stress_str_func)(stress_args_t *args, stress_str_args_t *info);
 
 typedef struct {
 	const char 		*name;	/* human readable form of stressor */
@@ -53,14 +53,12 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		   NULL }
 };
 
-static const stress_str_method_info_t str_methods[];
-
 static inline void strchk(
 	stress_str_args_t *info,
 	const int ok,
 	const char *msg)
 {
-	if ((g_opt_flags & OPT_FLAGS_VERIFY) && (!ok)) {
+	if (UNLIKELY((g_opt_flags & OPT_FLAGS_VERIFY) && (!ok))) {
 		pr_fail("%s: %s did not return expected result\n",
 			info->name, msg);
 		info->failed = true;
@@ -76,7 +74,7 @@ static inline void strchk(
  *  stress_strcasecmp()
  *	stress on strcasecmp
  */
-static void stress_strcasecmp(stress_str_args_t *info)
+static size_t stress_strcasecmp(stress_args_t *args, stress_str_args_t *info)
 {
 	typedef int (*test_strcasecmp_t)(const char *s1, const char *s2);
 
@@ -86,7 +84,7 @@ static void stress_strcasecmp(stress_str_args_t *info)
 	const size_t len1 = info->len1;
 	register size_t i;
 
-	for (i = 1; keep_stressing_flag() && (i < len1); i++) {
+	for (i = 1; stress_continue_flag() && (i < len1); i++) {
 		STRCHK(info, 0 == test_strcasecmp(str1, str1));
 		STRCHK(info, 0 == test_strcasecmp(str2, str2));
 
@@ -100,6 +98,8 @@ static void stress_strcasecmp(stress_str_args_t *info)
 		STRCHK(info, 0 != test_strcasecmp(str1 + i, str2));
 		STRCHK(info, 0 != test_strcasecmp(str2, str1 + i));
 	}
+	stress_bogo_add(args, 9);
+	return i * 9;
 }
 #endif
 
@@ -108,7 +108,7 @@ static void stress_strcasecmp(stress_str_args_t *info)
  *  stress_strncasecmp()
  *	stress on strncasecmp
  */
-static void stress_strncasecmp(stress_str_args_t *info)
+static size_t stress_strncasecmp(stress_args_t *args, stress_str_args_t *info)
 {
 	typedef int (*test_strncasecmp_t)(const char *s1, const char *s2, size_t n);
 
@@ -119,7 +119,7 @@ static void stress_strncasecmp(stress_str_args_t *info)
 	const size_t len2 = info->len2;
 	register size_t i;
 
-	for (i = 1; keep_stressing_flag() && (i < len1); i++) {
+	for (i = 1; stress_continue_flag() && (i < len1); i++) {
 		STRCHK(info, 0 == test_strncasecmp(str1, str1, len1));
 		STRCHK(info, 0 == test_strncasecmp(str2, str2, len2));
 
@@ -133,15 +133,18 @@ static void stress_strncasecmp(stress_str_args_t *info)
 		STRCHK(info, 0 != test_strncasecmp(str1 + i, str2, len1));
 		STRCHK(info, 0 != test_strncasecmp(str2, str1 + i, len2));
 	}
+	stress_bogo_add(args, 9);
+	return i * 9;
 }
 #endif
 
-#if defined(HAVE_STRINGS_H)
+#if defined(HAVE_STRINGS_H) &&	\
+    defined(HAVE_INDEX)
 /*
  *  stress_index()
  *	stress on index
  */
-static void stress_index(stress_str_args_t *info)
+static size_t stress_index(stress_args_t *args, stress_str_args_t *info)
 {
 	typedef char * (*test_index_t)(const char *s, int c);
 
@@ -151,22 +154,25 @@ static void stress_index(stress_str_args_t *info)
 	const size_t len1 = info->len1;
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		STRCHK(info, NULL == test_index(str1, '+'));
 		STRCHK(info, NULL != test_index(str1, str1[0]));
 
 		STRCHK(info, NULL == test_index(str2, '+'));
 		STRCHK(info, NULL != test_index(str2, str2[0]));
 	}
+	stress_bogo_add(args, 4);
+	return i * 4;
 }
 #endif
 
-#if defined(HAVE_STRINGS_H)
+#if defined(HAVE_STRINGS_H) &&	\
+    defined(HAVE_RINDEX)
 /*
  *  stress_rindex()
  *	stress on rindex
  */
-static void stress_rindex(stress_str_args_t *info)
+static size_t stress_rindex(stress_args_t *args, stress_str_args_t *info)
 {
 	typedef char * (*test_rindex_t)(const char *s, int c);
 
@@ -176,13 +182,15 @@ static void stress_rindex(stress_str_args_t *info)
 	const size_t len1 = info->len1;
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		STRCHK(info, NULL == test_rindex(str1, '+'));
 		STRCHK(info, NULL != test_rindex(str1, str1[0]));
 
 		STRCHK(info, NULL == test_rindex(str2, '+'));
 		STRCHK(info, NULL != test_rindex(str2, str2[0]));
 	}
+	stress_bogo_add(args, 4);
+	return i * 4;
 }
 #endif
 
@@ -192,7 +200,7 @@ static void stress_rindex(stress_str_args_t *info)
  *  stress_strlcpy()
  *	stress on strlcpy
  */
-static void stress_strlcpy(stress_str_args_t *info)
+static size_t stress_strlcpy(stress_args_t *args, stress_str_args_t *info)
 {
 	typedef size_t (*test_strlcpy_t)(char *dest, const char *src, size_t len);
 
@@ -206,17 +214,19 @@ static void stress_strlcpy(stress_str_args_t *info)
 	const size_t strdstlen = info->strdstlen;
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		STRCHK(info, str_len1 == test_strlcpy(strdst, str1, strdstlen));
 		STRCHK(info, str_len2 == test_strlcpy(strdst, str2, strdstlen));
 	}
+	stress_bogo_add(args, 2);
+	return i * 2;
 }
 #else
 /*
  *  stress_strcpy()
  *	stress on strcpy
  */
-static void stress_strcpy(stress_str_args_t *info)
+static size_t stress_strcpy(stress_args_t *args, stress_str_args_t *info)
 {
 	typedef char * (*test_strcpy_t)(char *dest, const char *src);
 
@@ -227,10 +237,12 @@ static void stress_strcpy(stress_str_args_t *info)
 	const size_t len1 = info->len1;
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		STRCHK(info, strdst == test_strcpy(strdst, str1));
 		STRCHK(info, strdst == test_strcpy(strdst, str2));
 	}
+	stress_bogo_add(args, 2);
+	return i * 2;
 }
 #endif
 
@@ -241,7 +253,7 @@ static void stress_strcpy(stress_str_args_t *info)
  *  stress_strlcat()
  *	stress on strlcat
  */
-static void stress_strlcat(stress_str_args_t *info)
+static size_t stress_strlcat(stress_args_t *args, stress_str_args_t *info)
 {
 	typedef size_t (*test_strlcat_t)(char *dest, const char *src, size_t len);
 
@@ -256,7 +268,7 @@ static void stress_strlcat(stress_str_args_t *info)
 	const size_t strdstlen = info->strdstlen;
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		*strdst = '\0';
 		STRCHK(info, str_len1 == test_strlcat(strdst, str1, strdstlen));
 		*strdst = '\0';
@@ -268,13 +280,15 @@ static void stress_strlcat(stress_str_args_t *info)
 		STRCHK(info, str_len2 == test_strlcat(strdst, str2, strdstlen));
 		STRCHK(info, str_len  == test_strlcat(strdst, str1, strdstlen));
 	}
+	stress_bogo_add(args, 6);
+	return i * 6;
 }
 #else
 /*
  *  stress_strcat()
  *	stress on strcat
  */
-static void stress_strcat(stress_str_args_t *info)
+static size_t stress_strcat(stress_args_t *args, stress_str_args_t *info)
 {
 	typedef char * (*test_strcat_t)(char *dest, const char *src);
 
@@ -285,7 +299,7 @@ static void stress_strcat(stress_str_args_t *info)
 	const size_t len1 = info->len1;
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		*strdst = '\0';
 		STRCHK(info, strdst == test_strcat(strdst, str1));
 		*strdst = '\0';
@@ -297,6 +311,8 @@ static void stress_strcat(stress_str_args_t *info)
 		STRCHK(info, strdst == test_strcat(strdst, str2));
 		STRCHK(info, strdst == test_strcat(strdst, str1));
 	}
+	stress_bogo_add(args, 6);
+	return i * 6;
 }
 #endif
 
@@ -304,7 +320,7 @@ static void stress_strcat(stress_str_args_t *info)
  *  stress_strncat()
  *	stress on strncat
  */
-static void stress_strncat(stress_str_args_t *info)
+static size_t stress_strncat(stress_args_t *args, stress_str_args_t *info)
 {
 	typedef char * (*test_strncat_t)(char *dest, const char *src, size_t n);
 
@@ -316,7 +332,7 @@ static void stress_strncat(stress_str_args_t *info)
 	const size_t len2 = info->len2;
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		*strdst = '\0';
 		STRCHK(info, strdst == test_strncat(strdst, str1, len1));
 		*strdst = '\0';
@@ -328,13 +344,15 @@ static void stress_strncat(stress_str_args_t *info)
 		STRCHK(info, strdst == test_strncat(strdst, str2, i));
 		STRCHK(info, strdst == test_strncat(strdst, str1, i));
 	}
+	stress_bogo_add(args, 6);
+	return i * 6;
 }
 
 /*
  *  stress_strchr()
  *	stress on strchr
  */
-static void stress_strchr(stress_str_args_t *info)
+static size_t stress_strchr(stress_args_t *args, stress_str_args_t *info)
 {
 	typedef char * (*test_strchr_t)(const char *s, int c);
 
@@ -344,20 +362,22 @@ static void stress_strchr(stress_str_args_t *info)
 	const size_t len1 = info->len1;
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		STRCHK(info, NULL == test_strchr(str1, '+'));
 		STRCHK(info, NULL != test_strchr(str1, str1[0]));
 
 		STRCHK(info, NULL == test_strchr(str2, '+'));
 		STRCHK(info, NULL != test_strchr(str2, str2[0]));
 	}
+	stress_bogo_add(args, 4);
+	return i * 4;
 }
 
 /*
  *  stress_strrchr()
  *	stress on strrchr
  */
-static void stress_strrchr(stress_str_args_t *info)
+static size_t stress_strrchr(stress_args_t *args, stress_str_args_t *info)
 {
 	typedef char * (*test_strrchr_t)(const char *s, int c);
 
@@ -367,20 +387,22 @@ static void stress_strrchr(stress_str_args_t *info)
 	const size_t len1 = info->len1;
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		STRCHK(info, NULL == test_strrchr(str1, '+'));
 		STRCHK(info, NULL != test_strrchr(str1, str1[0]));
 
 		STRCHK(info, NULL == test_strrchr(str2, '+'));
 		STRCHK(info, NULL != test_strrchr(str2, str2[0]));
 	}
+	stress_bogo_add(args, 4);
+	return i * 4;
 }
 
 /*
  *  stress_strcmp()
  *	stress on strcmp
  */
-static void stress_strcmp(stress_str_args_t *info)
+static size_t stress_strcmp(stress_args_t *args, stress_str_args_t *info)
 {
 	typedef int (*test_strcmp_t)(const char *s1, const char *s2);
 
@@ -390,7 +412,7 @@ static void stress_strcmp(stress_str_args_t *info)
 	const size_t len1 = info->len1;
 	register size_t i;
 
-	for (i = 1; keep_stressing_flag() && (i < len1); i++) {
+	for (i = 1; stress_continue_flag() && (i < len1); i++) {
 		STRCHK(info, 0 == test_strcmp(str1, str1));
 		STRCHK(info, 0 == test_strcmp(str2, str2));
 
@@ -404,13 +426,15 @@ static void stress_strcmp(stress_str_args_t *info)
 		STRCHK(info, 0 != test_strcmp(str1 + i, str2));
 		STRCHK(info, 0 != test_strcmp(str2, str1 + i));
 	}
+	stress_bogo_add(args, 9);
+	return i * 9;
 }
 
 /*
  *  stress_strncmp()
  *	stress on strncmp
  */
-static void stress_strncmp(stress_str_args_t *info)
+static size_t stress_strncmp(stress_args_t *args, stress_str_args_t *info)
 {
 	typedef int (*test_strncmp_t)(const char *s1, const char *s2, size_t n);
 
@@ -421,7 +445,7 @@ static void stress_strncmp(stress_str_args_t *info)
 	const size_t len2 = info->len2;
 	register size_t i;
 
-	for (i = 1; keep_stressing_flag() && (i < len1); i++) {
+	for (i = 1; stress_continue_flag() && (i < len1); i++) {
 		STRCHK(info, 0 == test_strncmp(str1, str1, len1));
 		STRCHK(info, 0 == test_strncmp(str2, str2, len2));
 
@@ -435,12 +459,14 @@ static void stress_strncmp(stress_str_args_t *info)
 		STRCHK(info, 0 != test_strncmp(str1 + i, str2, len2));
 		STRCHK(info, 0 != test_strncmp(str2, str1 + i, len2));
 	}
+	stress_bogo_add(args, 9);
+	return i * 9;
 }
 /*
  *  stress_strcoll()
  *	stress on strcoll
  */
-static void stress_strcoll(stress_str_args_t *info)
+static size_t stress_strcoll(stress_args_t *args, stress_str_args_t *info)
 {
 	typedef int (*test_strcoll_t)(const char *s1, const char *s2);
 
@@ -450,7 +476,7 @@ static void stress_strcoll(stress_str_args_t *info)
 	const size_t len1 = info->len1;
 	register size_t i;
 
-	for (i = 1; keep_stressing_flag() && (i < len1); i++) {
+	for (i = 1; stress_continue_flag() && (i < len1); i++) {
 		STRCHK(info, 0 == test_strcoll(str1, str1));
 		STRCHK(info, 0 == test_strcoll(str2, str2));
 
@@ -464,13 +490,15 @@ static void stress_strcoll(stress_str_args_t *info)
 		STRCHK(info, 0 != test_strcoll(str1 + i, str2));
 		STRCHK(info, 0 != test_strcoll(str2, str1 + i));
 	}
+	stress_bogo_add(args, 9);
+	return i * 9;
 }
 
 /*
  *  stress_strlen()
  *	stress on strlen
  */
-static void stress_strlen(stress_str_args_t *info)
+static size_t stress_strlen(stress_args_t *args, stress_str_args_t *info)
 {
 	typedef size_t (*test_strlen_t)(const char *s);
 
@@ -480,23 +508,28 @@ static void stress_strlen(stress_str_args_t *info)
 	const size_t len1 = info->len1;
 	const size_t len2 = info->len2;
 	register size_t i;
+	size_t n;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		STRCHK(info, len1 - 1 == test_strlen(str1));
 		STRCHK(info, len1 - 1 - i == test_strlen(str1 + i));
 	}
+	n = i * 2;
 
-	for (i = 0; keep_stressing_flag() && (i < len2 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len2 - 1); i++) {
 		STRCHK(info, len2 - 1 == test_strlen(str2));
 		STRCHK(info, len2 - 1 - i == test_strlen(str2 + i));
 	}
+	n += i * 2;
+	stress_bogo_add(args, 4);
+	return n;
 }
 
 /*
  *  stress_strxfrm()
  *	stress on strxfrm
  */
-static void stress_strxfrm(stress_str_args_t *info)
+static size_t stress_strxfrm(stress_args_t *args, stress_str_args_t *info)
 {
 	typedef size_t (*test_strxfrm_t)(char *dest, const char *src, size_t n);
 
@@ -508,7 +541,7 @@ static void stress_strxfrm(stress_str_args_t *info)
 	const size_t strdstlen = info->strdstlen;
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		*strdst = '\0';
 		STRCHK(info, 0 != test_strxfrm(strdst, str1, strdstlen));
 		*strdst = '\0';
@@ -520,26 +553,11 @@ static void stress_strxfrm(stress_str_args_t *info)
 		STRCHK(info, 0 != test_strxfrm(strdst, str2, strdstlen));
 		STRCHK(info, 0 != test_strxfrm(strdst, str1, strdstlen));
 	}
+	stress_bogo_add(args, 6);
+	return i * 6;
 }
 
-
-/*
- *  stress_str_all()
- *	iterate over all string stressors
- */
-static void stress_str_all(stress_str_args_t *info)
-{
-	static int i = 1;	/* Skip over stress_str_all */
-	stress_str_args_t info_all = *info;
-
-	info_all.libc_func = str_methods[i].libc_func;
-
-	str_methods[i].func(&info_all);
-	i++;
-	if (!str_methods[i].func)
-		i = 1;
-	info->failed = info_all.failed;
-}
+static size_t stress_str_all(stress_args_t *args, stress_str_args_t *info);
 
 /*
  * Table of string stress methods
@@ -547,9 +565,15 @@ static void stress_str_all(stress_str_args_t *info)
 static const stress_str_method_info_t str_methods[] = {
 	{ "all",		stress_str_all,		NULL },	/* Special "all test */
 
-#if defined(HAVE_STRINGS_H)
+#if defined(HAVE_STRINGS_H) &&	\
+    defined(HAVE_INDEX)
 	{ "index",		stress_index,		(void *)index },
+#endif
+#if defined(HAVE_STRINGS_H) &&	\
+    defined(HAVE_RINDEX)
 	{ "rindex",		stress_rindex,		(void *)rindex  },
+#endif
+#if defined(HAVE_STRINGS_H)
 	{ "strcasecmp",		stress_strcasecmp,	(void *)strcasecmp },
 #endif
 #if defined(HAVE_BSD_STRLCAT) &&	\
@@ -575,46 +599,48 @@ static const stress_str_method_info_t str_methods[] = {
 	{ "strncmp",		stress_strncmp,		(void *)strncmp },
 	{ "strrchr",		stress_strrchr,		(void *)strrchr },
 	{ "strxfrm",		stress_strxfrm,		(void *)strxfrm },
-	{ NULL,			NULL,			NULL }
 };
 
+static stress_metrics_t metrics[SIZEOF_ARRAY(str_methods)];
+
 /*
- *  stress_set_str_method()
- *	set the default string stress method
+ *  stress_str_all()
+ *	iterate over all string stressors
  */
-static int stress_set_str_method(const char *name)
+static size_t stress_str_all(stress_args_t *args, stress_str_args_t *info)
 {
-	stress_str_method_info_t const *info;
+	static size_t i = 1;	/* Skip over stress_str_all */
+	stress_str_args_t info_all = *info;
+	double t;
 
-	for (info = str_methods; keep_stressing_flag() && info->func; info++) {
-		if (!strcmp(info->name, name)) {
-			stress_set_setting("str-method", TYPE_ID_UINTPTR_T, &info);
-			return 0;
-		}
-	}
+	info_all.libc_func = str_methods[i].libc_func;
 
-	(void)fprintf(stderr, "str-method must be one of:");
-	for (info = str_methods; info->func; info++) {
-		(void)fprintf(stderr, " %s", info->name);
-	}
-	(void)fprintf(stderr, "\n");
-
-	return -1;
+	t = stress_time_now();
+	metrics[i].count += (double)str_methods[i].func(args, &info_all);
+	metrics[i].duration += (stress_time_now() - t);
+	i++;
+	if (UNLIKELY(i >= SIZEOF_ARRAY(str_methods)))
+		i = 1;
+	info->failed = info_all.failed;
+	return 0;
 }
 
 /*
  *  stress_str()
  *	stress CPU by doing various string operations
  */
-static int stress_str(const stress_args_t *args)
+static int stress_str(stress_args_t *args)
 {
-	const stress_str_method_info_t *str_method = &str_methods[0];
 	char ALIGN64 str1[STR1LEN], ALIGN64 str2[STR2LEN];
 	char ALIGN64 strdst[STRDSTLEN];
 	stress_str_args_t info;
+	const stress_str_method_info_t *str_method_info;
+	size_t i, j, str_method = 0;
 
 	(void)stress_get_setting("str-method", &str_method);
-	info.libc_func = str_method->libc_func;
+	str_method_info = &str_methods[str_method];
+
+	info.libc_func = str_method_info->libc_func;
 	info.str1 = str1;
 	info.len1 = sizeof(str1);
 	info.str2 = str2;
@@ -626,14 +652,22 @@ static int stress_str(const stress_args_t *args)
 
 	stress_rndstr(info.str1, info.len1);
 
+	stress_zero_metrics(metrics, SIZEOF_ARRAY(metrics));
+
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		register char *tmpptr;
 		register size_t tmplen;
+		double t;
 
 		stress_rndstr(info.str2, info.len2);
-		str_method->func(&info);
+
+		t = stress_time_now();
+		metrics[str_method].count += (double)str_method_info->func(args, &info);
+		metrics[str_method].duration += (stress_time_now() - t);
 
 		tmpptr = info.str1;
 		info.str1 = info.str2;
@@ -642,30 +676,40 @@ static int stress_str(const stress_args_t *args)
 		tmplen = info.len1;
 		info.len1 = info.len2;
 		info.len2 = tmplen;
-
-		inc_counter(args);
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
+	/* dump metrics of methods except for first "all" method */
+	for (i = 1, j = 0; i < SIZEOF_ARRAY(metrics); i++) {
+		if (metrics[i].duration > 0.0) {
+			char msg[64];
+			const double rate = metrics[i].count / metrics[i].duration;
+
+			(void)snprintf(msg, sizeof(msg), "%s calls per sec", str_methods[i].name);
+			stress_metrics_set(args, j, msg,
+				rate, STRESS_METRIC_HARMONIC_MEAN);
+			j++;
+		}
+	}
+
 	return info.failed ? EXIT_FAILURE : EXIT_SUCCESS;
 }
 
-static void stress_str_set_default(void)
+static const char *stress_str_method(const size_t i)
 {
-	stress_set_str_method("all");
+	return (i < SIZEOF_ARRAY(str_methods)) ? str_methods[i].name : NULL;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_str_method,	stress_set_str_method },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_str_method, "str-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_str_method },
+	END_OPT,
 };
 
-stressor_info_t stress_str_info = {
+const stressor_info_t stress_str_info = {
 	.stressor = stress_str,
-	.set_default = stress_str_set_default,
 	.class = CLASS_CPU | CLASS_CPU_CACHE | CLASS_MEMORY,
-	.opt_set_funcs = opt_set_funcs,
 	.verify = VERIFY_OPTIONAL,
+	.opts = opts,
 	.help = help
 };
