@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,29 +18,30 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-mounts.h"
+
+#include <sys/ioctl.h>
+#include <sys/times.h>
 
 #if defined(HAVE_LINUX_FS_H)
 #include <linux/fs.h>
-#else
-UNEXPECTED
 #endif
 
 #if defined(HAVE_SYS_SYSMACROS_H)
 #include <sys/sysmacros.h>
-#else
-UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_MKDEV_H)
+#include <sys/mkdev.h>
 #endif
 
 #if defined(HAVE_SYS_STATFS_H)
 #include <sys/statfs.h>
-#else
-UNEXPECTED
 #endif
 
 #if defined(HAVE_SYS_STATVFS_H)
 #include <sys/statvfs.h>
-#else
-UNEXPECTED
 #endif
 
 static const stress_help_t help[] = {
@@ -53,9 +54,10 @@ static const stress_help_t help[] = {
  *  stress on system information
  *	stress system by rapid fetches of system information
  */
-static int stress_sysinfo(const stress_args_t *args)
+static int stress_sysinfo(stress_args_t *args)
 {
-	int n_mounts;
+	int n_mounts, rc = EXIT_SUCCESS;
+	const int verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 	char *mnts[128];
 #if defined(HAVE_SYS_SYSINFO_H) &&	\
     defined(HAVE_SYSINFO) &&		\
@@ -63,7 +65,7 @@ static int stress_sysinfo(const stress_args_t *args)
 	const int bad_fd = stress_get_bad_fd();
 #endif
 
-	(void)memset(mnts, 0, sizeof(mnts));
+	(void)shim_memset(mnts, 0, sizeof(mnts));
 
 	n_mounts = stress_mount_get(mnts, SIZEOF_ARRAY(mnts));
 	if (n_mounts < 0) {
@@ -74,6 +76,8 @@ static int stress_sysinfo(const stress_args_t *args)
 		pr_dbg("%s: found %d mount points\n",
 			args->name, n_mounts);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -88,40 +92,44 @@ static int stress_sysinfo(const stress_args_t *args)
 			int i, ret;
 
 			ret = sysinfo(&sysinfo_buf);
-			if ((ret < 0) &&
-			    (g_opt_flags & OPT_FLAGS_VERIFY) &&
-			    (errno != EPERM))
+			if (UNLIKELY((ret < 0) && (verify) && (errno != EPERM))) {
 			 	pr_fail("%s: sysinfo failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
+				rc = EXIT_FAILURE;
+				break;
+			}
 
-			if (!keep_stressing_flag())
+			if (UNLIKELY(!stress_continue_flag()))
 				break;
 
 			/* Linux statfs variant */
 			for (i = 0; i < n_mounts; i++) {
 				int fd;
 
-				if (!keep_stressing_flag())
+				if (UNLIKELY(!stress_continue_flag()))
 					break;
 
-				if (!mnts[i])
+				if (UNLIKELY(!mnts[i]))
 					continue;
 
 				ret = statfs(mnts[i], &statfs_buf);
 				/* Mount may have been removed, so purge it */
-				if ((ret < 0) && (errno == ENOENT)) {
+				if (UNLIKELY((ret < 0) && (errno == ENOENT))) {
 					free(mnts[i]);
 					mnts[i] = NULL;
 					continue;
 				}
-				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY)) {
-					if (errno != ENOSYS &&
-					    errno != EOVERFLOW &&
-					    errno != EACCES &&
-					    errno != EPERM) {
+				if (UNLIKELY((ret < 0) && (verify))) {
+					if ((errno != ENOSYS) &&
+					    (errno != EOVERFLOW) &&
+					    (errno != EACCES) &&
+					    (errno != ENOTCONN) &&
+					    (errno != EPERM)) {
 						pr_fail("%s: statfs on %s failed: errno=%d (%s)\n",
 							args->name, mnts[i], errno,
 							strerror(errno));
+						rc = EXIT_FAILURE;
+						break;
 					}
 				}
 
@@ -132,7 +140,7 @@ static int stress_sysinfo(const stress_args_t *args)
 				VOID_RET(int, statfs("", &statfs_buf));
 
 				fd = open(mnts[i], O_RDONLY | O_DIRECTORY);
-				if (fd < 0)
+				if (UNLIKELY(fd < 0))
 					continue;
 #if defined(FS_IOC_GETFSLABEL) &&	\
     defined(FSLABEL_MAX)
@@ -146,19 +154,26 @@ static int stress_sysinfo(const stress_args_t *args)
 #endif
 
 				ret = fstatfs(fd, &statfs_buf);
-				(void)close(fd);
-				if ((ret < 0) && (errno == ENOENT))
+				if (UNLIKELY((ret < 0) && (errno == ENOENT))) {
+					(void)close(fd);
 					continue;
-				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY)) {
-					if (errno != ENOSYS &&
-					    errno != EOVERFLOW &&
-					    errno != EACCES &&
-					    errno != EPERM) {
+				}
+				if (UNLIKELY((ret < 0) && (verify))) {
+					if ((errno != ENOSYS) &&
+					    (errno != EOVERFLOW) &&
+					    (errno != EACCES) &&
+					    (errno != ENOTCONN) &&
+					    (errno != EPERM)) {
 						pr_fail("%s: fstatfs on %s failed: errno=%d (%s)\n",
 							args->name, mnts[i], errno,
 							strerror(errno));
+						rc = EXIT_FAILURE;
+						(void)close(fd);
+						break;
 					}
 				}
+				(void)close(fd);
+
 				/*
 				 *  Exercise invalid fd
 				 */
@@ -172,26 +187,29 @@ static int stress_sysinfo(const stress_args_t *args)
 			struct stat sbuf;
 			struct shim_ustat ubuf;
 
-			if (!keep_stressing_flag())
+			if (UNLIKELY(!stress_continue_flag()))
 				break;
 
 			for (i = 0; i < n_mounts; i++) {
 				if (!mnts[i])
 					continue;
 
-				ret = stat(mnts[i], &sbuf);
-				if (ret < 0)
+				ret = shim_stat(mnts[i], &sbuf);
+				if (UNLIKELY(ret < 0))
 					continue;
 
 				ret = shim_ustat(sbuf.st_dev, &ubuf);
-				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY)) {
-					if (errno != EINVAL &&
-					    errno != EFAULT &&
-					    errno != ENOSYS &&
-					    errno != EPERM) {
+				if (UNLIKELY((ret < 0) && (verify))) {
+					if ((errno != EINVAL) &&
+					    (errno != EFAULT) &&
+					    (errno != ENOSYS) &&
+					    (errno != ENOTCONN) &&
+					    (errno != EPERM)) {
 						pr_fail("%s: ustat on %s failed: errno=%d (%s)\n",
 							args->name, mnts[i], errno,
 							strerror(errno));
+						rc = EXIT_FAILURE;
+						break;
 					}
 				}
 			}
@@ -200,12 +218,16 @@ static int stress_sysinfo(const stress_args_t *args)
 			 * Exercise invalid ustat, assuming that major ~0 is
 			 * invalid
 			 */
+#if defined(__QNXNTO__)
+			sbuf.st_dev = makedev(0, ~0, stress_mwc32());
+#else
 			sbuf.st_dev = makedev(~0, stress_mwc32());
+#endif
 			VOID_RET(int, shim_ustat(sbuf.st_dev, &ubuf));
 #endif
 		}
 
-		if (!keep_stressing_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 
 #if defined(HAVE_SYS_STATVFS_H)
@@ -217,21 +239,24 @@ static int stress_sysinfo(const stress_args_t *args)
 			for (i = 0; i < n_mounts; i++) {
 				int ret;
 
-				if (!keep_stressing_flag())
+				if (UNLIKELY(!stress_continue_flag()))
 					break;
 
-				if (!mnts[i])
+				if (UNLIKELY(!mnts[i]))
 					continue;
 
 				ret = statvfs(mnts[i], &statvfs_buf);
-				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY)) {
-					if (errno != ENOSYS &&
-					    errno != EOVERFLOW &&
-					    errno != EACCES &&
-					    errno != EPERM) {
+				if (UNLIKELY((ret < 0) && (verify))) {
+					if ((errno != ENOSYS) &&
+					    (errno != EOVERFLOW) &&
+					    (errno != EACCES) &&
+					    (errno != ENOTCONN) &&
+					    (errno != EPERM)) {
 						pr_fail("%s: statvfs on %s failed: errno=%d (%s)\n",
 							args->name, mnts[i], errno,
 							strerror(errno));
+						rc = EXIT_FAILURE;
+						break;
 					}
 				}
 				/*
@@ -242,24 +267,24 @@ static int stress_sysinfo(const stress_args_t *args)
 		}
 #endif
 
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 		clk = times(&tms_buf);
-		if ((clk == (clock_t)-1) && (g_opt_flags & OPT_FLAGS_VERIFY)) {
+		if (UNLIKELY((clk == (clock_t)-1) && (verify))) {
 			 pr_fail("%s: times failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while ((rc == EXIT_SUCCESS) && stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	stress_mount_free(mnts, n_mounts);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
-stressor_info_t stress_sysinfo_info = {
+const stressor_info_t stress_sysinfo_info = {
 	.stressor = stress_sysinfo,
 	.class = CLASS_OS,
 	.verify = VERIFY_OPTIONAL,
