@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-killpid.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"tee N",	"start N workers exercising the tee system call" },
@@ -28,13 +29,16 @@ static const stress_help_t help[] = {
 #if defined(HAVE_TEE) &&	\
     defined(SPLICE_F_NONBLOCK)
 
-#define TEE_IO_SIZE	(65536)
+typedef struct {
+	uint64_t	length;
+	uint64_t	counter;
+} stress_tee_t;
 
 static void stress_sigpipe_handler(int signum)
 {
 	(void)signum;
 
-	keep_stressing_set_flag(false);
+	stress_continue_set_flag(false);
 }
 
 /*
@@ -42,13 +46,13 @@ static void stress_sigpipe_handler(int signum)
  *	spawn off tee I/O processes
  */
 static pid_t stress_tee_spawn(
-	const stress_args_t *args,
-	void (*func)(const stress_args_t *args, int fds[2]),
+	stress_args_t *args,
+	void (*func)(stress_args_t *args, int fds[2]),
 	int fds[2])
 {
 	pid_t pid;
 
-	if (pipe(fds) < 0) {
+	if (UNLIKELY(pipe(fds) < 0)) {
 		pr_err("%s: pipe failed: %d (%s)\n",
 			args->name, errno, strerror(errno));
 		return -1;
@@ -57,17 +61,16 @@ static pid_t stress_tee_spawn(
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 		(void)close(fds[0]);
 		(void)close(fds[1]);
-		if (!keep_stressing(args))
+		if (UNLIKELY(!stress_continue(args)))
 			return -1;
 		pr_err("%s: fork failed: %d (%s)\n",
 			args->name, errno, strerror(errno));
 		return -1;
-	}
-	if (pid == 0) {
+	} else if (pid == 0) {
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
@@ -81,26 +84,34 @@ again:
  *  stress_tee_pipe_write()
  *	write data down a pipe
  */
-static void stress_tee_pipe_write(const stress_args_t *args, int fds[2])
+static void stress_tee_pipe_write(stress_args_t *args, int fds[2])
 {
-	static uint64_t ALIGN64 buffer[TEE_IO_SIZE / sizeof(uint64_t)];
-	uint64_t counter = 0;
+	static stress_tee_t ALIGN64 data;
+
+	data.length = sizeof(data);
+	data.counter = 0;
 
 	(void)args;
 	(void)close(fds[0]);
 
-	(void)memset(buffer, 0, sizeof(buffer));
-	while (keep_stressing_flag()) {
+	while (stress_continue_flag()) {
 		ssize_t ret;
 
-		buffer[0] = sizeof(buffer);
-		buffer[1] = counter++;
-
-		ret = write(fds[1], buffer, sizeof(buffer));
-		if (ret < 0) {
-			if (errno != EAGAIN)
+		ret = write(fds[1], &data, sizeof(data));
+		if (UNLIKELY(ret < 0)) {
+			switch (errno) {
+			case EPIPE:
 				break;
+			case EINTR:
+			case EAGAIN:
+				continue;
+			default:
+				pr_fail("%s: unexpected write error, errno=%d (%s)\n",
+					args->name, errno, strerror(errno));
+				break;
+			}
 		}
+		data.counter++;
 	}
 	(void)close(fds[1]);
 }
@@ -109,35 +120,47 @@ static void stress_tee_pipe_write(const stress_args_t *args, int fds[2])
  *  stress_tee_pipe_read()
  *	read data from a pipe
  */
-static void stress_tee_pipe_read(const stress_args_t *args, int fds[2])
+static void stress_tee_pipe_read(stress_args_t *args, int fds[2])
 {
-	static uint64_t ALIGN64 buffer[TEE_IO_SIZE / sizeof(uint64_t)];
-	uint64_t count = 0;
+	static stress_tee_t ALIGN64 data;
+	register uint64_t counter = 0;
+
+	data.length = sizeof(data);
 
 	(void)close(fds[1]);
 
-	while (keep_stressing_flag()) {
+	while (stress_continue_flag()) {
+		register size_t n = 0;
 		ssize_t ret;
-		size_t n = 0;
 
-		while (n < sizeof(buffer)) {
-			ret = read(fds[0], buffer, sizeof(buffer));
-			if (ret < 0) {
-				if (errno != EAGAIN)
-					break;
+		while (n < sizeof(data)) {
+			ret = read(fds[0], &data, sizeof(data));
+			if (UNLIKELY(ret < 0)) {
+				switch (errno) {
+				case EPIPE:
+					return;
+				case EAGAIN:
+				case EINTR:
+					continue;
+				default:
+					pr_fail("%s: unexpected read error, errno=%d (%s)\n",
+						args->name, errno, strerror(errno));
+					return;
+				}
 			} else {
 				n += (size_t)ret;
 			}
 		}
-		if (buffer[0] != sizeof(buffer)) {
-			pr_fail("%s: pipe read, wrong size detected\n", args->name);
+		if (UNLIKELY(data.length != sizeof(data))) {
+			pr_fail("%s: pipe read of %zu bytes, wrong size detected, got %" PRIu64
+				", expected %zu\n", args->name,
+				n, data.length, sizeof(data));
 		}
-		if (buffer[1] != count) {
+		if (UNLIKELY(data.counter != counter)) {
 			pr_fail("%s: pipe read, wrong check value detected\n", args->name);
 		}
-		count++;
+		counter++;
 	}
-	(void)close(fds[1]);
 }
 
 /*
@@ -145,7 +168,7 @@ static void stress_tee_pipe_read(const stress_args_t *args, int fds[2])
  *	exercise the tee syscall in most possible ways
  */
 static int exercise_tee(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int release,
 	const int fd_in,
 	const int fd_out)
@@ -162,7 +185,7 @@ static int exercise_tee(
 		 *  as it is throwing errors for pre-4.10 kernels
 		 */
 		ret = tee(fd_in, fd_out, INT_MAX, ~0U);
-		if (ret >= 0) {
+		if (UNLIKELY(ret >= 0)) {
 			pr_fail("%s: tee with illegal flags "
 				"unexpectedly succeeded\n",
 				args->name);
@@ -172,7 +195,7 @@ static int exercise_tee(
 
 	/* Exercise on same pipe */
 	ret = tee(fd_in, fd_in, INT_MAX, 0);
-	if (ret >= 0) {
+	if (UNLIKELY(ret >= 0)) {
 		pr_fail("%s: tee on same fd_out and fd_in "
 			"unexpectedly succeeded\n",
 			args->name);
@@ -184,7 +207,7 @@ static int exercise_tee(
 	 * no difference other than increase in kernel test coverage
 	 */
 	ret = tee(fd_in, fd_out, 0, 0);
-	if (ret < 0) {
+	if (UNLIKELY(ret < 0)) {
 		pr_fail("%s: tee with 0 len argument "
 			"unexpectedly failed\n",
 			args->name);
@@ -198,13 +221,15 @@ static int exercise_tee(
  *  stress_tee()
  *	stress the Linux tee syscall
  */
-static int stress_tee(const stress_args_t *args)
+static int stress_tee(stress_args_t *args)
 {
 	ssize_t len, slen;
 	int fd, pipe_in[2], pipe_out[2];
 	pid_t pids[2];
-	int ret = EXIT_FAILURE, status;
+	int ret = EXIT_FAILURE;
 	const int release = stress_get_kernel_release();
+	int metrics_count = 0;
+	double duration = 0.0, bytes = 0.0, rate;
 
 	if (stress_sighandler(args->name, SIGPIPE, stress_sigpipe_handler, NULL) < 0)
 		return EXIT_NO_RESOURCE;
@@ -216,30 +241,51 @@ static int stress_tee(const stress_args_t *args)
 		return EXIT_NO_RESOURCE;
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	pids[0] = stress_tee_spawn(args, stress_tee_pipe_write, pipe_in);
-	if (pids[0] < 0) {
-		(void)close(fd);
-		return EXIT_FAILURE;
-	}
+	if (pids[0] < 0)
+		goto tidy_fd;
 	(void)close(pipe_in[1]);
 
 	pids[1] = stress_tee_spawn(args, stress_tee_pipe_read, pipe_out);
-	if (pids[0] < 0)
+	if (pids[1] < 0)
 		goto tidy_child1;
 	(void)close(pipe_out[0]);
 
 	do {
-		len = tee(pipe_in[0], pipe_out[1],
-			INT_MAX, 0 & SPLICE_F_NONBLOCK);
+		if (LIKELY(metrics_count++ < 1000)) {
+			len = tee(pipe_in[0], pipe_out[1], INT_MAX, 0);
+			if (LIKELY(len > 0))
+				goto do_splice;
 
-		if (len < 0) {
-			if (errno == EAGAIN)
+		} else {
+			double t;
+
+			metrics_count = 0;
+			t = stress_time_now();
+			len = tee(pipe_in[0], pipe_out[1], INT_MAX, 0);
+			if (LIKELY(len > 0)) {
+				duration += stress_time_now() - t;
+				bytes += (double)len;
+				goto do_splice;
+			}
+		}
+		if (UNLIKELY(len == 0))
+			break;
+
+		if (UNLIKELY(len < 0)) {
+			switch (errno) {
+			case EPIPE:
+			case EAGAIN:
 				continue;
-			if (errno == EINTR)
+			case EINTR:
+				ret = EXIT_SUCCESS;
+				goto tidy_child2;
 				break;
-			if (errno == ENOMEM) {
+			case ENOMEM:
 				pr_inf_skip("%s: skipping stressor, out of memory\n",
 					args->name);
 				ret = EXIT_NO_RESOURCE;
@@ -248,17 +294,15 @@ static int stress_tee(const stress_args_t *args)
 			pr_fail("%s: tee failed: errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			goto tidy_child2;
-		} else {
-			if (len == 0)
-				break;
 		}
 
+do_splice:
 		while (len > 0) {
 			slen = splice(pipe_in[0], NULL, fd, NULL,
 				(size_t)len, SPLICE_F_MOVE);
-			if (errno == EINTR)
+			if (UNLIKELY(errno == EINTR))
 				break;
-			if (slen < 0) {
+			if (UNLIKELY(slen < 0)) {
 				pr_err("%s: splice failed: errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				goto tidy_child2;
@@ -266,41 +310,45 @@ static int stress_tee(const stress_args_t *args)
 			len -= slen;
 		}
 
-		if (exercise_tee(args, release, pipe_in[0], pipe_out[1]) < 0)
+		if (UNLIKELY(exercise_tee(args, release, pipe_in[0], pipe_out[1]) < 0))
 			goto tidy_child2;
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	ret = EXIT_SUCCESS;
 
+	rate = (duration > 0.0) ? bytes / duration : 0.0;
+	stress_metrics_set(args, 0, "MB per sec tee rate",
+		rate / (double)MB, STRESS_METRIC_HARMONIC_MEAN);
+
 tidy_child2:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)close(pipe_out[1]);
-	(void)kill(pids[1], SIGKILL);
-	(void)shim_waitpid(pids[1], &status, 0);
+	(void)stress_kill_pid_wait(pids[1], NULL);
 
 tidy_child1:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)close(pipe_in[0]);
-	(void)kill(pids[0], SIGKILL);
-	(void)shim_waitpid(pids[0], &status, 0);
-
+	(void)stress_kill_pid_wait(pids[0], NULL);
+tidy_fd:
 	(void)close(fd);
 
 	return ret;
 }
 
-stressor_info_t stress_tee_info = {
+const stressor_info_t stress_tee_info = {
 	.stressor = stress_tee,
 	.class = CLASS_PIPE_IO | CLASS_OS | CLASS_SCHEDULER,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_tee_info = {
-	.stressor = stress_not_implemented,
+const stressor_info_t stress_tee_info = {
+	.stressor = stress_unimplemented,
 	.class = CLASS_PIPE_IO | CLASS_OS | CLASS_SCHEDULER,
-	.help = help
+	.verify = VERIFY_ALWAYS,
+	.help = help,
+	.unimplemented_reason = "built without tee() system call or undefined SPLICE_F_NONBLOCK"
 };
 #endif
