@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,10 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
+
+#include <sched.h>
 
 #if defined(HAVE_SYS_UIO_H)
 #include <sys/uio.h>
@@ -45,7 +49,7 @@ static const stress_help_t help[] = {
 #define CHUNK_SIZE	(1 * GB)
 
 typedef struct {
-	const stress_args_t *args;
+	stress_args_t *args;
 	size_t sz;
 	size_t iov_count;
 	pid_t pid;
@@ -60,19 +64,9 @@ typedef struct {
 
 #endif
 
-static int stress_set_vm_rw_bytes(const char *opt)
-{
-	size_t vm_rw_bytes;
-
-	vm_rw_bytes = (size_t)stress_get_uint64_byte_memory(opt, 1);
-	stress_check_range_bytes("vm-rw-bytes", vm_rw_bytes,
-		MIN_VM_RW_BYTES, MAX_VM_RW_BYTES);
-	return stress_set_setting("vm-rw-bytes", TYPE_ID_SIZE_T, &vm_rw_bytes);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_vm_rw_bytes,	stress_set_vm_rw_bytes },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_vm_rw_bytes, "vm-rw-bytes", TYPE_ID_SIZE_T_BYTES_VM, MIN_VM_RW_BYTES, MAX_VM_RW_BYTES, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_PROCESS_VM_READV) &&	\
@@ -80,14 +74,15 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
     defined(HAVE_CLONE) &&		\
     defined(CLONE_VM)
 
-static int stress_vm_child(void *arg)
+static int OPTIMIZE3 stress_vm_child(void *arg)
 {
 	const stress_context_t *ctxt = (stress_context_t *)arg;
-	const stress_args_t *args = ctxt->args;
+	stress_args_t *args = ctxt->args;
+	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 
 	uint8_t *buf;
-	int ret = EXIT_SUCCESS;
-	stress_addr_msg_t msg_rd, msg_wr;
+	int rc = EXIT_SUCCESS;
+	stress_addr_msg_t msg_rd ALIGN64, msg_wr ALIGN64;
 
 	stress_parent_died_alarm();
 
@@ -98,55 +93,62 @@ static int stress_vm_child(void *arg)
 	buf = mmap(NULL, ctxt->sz, PROT_READ | PROT_WRITE,
 		MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (buf == MAP_FAILED) {
-		ret = stress_exit_status(errno);
+		rc = stress_exit_status(errno);
 		pr_fail("%s: mmap failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto cleanup;
 	}
+	stress_set_vma_anon_name(buf, ctxt->sz, "context");
 
-	while (keep_stressing_flag()) {
-		uint8_t *ptr, *end = buf + ctxt->sz;
+	while (stress_continue_flag()) {
+		register uint8_t *ptr;
+		register const uint8_t *end = buf + ctxt->sz;
 		ssize_t rwret;
 
-		(void)memset(&msg_wr, 0, sizeof(msg_wr));
+		(void)shim_memset(&msg_wr, 0, sizeof(msg_wr));
 		msg_wr.addr = buf;
 		msg_wr.val = 0;
 
 		/* Send address of buffer to parent */
 redo_wr1:
 		rwret = write(ctxt->pipe_wr[1], &msg_wr, sizeof(msg_wr));
-		if (rwret < 0) {
+		if (UNLIKELY(rwret < 0)) {
 			if ((errno == EAGAIN) || (errno == EINTR))
 				goto redo_wr1;
-			if (errno != EBADF)
+			if (errno != EBADF) {
 				pr_fail("%s: write failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
+				rc = EXIT_FAILURE;
+			}
 			break;
 		}
 redo_rd1:
 		/* Wait for parent to populate data */
 		rwret = read(ctxt->pipe_rd[0], &msg_rd, sizeof(msg_rd));
-		if (rwret < 0) {
+		if (UNLIKELY(rwret < 0)) {
 			if ((errno == EAGAIN) || (errno == EINTR))
 				goto redo_rd1;
 			pr_fail("%s: read failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
+			rc = EXIT_FAILURE;
 			break;
 		}
-		if (rwret == 0)
-			break;
-		if (rwret != sizeof(msg_rd)) {
+		if (UNLIKELY(rwret != sizeof(msg_rd))) {
+			if (rwret == 0)
+				break;
 			pr_fail("%s: read failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
+			rc = EXIT_FAILURE;
 			break;
 		}
 
-		if (g_opt_flags & OPT_FLAGS_VERIFY) {
+		if (UNLIKELY(verify)) {
 			/* Check memory altered by parent is sane */
 			for (ptr = buf; ptr < end; ptr += args->page_size) {
-				if (*ptr != msg_rd.val) {
+				if (UNLIKELY(*ptr != msg_rd.val)) {
 					pr_fail("%s: memory at %p (offset %tx): %d vs %d\n",
 						args->name, (void *)ptr, ptr - buf, *ptr, msg_rd.val);
+					rc = EXIT_FAILURE;
 					goto cleanup;
 				}
 				*ptr = 0;
@@ -157,7 +159,7 @@ cleanup:
 	/* Tell parent we're done */
 	msg_wr.addr = 0;
 	msg_wr.val = 0;
-	if (write(ctxt->pipe_wr[1], &msg_wr, sizeof(msg_wr)) <= 0) {
+	if (UNLIKELY(write(ctxt->pipe_wr[1], &msg_wr, sizeof(msg_wr)) <= 0)) {
 		if (errno != EBADF)
 			pr_dbg("%s: failed to write termination message "
 				"over pipe: errno=%d (%s)\n",
@@ -166,20 +168,20 @@ cleanup:
 
 	(void)close(ctxt->pipe_wr[1]);
 	(void)close(ctxt->pipe_rd[0]);
-	(void)munmap(buf, ctxt->sz);
-	return ret;
+	(void)munmap((void *)buf, ctxt->sz);
+	return rc;
 }
 
 
-static int stress_vm_parent(stress_context_t *ctxt)
+static int OPTIMIZE3 stress_vm_parent(stress_context_t *ctxt)
 {
 	/* Parent */
-	int status;
 	uint8_t val = 0x10;
 	uint8_t *localbuf;
 	stress_addr_msg_t msg_rd, msg_wr;
-	const stress_args_t *args = ctxt->args;
+	stress_args_t *args = ctxt->args;
 	size_t sz;
+	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 
 	localbuf = mmap(NULL, ctxt->sz, PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
@@ -198,32 +200,33 @@ static int stress_vm_parent(stress_context_t *ctxt)
 	(void)close(ctxt->pipe_rd[0]);
 
 	do {
-		struct iovec local[1], remote[1];
-		uint8_t *ptr1, *ptr2, *end = localbuf + ctxt->sz;
+		struct iovec local[1] ALIGN64, remote[1] ALIGN64;
+		uint8_t *ptr1, *ptr2;
+		const uint8_t *end = localbuf + ctxt->sz;
 		ssize_t rwret;
 		size_t i, len;
 
 		/* Wait for address of child's buffer */
 redo_rd2:
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 		rwret = read(ctxt->pipe_wr[0], &msg_rd, sizeof(msg_rd));
-		if (rwret < 0) {
+		if (UNLIKELY(rwret < 0)) {
 			if ((errno == EAGAIN) || (errno == EINTR))
 				goto redo_rd2;
 			pr_fail("%s: read failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			break;
 		}
-		if (rwret == 0)
-			break;
-		if (rwret != sizeof(msg_rd)) {
+		if (UNLIKELY(rwret != sizeof(msg_rd))) {
+			if (rwret == 0)
+				break;
 			pr_fail("%s: read failed, expected %zd bytes, got %zd\n",
 				args->name, sizeof(msg_rd), rwret);
 			break;
 		}
 		/* Child telling us it's terminating? */
-		if (!msg_rd.addr)
+		if (UNLIKELY(!msg_rd.addr))
 			break;
 
 		/* Perform read from child's memory */
@@ -241,17 +244,17 @@ redo_rd2:
 			ptr2 += len;
 			sz -= len;
 
-			if (process_vm_readv(ctxt->pid, local, 1, remote, 1, 0) < 0) {
+			if (UNLIKELY(process_vm_readv(ctxt->pid, local, 1, remote, 1, 0) < 0)) {
 				pr_fail("%s: process_vm_readv failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				goto fail;
 			}
 		}
 
-		if (g_opt_flags & OPT_FLAGS_VERIFY) {
+		if (UNLIKELY(verify)) {
 			/* Check data is sane */
 			for (ptr1 = localbuf; ptr1 < end; ptr1 += args->page_size) {
-				if (*ptr1) {
+				if (UNLIKELY(*ptr1)) {
 					pr_fail("%s: memory at %p (offset %tx): %d vs %d\n",
 						args->name, (void *)ptr1, ptr1 - localbuf, *ptr1, msg_rd.val);
 					goto fail;
@@ -293,7 +296,7 @@ redo_rd2:
 			remote[0].iov_len = len;
 			ptr2 += len;
 			sz -= len;
-			if (process_vm_writev(ctxt->pid, local, 1, remote, 1, 0) < 0) {
+			if (UNLIKELY(process_vm_writev(ctxt->pid, local, 1, remote, 1, 0) < 0)) {
 				pr_fail("%s: process_vm_writev failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				goto fail;
@@ -302,11 +305,11 @@ redo_rd2:
 		msg_wr.val = val;
 		val++;
 redo_wr2:
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 		/* Inform child that memory has been changed */
 		rwret = write(ctxt->pipe_rd[1], &msg_wr, sizeof(msg_wr));
-		if (rwret < 0) {
+		if (UNLIKELY(rwret < 0)) {
 			if ((errno == EAGAIN) || (errno == EINTR))
 				goto redo_wr2;
 			if (errno != EBADF)
@@ -330,8 +333,8 @@ redo_wr2:
 		remote[0].iov_len = len;
 		(void)process_vm_writev(~0, local, 1, remote, 1, 0);
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 fail:
 	/* Tell child we're done */
 	msg_wr.addr = NULL;
@@ -345,9 +348,9 @@ fail:
 	}
 	(void)close(ctxt->pipe_wr[0]);
 	(void)close(ctxt->pipe_rd[1]);
-	(void)stress_killpid(ctxt->pid);
-	(void)shim_waitpid(ctxt->pid, &status, 0);
-	(void)munmap(localbuf, ctxt->sz);
+	if (ctxt->pid > 1)
+		stress_kill_and_wait(args, ctxt->pid, SIGALRM, false);
+	(void)munmap((void *)localbuf, ctxt->sz);
 
 	return EXIT_SUCCESS;
 }
@@ -356,7 +359,7 @@ fail:
  *  stress_vm_rw
  *	stress vm_read_v/vm_write_v
  */
-static int stress_vm_rw(const stress_args_t *args)
+static int stress_vm_rw(stress_args_t *args)
 {
 	stress_context_t ctxt;
 	uint8_t stack[64*1024];
@@ -392,12 +395,16 @@ static int stress_vm_rw(const stress_args_t *args)
 		return EXIT_NO_RESOURCE;
 	}
 
+	(void)shim_memset(stack, 0, sizeof(stack));
+
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
 	ctxt.pid = clone(stress_vm_child, stress_align_stack(stack_top),
 		SIGCHLD | CLONE_VM, &ctxt);
 	if (ctxt.pid < 0) {
-		if (keep_stressing_flag() && (errno == EAGAIN))
+		if (stress_continue_flag() && (errno == EAGAIN))
 			goto again;
 		pr_fail("%s: clone failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -414,18 +421,19 @@ again:
 	return rc;
 }
 
-stressor_info_t stress_vm_rw_info = {
+const stressor_info_t stress_vm_rw_info = {
 	.stressor = stress_vm_rw,
 	.class = CLASS_VM | CLASS_MEMORY | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
-stressor_info_t stress_vm_rw_info = {
+const stressor_info_t stress_vm_rw_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_VM | CLASS_MEMORY | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
+	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "built without process_vm_readv(), process_vm_writev(), clone() or CLONE_VM defined"
 };
